@@ -1,4 +1,4 @@
-// $Id: CommandLine.cpp,v 1.56 2005/03/01 17:47:54 geuzaine Exp $
+// $Id: CommandLine.cpp,v 1.66 2005/10/24 15:38:13 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -44,9 +44,6 @@
 extern Context_T CTX;
 extern Mesh *THEM;
 
-char *TheFileNameTab[MAX_OPEN_FILES];
-char *TheBgmFileName = NULL;
-
 char gmsh_progname[]  = "Gmsh, a 3D mesh generator with pre- and post-processing facilities" ;
 char gmsh_copyright[] = "Copyright (C) 1997-2005 Christophe Geuzaine and Jean-Francois Remacle";
 char gmsh_version[]   = "Version        : " ;
@@ -72,16 +69,17 @@ void Print_Usage(char *name){
   Msg(DIRECT, "  -1, -2, -3            Perform batch 1D, 2D and 3D mesh generation");
   Msg(DIRECT, "  -saveall              Save all elements (discard physical group definitions)");
   Msg(DIRECT, "  -o file               Specify mesh output file name");
-  Msg(DIRECT, "  -format string        Set output mesh format (msh, unv, gref, p3d)");
-  Msg(DIRECT, "  -algo string          Select mesh algorithm (iso, tri, aniso, netgen)");
+  Msg(DIRECT, "  -format string        Set output mesh format (msh, unv, gref, stl, p3d)");
+  Msg(DIRECT, "  -algo string          Select mesh algorithm (iso, tri, aniso, netgen, tetgen)");
   Msg(DIRECT, "  -smooth int           Set number of mesh smoothing steps");
   Msg(DIRECT, "  -optimize             Optimize quality of tetrahedral elements");
+  Msg(DIRECT, "  -partition int        Load a mesh and partition it into $1 parts (METIS is required)");
   Msg(DIRECT, "  -order int            Set mesh order (1, 2)");
   Msg(DIRECT, "  -scale float          Set global scaling factor");
   Msg(DIRECT, "  -meshscale float      Set mesh scaling factor");
   Msg(DIRECT, "  -clscale float        Set characteristic length scaling factor");
   Msg(DIRECT, "  -rand float           Set random perturbation factor");
-  Msg(DIRECT, "  -bgm file             Load backround mesh from file");
+  Msg(DIRECT, "  -bgm file             Load background mesh from file");
   Msg(DIRECT, "  -constrain            Constrain background mesh with characteristic lengths");
   Msg(DIRECT, "  -histogram            Print mesh quality histogram");
   Msg(DIRECT, "  -extrude              Use old extrusion mesh generator");
@@ -91,7 +89,7 @@ void Print_Usage(char *name){
   Msg(DIRECT, "Post-processing options:");
   Msg(DIRECT, "  -noview               Hide all views on startup");
   Msg(DIRECT, "  -link int             Select link mode between views (0, 1, 2, 3, 4)");
-  Msg(DIRECT, "  -combine              Combine input views into multi-time-step ones");
+  Msg(DIRECT, "  -combine              Combine views having identical names into multi-time-step views");
   Msg(DIRECT, "Display options:");    
   Msg(DIRECT, "  -nodb                 Disable double buffering");
   Msg(DIRECT, "  -fontsize int         Specify the font size for the GUI");
@@ -103,6 +101,7 @@ void Print_Usage(char *name){
   Msg(DIRECT, "  -a, -g, -m, -s, -p    Start in automatic, geometry, mesh, solver or post-processing mode");
 #endif
   Msg(DIRECT, "  -pid                  Print process id on stdout");
+  Msg(DIRECT, "  -listen               Always listen to external solver requests");
   Msg(DIRECT, "  -v int                Set verbosity level");
   Msg(DIRECT, "  -nopopup              Don't popup dialog windows in scripts");
   Msg(DIRECT, "  -string \"string\"      Parse option string at startup");
@@ -128,6 +127,9 @@ char *Get_BuildOptions(void)
 #if defined(HAVE_NETGEN)
     strcat(opt, "NETGEN ");
 #endif
+#if defined(HAVE_TETGEN)
+    strcat(opt, "TETGEN ");
+#endif
 #if defined(HAVE_LIBJPEG)
     strcat(opt, "JPEG ");
 #endif
@@ -139,31 +141,26 @@ char *Get_BuildOptions(void)
 #endif
 #if defined(HAVE_MATH_EVAL)
     strcat(opt, "MATHEVAL ");
+#endif
+#if defined(HAVE_METIS)
+    strcat(opt, "METIS ");
+#endif
+#if defined(HAVE_ANN_)
+    strcat(opt, "ANN ");
 #endif
     first = 0;
   }
   return opt;
 }
 
-void Get_Options(int argc, char *argv[], int *nbfiles)
+void Get_Options(int argc, char *argv[])
 {
-  int i = 1;
-
-  // This symbol context is local to option parsing (the symbols will
-  // not interfere with subsequent OpenFiles)
-
-  InitSymbols();
-
   // Parse session and option files
-
   ParseFile(CTX.session_filename_fullpath, 1, 1);
   ParseFile(CTX.options_filename_fullpath, 1, 1);
 
   // Get command line options
-
-  TheFileNameTab[0] = CTX.default_filename_fullpath;
-  *nbfiles = 0;
-
+  int i = 1;
   while(i < argc) {
 
     if(argv[i][0] == '-') {
@@ -267,7 +264,7 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
       else if(!strcmp(argv[i] + 1, "bgm")) {
         i++;
         if(argv[i] != NULL)
-          TheBgmFileName = argv[i++];
+          CTX.bgm_filename = argv[i++];
         else {
           fprintf(stderr, ERROR_STR "Missing file name\n");
           exit(1);
@@ -299,10 +296,6 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
           fprintf(stderr, "Usage: %s -convert file file\n", argv[0]);
         exit(1);
       }
-      else if(!strcmp(argv[i] + 1, "old")) {
-        CTX.geom.old_circle = 1;
-        i++;
-      }
       else if(!strcmp(argv[i] + 1, "initial")) {
         i++;
         if(argv[i] != NULL)
@@ -363,6 +356,16 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
           exit(1);
         }
       }
+      else if(!strcmp(argv[i] + 1, "partition")) {
+        i++;
+        if(argv[i] != NULL) {
+          CTX.mesh.nbPartitions = atoi(argv[i++]);
+        }
+        else {
+          fprintf(stderr, ERROR_STR "Missing number\n");
+          exit(1);
+        }
+      }
       else if(!strcmp(argv[i] + 1, "smooth")) {
         i++;
         if(argv[i] != NULL)
@@ -396,6 +399,9 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
                   !strcmp(argv[i], "GREF") || !strcmp(argv[i], "Gref")) {
             CTX.mesh.format = FORMAT_GREF;
           }
+          else if(!strcmp(argv[i], "stl") || !strcmp(argv[i], "STL")) {
+            CTX.mesh.format = FORMAT_STL;
+          }
           else if(!strcmp(argv[i], "p3d") ||
                   !strcmp(argv[i], "P3D") || !strcmp(argv[i], "Plot3D")) {
             CTX.mesh.format = FORMAT_P3D;
@@ -422,6 +428,8 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
             CTX.mesh.algo2d = DELAUNAY_ANISO;
           else if(!strncmp(argv[i], "netgen", 6))
             CTX.mesh.algo3d = FRONTAL_NETGEN;
+          else if(!strncmp(argv[i], "tetgen", 6))
+            CTX.mesh.algo3d = DELAUNAY_TETGEN;
           else {
             fprintf(stderr, ERROR_STR "Unknown mesh algorithm\n");
             exit(1);
@@ -433,8 +441,11 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
           exit(1);
         }
       }
-      else if(!strcmp(argv[i] + 1, "version") ||
-              !strcmp(argv[i] + 1, "-version")) {
+      else if(!strcmp(argv[i] + 1, "listen")) {
+        CTX.solver.listen = 1;
+        i++;
+      }
+      else if(!strcmp(argv[i] + 1, "version") || !strcmp(argv[i] + 1, "-version")) {
         fprintf(stderr, "%s\n", GMSH_VERSION);
         exit(1);
       }
@@ -498,10 +509,6 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
           exit(1);
         }
       }
-      else if(!strcmp(argv[i] + 1, "fill")) {
-        opt_view_intervals_type(0, GMSH_SET, DRAW_POST_CONTINUOUS);
-        i++;
-      }
       else if(!strcmp(argv[i] + 1, "smoothview")) {
         CTX.post.smooth = 1;
         i++;
@@ -510,27 +517,6 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
         CTX.post.combine_time = 1;
         i++;
       }
-      else if(!strcmp(argv[i] + 1, "nbiso")) {
-        i++;
-        if(argv[i] != NULL)
-          opt_view_nb_iso(0, GMSH_SET, atoi(argv[i++]));
-        else {
-          fprintf(stderr, ERROR_STR "Missing number\n");
-          exit(1);
-        }
-      }
-      else if(!strcmp(argv[i] + 1, "threads")) {
-        CTX.threads = 1;
-        i++;
-      }
-      else if(!strcmp(argv[i] + 1, "nothreads")) {
-        CTX.threads = 0;
-        i++;
-      }
-      else if(!strcmp(argv[i] + 1, "db")) {
-        CTX.db = 1;
-        i++;
-      }
       else if(!strcmp(argv[i] + 1, "nodb")) {
         CTX.db = 0;
         i++;
@@ -583,18 +569,16 @@ void Get_Options(int argc, char *argv[], int *nbfiles)
         Print_Usage(argv[0]);
         exit(1);
       }
-    }
 
+    }
     else {
-      if(*nbfiles < MAX_OPEN_FILES)
-        TheFileNameTab[(*nbfiles)++] = argv[i++];
-      else {
-        fprintf(stderr, ERROR_STR "Too many input files\n");
-        exit(1);
-      }
+      List_Add(CTX.files, &argv[i++]);
     }
 
   }
 
-  strncpy(CTX.filename, TheFileNameTab[0], 255);
+  if(!List_Nbr(CTX.files))
+    strncpy(CTX.filename, CTX.default_filename_fullpath, 255);
+  else
+    strncpy(CTX.filename, *(char**)List_Pointer(CTX.files, 0), 255);
 }
