@@ -164,7 +164,7 @@ StringXString SolverOptions_String[] = {
     "Command associated with the second button for solver 0" },
   { F|O, "ThirdButton0" , opt_solver_third_button0 , "Pos" ,
     "Label of third button for solver 0" },
-  { F|O, "ThirdButtonCommand0" , opt_solver_third_button_command0 , "-bin -pos %s" ,
+  { F|O, "ThirdButtonCommand0" , opt_solver_third_button_command0 , "-pos %s" ,
     "Command associated with the third button for solver 0" },
   { F|O, "FourthButton0" , opt_solver_fourth_button0 , "" ,
     "Label of fourth button for solver 0" },
@@ -407,6 +407,27 @@ StringXString ViewOptions_String[] = {
   { F,   "Name" , opt_view_name , "" , 
     "Default post-processing view name" },
 
+  { F|O, "Stipple0" , opt_view_stipple0 , "1*0x1F1F" , 
+    "First stippling pattern" },
+  { F|O, "Stipple1" , opt_view_stipple1 , "1*0x3333" , 
+    "Second stippling pattern" },
+  { F|O, "Stipple2" , opt_view_stipple2 , "1*0x087F" , 
+    "Third stippling pattern" },
+  { F|O, "Stipple3" , opt_view_stipple3 , "1*0xCCCF" , 
+    "Fourth stippling pattern" },
+  { F|O, "Stipple4" , opt_view_stipple4 , "2*0x1111" , 
+    "Fifth stippling pattern" },
+  { F|O, "Stipple5" , opt_view_stipple5 , "2*0x0F0F" , 
+    "Sixth stippling pattern" },
+  { F|O, "Stipple6" , opt_view_stipple6 , "1*0xCFFF" , 
+    "Seventh stippling pattern" },
+  { F|O, "Stipple7" , opt_view_stipple7 , "2*0x0202" , 
+    "Eighth stippling pattern" },
+  { F|O, "Stipple8" , opt_view_stipple8 , "2*0x087F" , 
+    "Ninth stippling pattern" },
+  { F|O, "Stipple9" , opt_view_stipple9 , "1*0xFFFF" , 
+    "Tenth stippling pattern" },
+
   { 0, NULL , NULL , NULL , NULL }
 } ;
 
@@ -527,7 +548,7 @@ StringXNumber GeneralOptions_Number[] = {
   { F|O, "DrawBoundingBoxes" , opt_general_draw_bounding_box, 0. ,
     "Draw bounding boxes" },
 
-  { F|O, "FastRedraw" , opt_general_fast_redraw, 1. ,
+  { F|O, "FastRedraw" , opt_general_fast_redraw, 0. ,
     "Draw simplified model while rotating, panning and zooming" },
   { F|S, "FileChooserPositionX" , opt_general_file_chooser_position0 , 200. , 
     "Horizontal position (in pixels) of the upper left corner of the file chooser windows" }, 
@@ -789,10 +810,6 @@ StringXNumber GeometryOptions_Number[] = {
 
   { F|O, "ScalingFactor" , opt_geometry_scaling_factor , 1.0 , 
     "Global geometry scaling factor" },
-  { F|O, "StlCreateElementary" , opt_geometry_stl_create_elementary , 0. ,
-    "Treat each STL input face as a new geometrical surface" },
-  { F|O, "StlCreatePhysical" , opt_geometry_stl_create_physical , 1. ,
-    "Automatically create physical entities when importing STL faces as geometrical surfaces" },
   { F|O, "Surfaces" , opt_geometry_surfaces , 0. , 
     "Display geometry surfaces?" },
   { F|O, "SurfaceNumbers" , opt_geometry_surfaces_num , 0. , 
@@ -813,7 +830,7 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "Algorithm" , opt_mesh_algo2d , DELAUNAY_ISO ,
     "2D mesh algorithm (1=isotropic, 2=anisotropic, 3=triangle)" }, 
   { F|O, "Algorithm3D" , opt_mesh_algo3d , DELAUNAY_ISO ,
-    "3D mesh algorithm (1=isotropic, 4=netgen)" }, 
+    "3D mesh algorithm (1=isotropic, 4=netgen, 5=tetgen)" }, 
   { F,   "AllowDegeneratedExtrude" , opt_mesh_allow_degenerated_extrude , 0. , 
     "Allow the generation of degenerated hexahedra or prisms during extrusion" },
   { F|O, "AngleSmoothNormals" , opt_mesh_angle_smooth_normals , 180.0 ,
@@ -842,9 +859,13 @@ StringXNumber MeshOptions_Number[] = {
   { F,   "CutPlaneD" , opt_mesh_cut_planed , 0.0 , 
     "Fourth cut plane equation coefficient (`D' in `AX+BY+CZ+D=0')" },
 
+  { F|O, "DihedralAngleTol" , opt_mesh_dihedral_angle_tol, 22. ,
+    "Dihedral angle tolerance for edge creation in the remesher" },
   { F|O, "Dual" , opt_mesh_dual , 0. ,
     "Display the dual mesh obtained by barycentric subdivision" },
 
+  { F|O, "EdgeProlongationThreshold" , opt_mesh_edge_prolongation_threshold, 1. ,
+    "Edge prolongation threshold in the remesher" },
   { F|O, "ElementOrder" , opt_mesh_order , 1. , // "Order" is already a lex token
     "Element order (1=linear elements, 2=quadratic elements)" },
   { F|O, "Explode" , opt_mesh_explode , 1.0 ,
@@ -853,16 +874,15 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "Format" , opt_mesh_format , FORMAT_MSH , 
     "Mesh output format (1=msh, 2=unv, 3=gref, 19=vrml)" },
 
-  { F|O, "GammaInf" , opt_mesh_gamma_inf , 0.0 , 
-    "Only display elements whose Gamma factor is greater than GammaInf" },
-  { F|O, "GammaSup" , opt_mesh_gamma_sup , 0.0 , 
-    "Only display elements whose Gamma factor is smaller than GammaSup" },
-
   { F|O, "Interactive" , opt_mesh_interactive , 0. ,
     "Show the construction of 2D anisotropic mesh in real time" },
 
+  { F|O, "LabelType" , opt_mesh_label_type , 0. , 
+    "Type of element label (0=element number, 1=elementary entity number, 2=physical entity number, 3=partition number)" },
   { F|O, "Light" , opt_mesh_light , 0. , 
     "Enable lighting for the mesh" },
+  { F|O, "LightLines" , opt_mesh_light_lines , 1. , 
+    "Enable lighting for mesh lines (element edges)" },
   { F|O, "LightTwoSide" , opt_mesh_light_two_side , 1. , 
     "Light both sides of mesh elements (leads to slower rendering)" },
   { F|O, "Lines" , opt_mesh_lines , 0. , 
@@ -879,6 +899,14 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "MshFileVersion" , opt_mesh_msh_file_version , 1.0 , 
     "Version of the `msh' file format to use" },
 
+  { F|O, "NbElemsPerRadiusOfCurv" , opt_mesh_nb_elem_per_rc, 5. ,
+    "Number of elements per radius of curvature in the remesher" },
+  { F|O, "TargetElmentSizeFact" , opt_mesh_target_elem_size_fact, 20. ,
+    "Target element size factor in the Remesher" },
+  { F|O, "MinimumElementSizeFact" , opt_mesh_min_elem_size_fact, 500. ,
+    "Minimum element size factor in the Remesher" },
+  { F|O, "BetaSmoothMetric" ,opt_mesh_beta_smooth_metric, 0.9 ,
+    "Maximum ratio of two consecutive edge lengths" },
   { F, "NbHexahedra" , opt_mesh_nb_hexahedra , 0. , 
     "Number of hexahedra in the current mesh (read-only)" },
   { F, "NbNodes" , opt_mesh_nb_nodes , 0. , 
@@ -912,8 +940,14 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "PointType" , opt_mesh_point_type , 0. , 
     "Display mesh vertices as solid color dots (0) or 3D spheres (1)" },
 
-  //{ F|O, "Quality" , opt_mesh_quality , 0.0 ,
-  //  "Target quality for tetrahedral elements (not fully functional)" },
+  { F|O, "Quality" , opt_mesh_quality , 1.0 ,
+    "Target quality for tetrahedral elements (currently only used by Tetgen)" },
+  { F|O, "QualityInf" , opt_mesh_quality_inf , 0.0 , 
+    "Only display elements whose quality measure is greater than QualityInf" },
+  { F|O, "QualitySup" , opt_mesh_quality_sup , 0.0 , 
+    "Only display elements whose quality measure is smaller than QualitySup" },
+  { F|O, "QualityType" , opt_mesh_quality_type , 0. , 
+    "Type of quality measure (0=gamma~vol/sum_face/max_edge, 1=eta~vol^(2/3)/sum_edge^2, 2=rho~min_edge/max_edge)" },
 
   { F|O, "RadiusInf" , opt_mesh_radius_inf , 0.0 , 
     "Only display elements whose longest edge is greater than RadiusInf" },
@@ -921,6 +955,8 @@ StringXNumber MeshOptions_Number[] = {
     "Only display elements whose longest edge is smaller than RadiusSup" },
   { F|O, "RandomFactor" , opt_mesh_rand_factor , 1.e-4 ,
     "Random factor used in 2D and 3D meshing algorithm (test other values when the algorithm fails)" },
+  { F|O, "RecombineAlgo" , opt_mesh_recombine_algo , 1 ,
+    "Recombine algorithm (1=mixed triangles-quadrangles, 2=all quadrangles)" }, 
 
   { F,   "SaveAll" , opt_mesh_save_all , 0. , 
     "Ignore Physical definitions and save all elements" },
@@ -928,10 +964,14 @@ StringXNumber MeshOptions_Number[] = {
     "Global scaling factor applied to the saved mesh" },
   { F|O, "Smoothing" , opt_mesh_nb_smoothing , 1. ,
     "Number of smoothing steps applied to the final mesh" },
+  { F|O, "Partitioning" , opt_mesh_nb_partitions , 1. ,
+    "Number of partitions applied to the final mesh" },
   { F|O, "SmoothNormals" , opt_mesh_smooth_normals , 0. , 
     "Smooth the mesh normals?" },
   { F|O, "SpeedMax" , opt_mesh_speed_max , 0. ,
     "Disable dubious point insertion tests" },
+  { F|O, "StlDistanceTol" , opt_mesh_stl_distance_tol, 5.e-7 ,
+    "Distance tolerance between two distinct vertices in STL meshes" },
   { F|O, "SurfaceEdges" , opt_mesh_surfaces_edges , 1. , 
     "Display edges of surface mesh?" },
   { F|O, "SurfaceFaces" , opt_mesh_surfaces_faces , 0. , 
@@ -951,6 +991,9 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "VolumeNumbers" , opt_mesh_volumes_num , 0. , 
     "Display volume mesh element numbers?" },
 
+  { F|O, "LabelsFrequency" , opt_mesh_label_frequency , 100. , 
+    "Labels display frequency?" },
+
   { 0, NULL , NULL , 0. , NULL }
 } ;
 
@@ -1028,8 +1071,6 @@ StringXNumber PostProcessingOptions_Number[] = {
   { F|O, "Plugins" , opt_post_plugins , 1. ,
     "Enable default post-processing plugins?" },
 
-  { F|O, "Scales" , opt_post_scales , 1. , 
-    "Show value scales" },
   { F|O, "Smoothing" , opt_post_smooth , 0. ,
     "Apply (non-reversible) smoothing to post-processing view when merged" },
 
@@ -1044,8 +1085,6 @@ StringXNumber ViewOptions_Number[] = {
     "Threshold angle below which normals are not smoothed" },
   { F|O, "ArrowHeadRadius" , opt_view_arrow_head_radius , 0.12 ,
     "Relative radius of arrow head" },
-  { F|O, "ArrowLocation" , opt_view_arrow_location , DRAW_POST_LOCATE_COG , 
-    "Arrow location (1=center of gravity, 2=node)" },
   { F|O, "ArrowSize" , opt_view_arrow_size , 60. ,
     "Display size of arrows (in pixels)" },
   { F|O, "ArrowSizeProportional" , opt_view_arrow_size_proportional , 1. ,
@@ -1145,6 +1184,8 @@ StringXNumber ViewOptions_Number[] = {
     "Generalized raise amplification factor" },
   { F|O, "GeneralizedRaiseView" , opt_view_gen_raise_view , -1. ,
     "Index of the view used for generalized raise (-1=self)" },
+  { F|O, "GlyphLocation" , opt_view_glyph_location , DRAW_POST_LOCATE_COG , 
+    "Glyph (arrow, number, etc.) location (1=center of gravity, 2=node)" },
   // for backward compatibility:
   { F,   "Grid" , opt_view_axes , 0 ,
     "Grid mode (this option is deprecated: use View.Axes instead)" },
@@ -1157,6 +1198,8 @@ StringXNumber ViewOptions_Number[] = {
 
   { F|O, "Light" , opt_view_light , 1. ,
     "Enable lighting for the view" },
+  { F|O, "LightLines" , opt_view_light_lines , 0. , 
+    "Light element edges" },
   { F|O, "LightTwoSide" , opt_view_light_two_side , 1. , 
     "Light both sides of view elements (leads to slower rendering)" },
   { F|O, "LineType" , opt_view_line_type , 0. , 
@@ -1221,10 +1264,12 @@ StringXNumber ViewOptions_Number[] = {
     "Show element boundaries?" },
   { F|O, "ShowScale" , opt_view_show_scale , 1. ,
     "Show value scale?" },
-  { F|O, "ShowTime" , opt_view_show_time , 1. ,
-    "Show time value in the scale? (1=only if NbTimeStep>1, 2=always)" },
+  { F|O, "ShowTime" , opt_view_show_time , 3. ,
+    "Time display mode (0=hidden, 1=value if multiple, 2=value always, 3=step if multiple, 4=step always)" },
   { F|O, "SmoothNormals" , opt_view_smooth_normals , 0. ,
     "Smooth the normals?" },
+  { F|O, "Stipple" , opt_view_use_stipple , 0. ,
+    "Stipple curves in 2D plots?" },
 
   { F|O, "Tangents" , opt_view_tangents , 0. ,
     "Display size of tangent vectors (in pixels)" }, 
@@ -1326,7 +1371,7 @@ StringXColor GeneralOptions_Color[] = {
     PACK_COLOR(0,   0,   0,   255),
     "Text color" },
   { F|O, "Axes" , opt_general_color_axes ,
-    PACK_COLOR(255, 255, 255,   255),
+    PACK_COLOR(255, 255, 255, 255),
     PACK_COLOR(128, 128, 128, 255),
     PACK_COLOR(0,   0,   0,   255),
     "Axes color" },
@@ -1407,21 +1452,77 @@ StringXColor GeometryOptions_Color[] = {
   { 0, NULL , NULL , 0, 0, 0 , NULL }
 } ;
 
-#define COL1  PACK_COLOR( 232, 210, 23, 255  )
-#define COL2  PACK_COLOR( 226, 167, 29, 255  )
-#define COL3  PACK_COLOR( 217, 113, 38, 255  )
-#define COL4  PACK_COLOR( 201, 54, 54, 255   )
-#define COL5  PACK_COLOR( 169, 12, 86, 255   )
-#define COL6  PACK_COLOR( 114, 2, 141, 255   )
-#define COL7  PACK_COLOR( 67, 30, 188, 255   )
-#define COL8  PACK_COLOR( 44, 86, 211, 255   )
-#define COL9  PACK_COLOR( 32, 145, 223, 255  )
-#define COL10 PACK_COLOR( 25, 193, 230, 255  )
 #define COLW  PACK_COLOR( 210, 210, 210, 255 )
 #define COLT  PACK_COLOR( 160, 150, 255, 255 )
 #define COLQ  PACK_COLOR( 130, 120, 225, 255 )
-#define COLP  PACK_COLOR( 232, 210, 23, 255  )
-#define COLY  PACK_COLOR( 217, 113, 38, 255  )
+#define COLP  PACK_COLOR( 232, 210, 23, 255 )
+#define COLY  PACK_COLOR( 217, 113, 38, 255 )
+
+// dark blue to yellow:
+#define COL0   PACK_COLOR( 9, 3, 103, 255 )
+#define COL1   PACK_COLOR( 35, 0, 111, 255 )
+#define COL2   PACK_COLOR( 60, 0, 114, 255 )
+#define COL3   PACK_COLOR( 83, 0, 114, 255 )
+#define COL4   PACK_COLOR( 105, 1, 109, 255 )
+#define COL5   PACK_COLOR( 126, 10, 102, 255 )
+#define COL6   PACK_COLOR( 145, 21, 93, 255 )
+#define COL7   PACK_COLOR( 163, 34, 82, 255 )
+#define COL8   PACK_COLOR( 179, 50, 69, 255 )
+#define COL9   PACK_COLOR( 193, 67, 57, 255 )
+#define COL10  PACK_COLOR( 207, 85, 44, 255 )
+#define COL11  PACK_COLOR( 219, 104, 32, 255 )
+#define COL12  PACK_COLOR( 229, 124, 22, 255 )
+#define COL13  PACK_COLOR( 238, 143, 13, 255 )
+#define COL14  PACK_COLOR( 246, 162, 8, 255 )
+#define COL15  PACK_COLOR( 252, 181, 5, 255 )
+#define COL16  PACK_COLOR( 255, 198, 6, 255 )
+#define COL17  PACK_COLOR( 255, 214, 11, 255 )
+#define COL18  PACK_COLOR( 255, 228, 22, 255 )
+#define COL19  PACK_COLOR( 255, 240, 38, 255 )
+
+// jet:
+// #define COL0   PACK_COLOR( 0, 0, 143, 255)
+// #define COL1   PACK_COLOR( 0, 0, 197, 255)
+// #define COL2   PACK_COLOR( 0, 0, 250, 255)
+// #define COL3   PACK_COLOR( 0, 49, 255, 255)
+// #define COL4   PACK_COLOR( 0, 103, 255, 255)
+// #define COL5   PACK_COLOR( 0, 156, 255, 255)
+// #define COL6   PACK_COLOR( 0, 210, 255, 255)
+// #define COL7   PACK_COLOR( 9, 255, 253, 255)
+// #define COL8   PACK_COLOR( 62, 255, 200, 255)
+// #define COL9   PACK_COLOR( 116, 255, 146, 255)
+// #define COL10  PACK_COLOR( 170, 255, 92, 255)
+// #define COL11  PACK_COLOR( 223, 255, 39, 255)
+// #define COL12  PACK_COLOR( 255, 240, 0, 255)
+// #define COL13  PACK_COLOR( 255, 186, 0, 255)
+// #define COL14  PACK_COLOR( 255, 132, 0, 255)
+// #define COL15  PACK_COLOR( 255, 79, 0, 255)
+// #define COL16  PACK_COLOR( 255, 25, 0, 255)
+// #define COL17  PACK_COLOR( 226, 0, 0, 255)
+// #define COL18  PACK_COLOR( 173, 0, 0, 255)
+// #define COL19  PACK_COLOR( 119, 0, 0, 255)
+
+// truncated hsv:
+// #define COL0  PACK_COLOR(  255, 0, 0, 255)
+// #define COL1  PACK_COLOR(  255, 67, 0, 255)
+// #define COL2  PACK_COLOR(  255, 134, 0, 255)
+// #define COL3  PACK_COLOR(  255, 201, 0, 255)
+// #define COL4  PACK_COLOR(  241, 255, 0, 255)
+// #define COL5  PACK_COLOR(  174, 255, 0, 255)
+// #define COL6  PACK_COLOR(  107, 255, 0, 255)
+// #define COL7  PACK_COLOR(  40, 255, 0, 255)
+// #define COL8  PACK_COLOR(  0, 255, 26, 255)
+// #define COL9  PACK_COLOR(  0, 255, 93, 255)
+// #define COL10  PACK_COLOR( 0, 255, 161, 255)
+// #define COL11  PACK_COLOR( 0, 255, 228, 255)
+// #define COL12  PACK_COLOR( 0, 214, 255, 255)
+// #define COL13  PACK_COLOR( 0, 147, 255, 255)
+// #define COL14  PACK_COLOR( 0, 80, 255, 255)
+// #define COL15  PACK_COLOR( 0, 13, 255, 255)
+// #define COL16  PACK_COLOR( 53, 0, 255, 255)
+// #define COL17  PACK_COLOR( 120, 0, 255, 255)
+// #define COL18  PACK_COLOR( 187, 0, 255, 255)
+// #define COL19  PACK_COLOR( 255, 0, 254, 255)
 
 StringXColor MeshOptions_Color[] = {
   { F|O, "Points" , opt_mesh_color_points , 
@@ -1461,31 +1562,29 @@ StringXColor MeshOptions_Color[] = {
     PACK_COLOR(255, 0, 0, 255),
     PACK_COLOR(0,   0, 0, 255),
     "Normal mesh vector color" },
-  { F|O, "One" , opt_mesh_color_1   , COL1, COL1, COLW, "First color in color carousel" },
-  { F|O, "Two" , opt_mesh_color_2   , COL2, COL2, COLW, "Second color in color carousel" },
-  { F|O, "Three" , opt_mesh_color_3 , COL3, COL3, COLW, "Third color in color carousel" },
-  { F|O, "Four" , opt_mesh_color_4  , COL4, COL4, COLW, "Fourth color in color carousel" },
-  { F|O, "Five" , opt_mesh_color_5  , COL5, COL5, COLW, "Fifth color in color carousel" },
-  { F|O, "Six" , opt_mesh_color_6   , COL6, COL6, COLW, "Sixth color in color carousel" },
-  { F|O, "Seven" , opt_mesh_color_7 , COL7, COL7, COLW, "Seventh color in color carousel" },
-  { F|O, "Eight" , opt_mesh_color_8 , COL8, COL8, COLW, "Eighth color in color carousel" },
-  { F|O, "Nine" , opt_mesh_color_9  , COL9, COL9, COLW, "Ninth color in color carousel" },
-  { F|O, "Ten" , opt_mesh_color_10  , COL10, COL10, COLW,  "Tenth color in color carousel" },
+  { F|O, "Zero"     , opt_mesh_color_0 , COL0, COL0, COLW, "Color 0 in color carousel" },
+  { F|O, "One"      , opt_mesh_color_1 , COL1, COL1, COLW, "Color 1 in color carousel" },
+  { F|O, "Two"      , opt_mesh_color_2 , COL2, COL2, COLW, "Color 2 in color carousel" },
+  { F|O, "Three"    , opt_mesh_color_3 , COL3, COL3, COLW, "Color 3 in color carousel" },
+  { F|O, "Four"     , opt_mesh_color_4 , COL4, COL4, COLW, "Color 4 in color carousel" },
+  { F|O, "Five"     , opt_mesh_color_5 , COL5, COL5, COLW, "Color 5 in color carousel" },
+  { F|O, "Six"      , opt_mesh_color_6 , COL6, COL6, COLW, "Color 6 in color carousel" },
+  { F|O, "Seven"    , opt_mesh_color_7 , COL7, COL7, COLW, "Color 7 in color carousel" },
+  { F|O, "Eight"    , opt_mesh_color_8 , COL8, COL8, COLW, "Color 8 in color carousel" },
+  { F|O, "Nine"     , opt_mesh_color_9 , COL9, COL9, COLW, "Color 9 in color carousel" },
+  { F|O, "Ten"      , opt_mesh_color_10, COL10, COL10, COLW, "Color 10 in color carousel" },
+  { F|O, "Eleven"   , opt_mesh_color_11, COL11, COL11, COLW, "Color 11 in color carousel" },
+  { F|O, "Twelve"   , opt_mesh_color_12, COL12, COL12, COLW, "Color 12 in color carousel" },
+  { F|O, "Thirtheen", opt_mesh_color_13, COL13, COL13, COLW, "Color 13 in color carousel" },
+  { F|O, "Fourteen" , opt_mesh_color_14, COL14, COL14, COLW, "Color 14 in color carousel" },
+  { F|O, "Fifteen"  , opt_mesh_color_15, COL15, COL15, COLW, "Color 15 in color carousel" },
+  { F|O, "Sixteen"  , opt_mesh_color_16, COL16, COL16, COLW, "Color 16 in color carousel" },
+  { F|O, "Seventeen", opt_mesh_color_17, COL17, COL17, COLW, "Color 17 in color carousel" },
+  { F|O, "Eighteen" , opt_mesh_color_18, COL18, COL18, COLW, "Color 18 in color carousel" },
+  { F|O, "Nineteen" , opt_mesh_color_19, COL19, COL19, COLW, "Color 19 in color carousel" },
   { 0, NULL , NULL , 0, 0, 0 , NULL }
 } ;
 
-#undef COL1
-#undef COL2
-#undef COL3
-#undef COL4
-#undef COL5
-#undef COL6
-#undef COL7
-#undef COL8
-#undef COL9
-#undef COL10
-#undef COLW
-
 StringXColor SolverOptions_Color[] = {
   { 0, NULL , NULL , 0, 0, 0 , NULL }
 } ;
@@ -1521,8 +1620,6 @@ StringXColor ViewOptions_Color[] = {
   { 0, NULL , NULL , 0, 0, 0 , NULL }
 } ;
 
-#undef ELECOL
-
 StringXColor PrintOptions_Color[] = {
   { 0, NULL , NULL , 0, 0, 0 , NULL }
 } ;
