@@ -211,6 +211,16 @@ char * opt_view_axes_format2(OPT_ARGS_STR);
 char * opt_view_gen_raise0(OPT_ARGS_STR);
 char * opt_view_gen_raise1(OPT_ARGS_STR);
 char * opt_view_gen_raise2(OPT_ARGS_STR);
+char * opt_view_stipple0(OPT_ARGS_STR);
+char * opt_view_stipple1(OPT_ARGS_STR);
+char * opt_view_stipple2(OPT_ARGS_STR);
+char * opt_view_stipple3(OPT_ARGS_STR);
+char * opt_view_stipple4(OPT_ARGS_STR);
+char * opt_view_stipple5(OPT_ARGS_STR);
+char * opt_view_stipple6(OPT_ARGS_STR);
+char * opt_view_stipple7(OPT_ARGS_STR);
+char * opt_view_stipple8(OPT_ARGS_STR);
+char * opt_view_stipple9(OPT_ARGS_STR);
 
 // NUMBERS
 
@@ -381,6 +391,7 @@ double opt_geometry_point_size(OPT_ARGS_NUM);
 double opt_geometry_point_sel_size(OPT_ARGS_NUM);
 double opt_geometry_point_type(OPT_ARGS_NUM);
 double opt_geometry_line_width(OPT_ARGS_NUM);
+double opt_mesh_label_frequency(OPT_ARGS_NUM);
 double opt_geometry_line_sel_width(OPT_ARGS_NUM);
 double opt_geometry_line_type(OPT_ARGS_NUM);
 double opt_geometry_light(OPT_ARGS_NUM);
@@ -390,8 +401,6 @@ double opt_geometry_circle_points(OPT_ARGS_NUM);
 double opt_geometry_circle_warning(OPT_ARGS_NUM);
 double opt_geometry_extrude_spline_points(OPT_ARGS_NUM);
 double opt_geometry_scaling_factor(OPT_ARGS_NUM);
-double opt_geometry_stl_create_elementary(OPT_ARGS_NUM);
-double opt_geometry_stl_create_physical(OPT_ARGS_NUM);
 double opt_mesh_optimize(OPT_ARGS_NUM);
 double opt_mesh_quality(OPT_ARGS_NUM);
 double opt_mesh_normals(OPT_ARGS_NUM);
@@ -400,10 +409,12 @@ double opt_mesh_explode(OPT_ARGS_NUM);
 double opt_mesh_scaling_factor(OPT_ARGS_NUM);
 double opt_mesh_lc_factor(OPT_ARGS_NUM);
 double opt_mesh_rand_factor(OPT_ARGS_NUM);
-double opt_mesh_gamma_inf(OPT_ARGS_NUM);
-double opt_mesh_gamma_sup(OPT_ARGS_NUM);
+double opt_mesh_quality_inf(OPT_ARGS_NUM);
+double opt_mesh_quality_sup(OPT_ARGS_NUM);
+double opt_mesh_quality_type(OPT_ARGS_NUM);
 double opt_mesh_radius_inf(OPT_ARGS_NUM);
 double opt_mesh_radius_sup(OPT_ARGS_NUM);
+double opt_mesh_label_type(OPT_ARGS_NUM);
 double opt_mesh_points(OPT_ARGS_NUM);
 double opt_mesh_lines(OPT_ARGS_NUM);
 double opt_mesh_surfaces_edges(OPT_ARGS_NUM);
@@ -422,11 +433,21 @@ double opt_mesh_vertex_arrays(OPT_ARGS_NUM);
 double opt_mesh_smooth_normals(OPT_ARGS_NUM);
 double opt_mesh_angle_smooth_normals(OPT_ARGS_NUM);
 double opt_mesh_light(OPT_ARGS_NUM);
+double opt_mesh_light_lines(OPT_ARGS_NUM);
 double opt_mesh_light_two_side(OPT_ARGS_NUM);
 double opt_mesh_format(OPT_ARGS_NUM);
 double opt_mesh_msh_file_version(OPT_ARGS_NUM);
 double opt_mesh_nb_smoothing(OPT_ARGS_NUM);
+double opt_mesh_stl_distance_tol(OPT_ARGS_NUM);
+double opt_mesh_nb_elem_per_rc(OPT_ARGS_NUM);
+double opt_mesh_min_elem_size_fact(OPT_ARGS_NUM);
+double opt_mesh_target_elem_size_fact(OPT_ARGS_NUM);
+double opt_mesh_beta_smooth_metric(OPT_ARGS_NUM);
+double opt_mesh_dihedral_angle_tol(OPT_ARGS_NUM);
+double opt_mesh_edge_prolongation_threshold(OPT_ARGS_NUM);
+double opt_mesh_nb_partitions(OPT_ARGS_NUM);
 double opt_mesh_algo2d(OPT_ARGS_NUM);
+double opt_mesh_recombine_algo(OPT_ARGS_NUM);
 double opt_mesh_algo3d(OPT_ARGS_NUM);
 double opt_mesh_point_insertion(OPT_ARGS_NUM);
 double opt_mesh_speed_max(OPT_ARGS_NUM);
@@ -474,7 +495,6 @@ double opt_solver_merge_views2(OPT_ARGS_NUM);
 double opt_solver_merge_views3(OPT_ARGS_NUM);
 double opt_solver_merge_views4(OPT_ARGS_NUM);
 double opt_post_vertex_arrays(OPT_ARGS_NUM);
-double opt_post_scales(OPT_ARGS_NUM);
 double opt_post_horizontal_scales(OPT_ARGS_NUM);
 double opt_post_link(OPT_ARGS_NUM);
 double opt_post_smooth(OPT_ARGS_NUM);
@@ -520,6 +540,7 @@ double opt_view_normals(OPT_ARGS_NUM);
 double opt_view_tangents(OPT_ARGS_NUM);
 double opt_view_displacement_factor(OPT_ARGS_NUM);
 double opt_view_fake_transparency(OPT_ARGS_NUM);
+double opt_view_use_stipple(OPT_ARGS_NUM);
 double opt_view_explode(OPT_ARGS_NUM);
 double opt_view_visible(OPT_ARGS_NUM);
 double opt_view_intervals_type(OPT_ARGS_NUM);
@@ -560,6 +581,7 @@ double opt_view_nb_iso(OPT_ARGS_NUM);
 double opt_view_boundary(OPT_ARGS_NUM);
 double opt_view_light(OPT_ARGS_NUM);
 double opt_view_light_two_side(OPT_ARGS_NUM);
+double opt_view_light_lines(OPT_ARGS_NUM);
 double opt_view_smooth_normals(OPT_ARGS_NUM);
 double opt_view_angle_smooth_normals(OPT_ARGS_NUM);
 double opt_view_show_element(OPT_ARGS_NUM);
@@ -581,7 +603,7 @@ double opt_view_scale_type(OPT_ARGS_NUM);
 double opt_view_tensor_type(OPT_ARGS_NUM);
 double opt_view_range_type(OPT_ARGS_NUM);
 double opt_view_vector_type(OPT_ARGS_NUM);
-double opt_view_arrow_location(OPT_ARGS_NUM);
+double opt_view_glyph_location(OPT_ARGS_NUM);
 double opt_view_point_size(OPT_ARGS_NUM);
 double opt_view_line_width(OPT_ARGS_NUM);
 double opt_view_point_type(OPT_ARGS_NUM);
@@ -633,6 +655,7 @@ unsigned int opt_mesh_color_prisms(OPT_ARGS_COL);
 unsigned int opt_mesh_color_pyramid(OPT_ARGS_COL);
 unsigned int opt_mesh_color_tangents(OPT_ARGS_COL);
 unsigned int opt_mesh_color_normals(OPT_ARGS_COL);
+unsigned int opt_mesh_color_0(OPT_ARGS_COL);
 unsigned int opt_mesh_color_1(OPT_ARGS_COL);
 unsigned int opt_mesh_color_2(OPT_ARGS_COL);
 unsigned int opt_mesh_color_3(OPT_ARGS_COL);
@@ -643,6 +666,15 @@ unsigned int opt_mesh_color_7(OPT_ARGS_COL);
 unsigned int opt_mesh_color_8(OPT_ARGS_COL);
 unsigned int opt_mesh_color_9(OPT_ARGS_COL);
 unsigned int opt_mesh_color_10(OPT_ARGS_COL);
+unsigned int opt_mesh_color_11(OPT_ARGS_COL);
+unsigned int opt_mesh_color_12(OPT_ARGS_COL);
+unsigned int opt_mesh_color_13(OPT_ARGS_COL);
+unsigned int opt_mesh_color_14(OPT_ARGS_COL);
+unsigned int opt_mesh_color_15(OPT_ARGS_COL);
+unsigned int opt_mesh_color_16(OPT_ARGS_COL);
+unsigned int opt_mesh_color_17(OPT_ARGS_COL);
+unsigned int opt_mesh_color_18(OPT_ARGS_COL);
+unsigned int opt_mesh_color_19(OPT_ARGS_COL);
 unsigned int opt_view_color_points(OPT_ARGS_COL);
 unsigned int opt_view_color_lines(OPT_ARGS_COL);
 unsigned int opt_view_color_triangles(OPT_ARGS_COL);
