@@ -1,4 +1,4 @@
-// $Id: VertexArray.cpp,v 1.4 2005/01/01 19:35:27 geuzaine Exp $
+// $Id: VertexArray.cpp,v 1.8 2005/10/09 17:45:37 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -27,8 +27,8 @@
 VertexArray::VertexArray(int numNodesPerElement, int numElements) 
 {
   type = numNodesPerElement;
-  if(type != 3 && type != 4){
-    Msg(GERROR, "Vertex arrays should only contain triangles or quadrangles");
+  if(type < 1 || type > 4){
+    Msg(GERROR, "Vertex arrays not done for %d-node element", type);
     type = 3;
   }
   num = fill = 0;
@@ -66,6 +66,21 @@ void VertexArray::add(float x, float y, float z,
   List_Add(colors, &a);
 }
 
+void VertexArray::add(float x, float y, float z, unsigned int col)
+{
+  unsigned char r = UNPACK_RED(col);
+  unsigned char g = UNPACK_GREEN(col);
+  unsigned char b = UNPACK_BLUE(col);
+  unsigned char a = UNPACK_ALPHA(col);
+  List_Add(vertices, &x);
+  List_Add(vertices, &y);
+  List_Add(vertices, &z);
+  List_Add(colors, &r);
+  List_Add(colors, &g);
+  List_Add(colors, &b);
+  List_Add(colors, &a);
+}
+
 static double theeye[3];
 
 int compareTriEye(const void *a, const void *b)
