@@ -1,4 +1,4 @@
-// $Id: Main.cpp,v 1.75 2005/01/01 19:35:28 geuzaine Exp $
+// $Id: Main.cpp,v 1.84 2005/10/24 15:38:13 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -19,9 +19,13 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
+#include <iostream>
+#include <vector>
 #include <signal.h>
 #include <time.h>
 
+#include "GUI.h"
+
 #include "PluginManager.h"
 #include "Gmsh.h"
 #include "GmshUI.h"
@@ -34,10 +38,10 @@
 #include "Options.h"
 #include "ColorTable.h"
 #include "Parser.h"
-#include "GUI.h"
 #include "OpenFile.h"
 #include "CommandLine.h"
 #include "Numeric.h"
+#include "Solvers.h"
 
 char yyname[256] = "";
 int yyerrorstate = 0;
@@ -47,7 +51,6 @@ GUI *WID = NULL;
 
 int main(int argc, char *argv[])
 {
-  int i, nbf;
   char *cmdline, currtime[100];
   time_t now;
 
@@ -58,36 +61,28 @@ int main(int argc, char *argv[])
   currtime[strlen(currtime) - 1] = '\0';
 
   int cll = 0;
-  for(i = 0; i < argc; i++) {
+  for(int i = 0; i < argc; i++) {
     cll += strlen(argv[i]);
   }
   cmdline = (char*)Malloc((cll+argc+1)*sizeof(char));
   cmdline[0] = '\0';
-  for(i = 0; i < argc; i++) {
+  for(int i = 0; i < argc; i++) {
     strcat(cmdline, argv[i]);
     strcat(cmdline, " ");
   }
 
+  // Initialize the symbol tree that will hold variable names
+  
+  InitSymbols();
+
+  // Initialize the static Mesh
+  
+  Init_Mesh0(&M);
+
   // Gmsh default options
 
   Init_Options(0);
 
-  // Initialize the static Mesh
-
-  M.Vertices = NULL;
-  M.Simplexes = NULL;
-  M.Points = NULL;
-  M.Curves = NULL;
-  M.SurfaceLoops = NULL;
-  M.EdgeLoops = NULL;
-  M.Surfaces = NULL;
-  M.Volumes = NULL;
-  M.PhysicalGroups = NULL;
-  M.Partitions = NULL;
-  M.Metric = NULL;
-  M.BGM.bgm = NULL;
-  M.Grid.init = 0;
-
   // Generate automatic documentation (before getting user-defined options)
   
   if(argc == 2 && !strcmp(argv[1], "-doc")){
@@ -99,7 +94,7 @@ int main(int argc, char *argv[])
 
   // Read configuration files and command line options
 
-  Get_Options(argc, argv, &nbf);
+  Get_Options(argc, argv);
 
   // Always print info on terminal for non-interactive execution
 
@@ -108,9 +103,12 @@ int main(int argc, char *argv[])
 
   // Signal handling
 
+  // FIXME: could not make this work on IRIX
+#if !defined(__sgi__) 
   signal(SIGINT, Signal);
   signal(SIGSEGV, Signal);
   signal(SIGFPE, Signal);
+#endif
 
   // Initialize the default plugins
 
@@ -125,12 +123,12 @@ int main(int argc, char *argv[])
     if(yyerrorstate)
       exit(1);
     else {
-      for(i = 1; i < nbf; i++)
-        MergeProblem(TheFileNameTab[i]);
+      for(int i = 1; i < List_Nbr(CTX.files); i++)
+        MergeProblem(*(char**)List_Pointer(CTX.files, i));
       if(CTX.post.combine_time)
 	CombineViews(1, 2, CTX.post.combine_remove_orig);
-      if(TheBgmFileName) {
-        MergeProblem(TheBgmFileName);
+      if(CTX.bgm_filename) {
+        MergeProblem(CTX.bgm_filename);
         if(List_Nbr(CTX.post.list))
           BGMWithView(*(Post_View **)
                       List_Pointer(CTX.post.list,
@@ -201,8 +199,8 @@ int main(int argc, char *argv[])
   // Open project file and merge all other input files
 
   OpenProblem(CTX.filename);
-  for(i = 1; i < nbf; i++)
-    MergeProblem(TheFileNameTab[i]);
+  for(int i = 1; i < List_Nbr(CTX.files); i++)
+    MergeProblem(*(char**)List_Pointer(CTX.files, i));
   if(CTX.post.combine_time)
     CombineViews(1, 2, CTX.post.combine_remove_orig);
   
@@ -231,8 +229,8 @@ int main(int argc, char *argv[])
 
   // Read background mesh on disk
 
-  if(TheBgmFileName) {
-    MergeProblem(TheBgmFileName);
+  if(CTX.bgm_filename) {
+    MergeProblem(CTX.bgm_filename);
     if(List_Nbr(CTX.post.list))
       BGMWithView(*(Post_View **)
                   List_Pointer(CTX.post.list, List_Nbr(CTX.post.list) - 1));
@@ -243,6 +241,10 @@ int main(int argc, char *argv[])
   // Draw the actual scene
   Draw();
 
+  // Listen to external solvers
+  if(CTX.solver.listen)
+    Solver(-1, NULL);
+
   // loop
   return WID->run();
 }
