@@ -38,6 +38,14 @@ void Orthogonalize(int x, int y);
 void ClearOpengl(void);
 void SetOpenglContext(void);
 
+typedef struct{
+  unsigned int type, ient, depth;
+} hit;
+
+void Process_SelectionBuffer(int x, int y, int *n, hit *hits);
+int Filter_SelectionBuffer(int type, int n, hit *hits, 
+			    Vertex **thev, Curve **thec, Surface **thes, Mesh *m);
+
 void unproject(double x, double y, double p[3], double d[3]);
 void Viewport2World(double win[3], double xyz[3]);
 void World2Viewport(double xyz[3], double win[3]);
@@ -100,7 +108,7 @@ void Draw_Mesh_Tetrahedron(void *a, void *b);
 void Draw_Mesh_Hexahedron(void *a, void *b);
 void Draw_Mesh_Prism(void *a, void *b);
 void Draw_Mesh_Pyramid(void *a, void *b);
-void Draw_Mesh_Array(VertexArray *va, int faces, int edges);
+void Draw_Mesh_Array(VertexArray *va, int faces=0, int edges=0);
 
 #define ARGS Post_View *View, int preproNormals, \
              double ValMin, double ValMax, 	 \
