@@ -1,4 +1,4 @@
-// $Id: Geom.cpp,v 1.81 2005/02/20 06:36:54 geuzaine Exp $
+// $Id: Geom.cpp,v 1.94 2005/10/15 19:49:21 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -19,6 +19,7 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
+#include "BDS.h"
 #include "Gmsh.h"
 #include "GmshUI.h"
 #include "Numeric.h"
@@ -66,10 +67,10 @@ void Draw_Geo_Point(void *a, void *b)
   if(CTX.geom.points) {
     if(CTX.geom.point_type == 1) {
       if(v->Frozen)
-        Draw_Sphere(CTX.geom.point_sel_size, v->Pos.X, v->Pos.Y, v->Pos.Z, 
+	Draw_Sphere(CTX.geom.point_sel_size, v->Pos.X, v->Pos.Y, v->Pos.Z, 
 		    CTX.geom.light);
       else
-        Draw_Sphere(CTX.geom.point_size, v->Pos.X, v->Pos.Y, v->Pos.Z,
+	Draw_Sphere(CTX.geom.point_size, v->Pos.X, v->Pos.Y, v->Pos.Z,
 		    CTX.geom.light);
     }
     else if(CTX.geom.point_type == 2) {
@@ -93,8 +94,8 @@ void Draw_Geo_Point(void *a, void *b)
     sprintf(Num, "%d", v->Num);
     double offset = (0.5 * CTX.geom.point_size + 0.3 * CTX.gl_fontsize) * CTX.pixel_equiv_x;
     glRasterPos3d(v->Pos.X + offset / CTX.s[0],
-                  v->Pos.Y + offset / CTX.s[1],
-                  v->Pos.Z + offset / CTX.s[2]);
+		  v->Pos.Y + offset / CTX.s[1],
+		  v->Pos.Z + offset / CTX.s[2]);
     Draw_String(Num);
   }
 
@@ -114,7 +115,7 @@ void Draw_Curve(void *a, void *b)
 
   Curve *c = *(Curve **) a;
 
-  if(c->Num < 0 || !(c->Visible & VIS_GEOM) || c->Dirty)
+  if(c->Num < 0 || !(c->Visible & VIS_GEOM))
     return;
 
   if(CTX.render_mode == GMSH_SELECT) {
@@ -122,7 +123,7 @@ void Draw_Curve(void *a, void *b)
     glPushName(c->Num);
   }
 
-  if(c->ipar[3]) {
+  if(c->ipar[3] > 0) {
     glLineWidth(CTX.geom.line_sel_width);
     gl2psLineWidth(CTX.geom.line_sel_width * CTX.print.eps_line_width_factor);
     glColor4ubv((GLubyte *) & CTX.color.geom.line_sel);
@@ -149,40 +150,48 @@ void Draw_Curve(void *a, void *b)
       N = 10 * n;
       break;
     }
-    if(c->theSegmRep) {
-      glBegin(GL_LINE_STRIP);
-      for(int i = 0; i < List_Nbr(c->theSegmRep->points); i+=3) {
-        double *p = (double*)List_Pointer(c->theSegmRep->points, i);
-        glVertex3d(p[0], p[1], p[2]);
+    if(c->Typ == MSH_SEGM_DISCRETE && THEM->bds) {
+      BDS_GeomEntity *g = THEM->bds->get_geom ( c->Num,1);	
+      std::list<BDS_Edge*>::iterator it  = g->e.begin();
+      std::list<BDS_Edge*>::iterator ite = g->e.end();
+      while (it!=ite){
+	BDS_Edge *e = (*it);
+	if(CTX.geom.line_type < 1) {
+	  glBegin(GL_LINES);
+	  glVertex3d(e->p1->X,e->p1->Y,e->p1->Z);
+	  glVertex3d(e->p2->X,e->p2->Y,e->p2->Z);
+	  glEnd();
+	}
+	else{
+	  x[0] = e->p1->X;
+	  y[0] = e->p1->Y;
+	  z[0] = e->p1->Z;
+	  x[1] = e->p2->X;
+	  y[1] = e->p2->Y;
+	  z[1] = e->p2->Z;
+	  Draw_Cylinder(c->ipar[3] > 0 ? CTX.geom.line_sel_width : CTX.geom.line_width,
+			x, y, z, CTX.geom.light);
+	}
+	++it;
       }
-      glEnd();
     }
     else if(c->Typ == MSH_SEGM_DISCRETE) {
-      Simplex *s;
-      List_T *temp = Tree2List(c->Simplexes);
-      for(int i = 0; i < List_Nbr(temp); i++) {
-        List_Read(temp, i, &s);
-        glBegin(GL_LINES);
-        glVertex3d(s->V[0]->Pos.X, s->V[0]->Pos.Y, s->V[0]->Pos.Z);
-        glVertex3d(s->V[1]->Pos.X, s->V[1]->Pos.Y, s->V[1]->Pos.Z);
-        glEnd();
-      }
-      List_Delete(temp);
+      // do nothing: we draw the elements in the mesh drawing routines
     }
     else {
       if(CTX.geom.line_type >= 1) {
-        for(int i = 0; i < N - 1; i++) {
-          v = InterpolateCurve(c, (double)i / (double)(N - 1), 0);
-          dv = InterpolateCurve(c, (double)(i + 1) / (double)(N - 1), 0);
-          x[0] = v.Pos.X;
-          y[0] = v.Pos.Y;
-          z[0] = v.Pos.Z;
-          x[1] = dv.Pos.X;
-          y[1] = dv.Pos.Y;
-          z[1] = dv.Pos.Z;
-          Draw_Cylinder(c->ipar[3] ? CTX.geom.line_sel_width : CTX.geom.line_width,
+	for(int i = 0; i < N - 1; i++) {
+	  v = InterpolateCurve(c, (double)i / (double)(N - 1), 0);
+	  dv = InterpolateCurve(c, (double)(i + 1) / (double)(N - 1), 0);
+	  x[0] = v.Pos.X;
+	  y[0] = v.Pos.Y;
+	  z[0] = v.Pos.Z;
+	  x[1] = dv.Pos.X;
+	  y[1] = dv.Pos.Y;
+	  z[1] = dv.Pos.Z;
+	  Draw_Cylinder(c->ipar[3] > 0 ? CTX.geom.line_sel_width : CTX.geom.line_width,
 			x, y, z, CTX.geom.light);
-        }
+	}
 	if(CTX.geom.line_type == 2) {
 	  GMSH_Solve_Plugin *sp = GMSH_PluginManager::instance()->findSolverPlugin();
 	  if(sp) {
@@ -197,12 +206,12 @@ void Draw_Curve(void *a, void *b)
 	}
       }
       else {
-        glBegin(GL_LINE_STRIP);
-        for(int i = 0; i < N; i++) {
-          v = InterpolateCurve(c, (double)i / (double)(N - 1), 0);
-          glVertex3d(v.Pos.X, v.Pos.Y, v.Pos.Z);
-        }
-        glEnd();
+	glBegin(GL_LINE_STRIP);
+	for(int i = 0; i < N; i++) {
+	  v = InterpolateCurve(c, (double)i / (double)(N - 1), 0);
+	  glVertex3d(v.Pos.X, v.Pos.Y, v.Pos.Z);
+	}
+	glEnd();
       }
     }
   }
@@ -212,8 +221,8 @@ void Draw_Curve(void *a, void *b)
     sprintf(Num, "%d", c->Num);
     double offset = (0.5 * CTX.geom.line_width + 0.3 * CTX.gl_fontsize) * CTX.pixel_equiv_x;
     glRasterPos3d(v.Pos.X + offset / CTX.s[0],
-                  v.Pos.Y + offset / CTX.s[1],
-                  v.Pos.Z + offset / CTX.s[2]);
+		  v.Pos.Y + offset / CTX.s[1],
+		  v.Pos.Z + offset / CTX.s[2]);
     Draw_String(Num);
   }
 
@@ -228,7 +237,7 @@ void Draw_Curve(void *a, void *b)
     Draw_Vector(CTX.vector_type, 0, CTX.arrow_rel_head_radius, 
 		CTX.arrow_rel_stem_length, CTX.arrow_rel_stem_radius,
 		v.Pos.X, v.Pos.Y, v.Pos.Z,
-                dv.Pos.X, dv.Pos.Y, dv.Pos.Z, CTX.geom.light);
+		dv.Pos.X, dv.Pos.Y, dv.Pos.Z, CTX.geom.light);
   }
 
   if(CTX.render_mode == GMSH_SELECT) {
@@ -307,11 +316,17 @@ void getPlaneSurfaceNormal(Surface *s, double x, double y, double z, double n[3]
       t1[0] = x - c->beg->Pos.X;
       t1[1] = y - c->beg->Pos.Y;
       t1[2] = z - c->beg->Pos.Z;
-      for(int i = 1; i < List_Nbr(s->Generatrices); i++){
+      // 1) try to get a point close to 'beg' on the same curve
+      // 2) if we are really unlucky and these two points are aligned 
+      // with (x,y,z), which we know is inside or on the boundary of
+      // the surface, then get a point from the next generatrice
+      // 3) repeat
+      for(int i = 0; i < List_Nbr(s->Generatrices); i++){
 	List_Read(s->Generatrices, i, &c);
-	t2[0] = x - c->beg->Pos.X;
-	t2[1] = y - c->beg->Pos.Y;
-	t2[2] = z - c->beg->Pos.Z;
+	Vertex v = InterpolateCurve(c, 0.1, 0);
+	t2[0] = x - v.Pos.X;
+	t2[1] = y - v.Pos.Y;
+	t2[2] = z - v.Pos.Z;
 	prodve(t1, t2, n);
 	if(norme(n))
 	  break;
@@ -332,36 +347,24 @@ void Draw_Polygonal_Surface(Surface * s)
   if(CTX.geom.surfaces) {
     if(CTX.geom.light) glEnable(GL_LIGHTING);
     glEnable(GL_POLYGON_OFFSET_FILL); // always!
-    int k = 0;
-    while (k < List_Nbr(s->thePolyRep->polygons)){
-      double *points = (double*)List_Pointer(s->thePolyRep->polygons,k);
-      k+= ((int)points[0] + 1);
-      if (points[0] == 3){
-	glBegin(GL_TRIANGLES);
-	double *p1 = (double*)List_Pointer(s->thePolyRep->points_and_normals, 
-					   6*(int)points[1]);
-	double *p2 = (double*)List_Pointer(s->thePolyRep->points_and_normals,
-					   6*(int)points[2]);
-	double *p3 = (double*)List_Pointer(s->thePolyRep->points_and_normals,
-					   6*(int)points[3]);
-	if(CTX.geom.light) glNormal3dv(&p1[3]);
-	glVertex3d(p1[0],p1[1],p1[2]);
-	if(CTX.geom.light) glNormal3dv(&p2[3]);
-	glVertex3d(p2[0],p2[1],p2[2]);
-	if(CTX.geom.light) glNormal3dv(&p3[3]);
-	glVertex3d(p3[0],p3[1],p3[2]);
-	glEnd();
-      }
-      else{
-	glBegin(GL_POLYGON);
-	for(int i = 0; i < (int)points[0]; i++){
-	  double *p = (double*)List_Pointer(s->thePolyRep->points_and_normals,
-					    6*(int)points[i+1]);
-	  if(CTX.geom.light) glNormal3dv(&p[3]);
-	  glVertex3d(p[0],p[1],p[2]);
-	}
-	glEnd();
-      }
+    BDS_GeomEntity *g = THEM->bds->get_geom ( s->Num,2);	
+    std::list<BDS_Triangle*>::iterator it  = g->t.begin();
+    std::list<BDS_Triangle*>::iterator ite = g->t.end();
+    while (it!=ite) {
+      glBegin(GL_TRIANGLES);
+      BDS_Point *n[3];
+      BDS_Triangle *t = (*it);
+      double c[3];
+      t->getNodes (n);
+      normal_triangle (n[0],n[1],n[2],c);
+      if(CTX.geom.light) glNormal3dv(c);
+      glVertex3d(n[0]->X,n[0]->Y,n[0]->Z);
+      if(CTX.geom.light) glNormal3dv(c);
+      glVertex3d(n[1]->X,n[1]->Y,n[1]->Z);
+      if(CTX.geom.light) glNormal3dv(c);
+      glVertex3d(n[2]->X,n[2]->Y,n[2]->Z);
+      glEnd();
+      ++it;
     }
     glDisable(GL_POLYGON_OFFSET_FILL);
     glDisable(GL_LIGHTING);
@@ -381,7 +384,7 @@ void Draw_Plane_Surface(Surface * s)
     for(int i = 0; i < List_Nbr(s->Generatrices); i++) {
       List_Read(s->Generatrices, i, &c);
       for(int j = 0; j < List_Nbr(c->Control_Points); j++) {
-        List_Add(points, List_Pointer(c->Control_Points, j));
+	List_Add(points, List_Pointer(c->Control_Points, j));
       }
     }
 
@@ -404,8 +407,8 @@ void Draw_Plane_Surface(Surface * s)
       Projette(&P2, s->plan);
       Projette(&P3, s->plan);
       if(!i) {
-        minx = maxx = P1.Pos.X;
-        miny = maxy = P1.Pos.Y;
+	minx = maxx = P1.Pos.X;
+	miny = maxy = P1.Pos.Y;
       }
       minx = DMIN(DMIN(DMIN(minx, P1.Pos.X), P2.Pos.X), P3.Pos.X);
       miny = DMIN(DMIN(DMIN(miny, P1.Pos.Y), P2.Pos.Y), P3.Pos.Y);
@@ -505,8 +508,8 @@ void Draw_Plane_Surface(Surface * s)
       sprintf(Num, "%d", s->Num);
       double offset = 0.3 * CTX.gl_fontsize * CTX.pixel_equiv_x;
       glRasterPos3d((vv2.Pos.X + vv1.Pos.X) / 2. + offset / CTX.s[0],
-                    (vv2.Pos.Y + vv1.Pos.Y) / 2. + offset / CTX.s[1],
-                    (vv2.Pos.Z + vv1.Pos.Z) / 2. + offset / CTX.s[2]);
+		    (vv2.Pos.Y + vv1.Pos.Y) / 2. + offset / CTX.s[1],
+		    (vv2.Pos.Z + vv1.Pos.Z) / 2. + offset / CTX.s[2]);
       Draw_String(Num);
     }
 
@@ -580,8 +583,8 @@ void Draw_NonPlane_Surface(Surface * s)
     sprintf(Num, "%d", s->Num);
     double offset = 0.3 * CTX.gl_fontsize * CTX.pixel_equiv_x;
     glRasterPos3d(v.Pos.X + offset / CTX.s[0],
-                  v.Pos.Y + offset / CTX.s[1],
-                  v.Pos.Z + offset / CTX.s[2]);
+		  v.Pos.Y + offset / CTX.s[1],
+		  v.Pos.Z + offset / CTX.s[2]);
     Draw_String(Num);
   }
 
@@ -608,7 +611,7 @@ void Draw_Surface(void *a, void *b)
 {
   Surface *s = *(Surface **) a;
 
-  if(!s || !s->Support || !(s->Visible & VIS_GEOM) || s->Dirty)
+  if(!s || !s->Support || !(s->Visible & VIS_GEOM))
     return;
 
   if(CTX.render_mode == GMSH_SELECT) {
@@ -616,7 +619,7 @@ void Draw_Surface(void *a, void *b)
     glPushName(s->Num);
   }
 
-  if(s->ipar[4]) {
+  if(s->ipar[4] > 0) {
     glLineWidth(CTX.geom.line_sel_width / 2.);
     gl2psLineWidth(CTX.geom.line_sel_width / 2. *
 		   CTX.print.eps_line_width_factor);
@@ -628,14 +631,18 @@ void Draw_Surface(void *a, void *b)
     glColor4ubv((GLubyte *) & CTX.color.geom.surface);
   }
 
-  if(s->thePolyRep) // always draw the poly reprentation if available
+  if(THEM->bds){
     Draw_Polygonal_Surface(s);
-  else if(s->Typ == MSH_SURF_DISCRETE)
-    Tree_Action(s->Simplexes, Draw_Mesh_Triangle);
-  else if(s->Typ == MSH_SURF_PLAN)
+  }
+  else if(s->Typ == MSH_SURF_DISCRETE){
+    // do nothing: we draw the elements in the mesh drawing routines
+  }
+  else if(s->Typ == MSH_SURF_PLAN){
     Draw_Plane_Surface(s);
-  else
+  }
+  else{
     Draw_NonPlane_Surface(s);
+  }
 
   if(CTX.render_mode == GMSH_SELECT) {
     glPopName();
@@ -672,16 +679,19 @@ void HighlightEntity(Vertex * v, Curve * c, Surface * s, int permanent)
   Curve *cc;
   char Message[256], temp[256];
 
-  if(permanent){
-    // we want to draw incrementally (in-between to "Draw()" calls):
-    // we need to make sure that the opengl context is set correctly
-    SetOpenglContext();
-  }
+  // Note: in Gmsh < 1.61, we used to draw permanent highlights (the
+  // "red" selected lines, etc.) using incremental drawing, i.e., by
+  // drawing "over" the current picture in-between Draw() calls. This
+  // was fine for simple overlays on points and lines (that we could
+  // draw with a slightly larger width so that they would cover the
+  // old ones), but it does not work well when drawing surfaces,
+  // post-pro views, etc. And since real cross-platform overlays are
+  // unmanageable, the best solution is actually to redraw the whole
+  // scene. This is the approach we follow now.
 
   if(v) {
     if(permanent){
       v->Frozen = 1;
-      Draw_Geo_Point(&v,NULL);
     }
     else{
       Msg(STATUS1N, "Point %d {%.5g,%.5g,%.5g} (%.5g)", v->Num, v->Pos.X,
@@ -690,8 +700,7 @@ void HighlightEntity(Vertex * v, Curve * c, Surface * s, int permanent)
   }
   else if(c) {
     if(permanent){
-      c->ipar[3] = 1;
-      Draw_Curve(&c,NULL);
+      c->ipar[3] = 2;
     }
     else{
       if(c->beg && c->end)
@@ -702,8 +711,7 @@ void HighlightEntity(Vertex * v, Curve * c, Surface * s, int permanent)
   }
   else if(s) {
     if(permanent){
-      s->ipar[4] = 1;
-      Draw_Surface(&s,NULL);
+      s->ipar[4] = 2;
     }
     else{
       int nbg = List_Nbr(s->Generatrices);
@@ -725,6 +733,13 @@ void HighlightEntity(Vertex * v, Curve * c, Surface * s, int permanent)
 	strcat(Message, "}");
 	Msg(STATUS1N, Message);
       }
+      else if (THEM->bds){
+	BDS_GeomEntity *g = THEM->bds->get_geom(s->Num, 2);
+	if (g && g->surf)
+	  Msg(STATUS1N, "Surface %d (%s)", s->Num,g->surf->nameOf().c_str());
+	else
+	  Msg(STATUS1N, "Surface %d (unknown type)", s->Num);
+      }
       else{
 	Msg(STATUS1N, "Surface %d", s->Num);
       }
@@ -734,6 +749,7 @@ void HighlightEntity(Vertex * v, Curve * c, Surface * s, int permanent)
     if(!permanent)
       Msg(STATUS1N, " ");
   }
+
 }
 
 void HighlightEntityNum(int v, int c, int s, int permanent)
@@ -757,23 +773,20 @@ void HighlightEntityNum(int v, int c, int s, int permanent)
 
 void ZeroHighlightPoint(void *a, void *b)
 {
-  Vertex *v;
-  v = *(Vertex **) a;
+  Vertex *v = *(Vertex **) a;
   v->Frozen = 0;
 }
 
 void ZeroHighlightCurve(void *a, void *b)
 {
-  Curve *c;
-  c = *(Curve **) a;
-  c->ipar[3] = 0;
+  Curve *c = *(Curve **) a;
+  c->ipar[3] = -2;
 }
 
 void ZeroHighlightSurface(void *a, void *b)
 {
-  Surface *s;
-  s = *(Surface **) a;
-  s->ipar[4] = 0;
+  Surface *s = *(Surface **) a;
+  s->ipar[4] = -2;
 }
 
 void ZeroHighlight(Mesh * m)
