@@ -1,4 +1,4 @@
-// $Id: Mesh.cpp,v 1.125 2005/03/15 15:43:31 geuzaine Exp $
+// $Id: Mesh.cpp,v 1.142 2005/11/20 03:58:29 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -47,8 +47,11 @@ extern int quadfaces_pyramid[1][4];
 
 static DrawingColor theColor;
 static int thePhysical = 0;
+static Curve *theCurve = NULL;
 static Surface *theSurface = NULL;
 static Volume *theVolume = NULL;
+static int numLabelsDisplayed = 0;
+static int stepLabelsDisplayed = 1;
 
 void draw_polygon_2d(double r, double g, double b, int n,
                      double *x, double *y, double *z)
@@ -76,11 +79,24 @@ void draw_polygon_2d(double r, double g, double b, int n,
 
 int getFirstPhysical(int type, int num)
 {
-  for(int i = 0; i < List_Nbr(THEM->PhysicalGroups); i++){
-    PhysicalGroup *p = *(PhysicalGroup**)List_Pointer(THEM->PhysicalGroups, i);
-    if(p->Typ == type && List_Query(p->Entities, &num, fcmp_int))
-      return p->Num;
-  }  
+  // If we visualize the mesh by coloring the physical entities, or if
+  // we ask to display physical entity labels on elements, this
+  // routine returns the number of the first physical entity of type
+  // "type" that contains the elementary entity "num"
+  if(CTX.mesh.color_carousel == 2 || CTX.mesh.label_type == 2){
+    static int warn = 1;
+    if(List_Nbr(THEM->PhysicalGroups) > 100 && warn){
+      Msg(WARNING, "There are many physical entities in the mesh (%d)",
+	  List_Nbr(THEM->PhysicalGroups));
+      Msg(WARNING, "You might want to color and/or label the mesh by elementary entity instead");
+      warn = 0;
+    }
+    for(int i = 0; i < List_Nbr(THEM->PhysicalGroups); i++){
+      PhysicalGroup *p = *(PhysicalGroup**)List_Pointer(THEM->PhysicalGroups, i);
+      if(p->Typ == type && List_Query(p->Entities, &num, fcmp_int))
+	return p->Num;
+    }
+  }
   return 0;
 }
 
@@ -111,6 +127,17 @@ double intersectCutPlane(int num, Vertex **v)
   return intersectCutPlane(num, v, &dummy, &dummy);
 }
 
+int getPartition(int index)
+{
+  MeshPartition **part = (MeshPartition**)List_Pointer_Test(THEM->Partitions, index);
+  if(!part)
+    return 0; // OK, no partitions available
+  else if(!(*part)->Visible) 
+    return -1; // partition should be hidden
+  else
+    return (*part)->Num; // partition number
+}
+
 void Draw_Mesh(Mesh * M)
 {
   InitPosition();
@@ -135,11 +162,11 @@ void Draw_Mesh(Mesh * M)
       glDisable((GLenum)(GL_CLIP_PLANE0 + i));
   }
 
-  // if we're in selection mode, we're done
-
-  if(CTX.render_mode == GMSH_SELECT)
-    return;
-
+  // We used to be able to exit here when in SELECT mode. Since we
+  // also allow to select mesh entities now, we cannot anymore. Not
+  // sure what we should do eventually
+  //if(CTX.render_mode == GMSH_SELECT) return;
+  
   // draw the bounding box if we asked for it and we have a geometry
   // or a mesh, or if we are in fast redraw mode and there is no
   // geometry but there is a mesh
@@ -238,13 +265,17 @@ void Draw_Mesh(Mesh * M)
 	glEnable((GLenum)(GL_CLIP_PLANE0 + i));
       else
 	glDisable((GLenum)(GL_CLIP_PLANE0 + i));
-    
+
+    // Dimension 3
+
     if(M->status >= 3 && (CTX.mesh.volumes_faces || CTX.mesh.volumes_edges ||
 			  CTX.mesh.volumes_num || CTX.mesh.points_per_element ||
 			  (CTX.mesh.use_cut_plane && CTX.mesh.cut_plane_as_surface &&
 			   (CTX.mesh.surfaces_edges || CTX.mesh.surfaces_faces)))) {
       Tree_Action(M->Volumes, Draw_Mesh_Volume);
     }
+
+    // Dimension 2
    
     if(M->status >= 2 && (CTX.mesh.surfaces_faces || CTX.mesh.surfaces_edges ||
 			  CTX.mesh.surfaces_num || CTX.mesh.points_per_element ||
@@ -254,21 +285,30 @@ void Draw_Mesh(Mesh * M)
 	Tree_Action(M->Volumes, Draw_Mesh_Extruded_Surfaces);
     }
     
+    // Dimension 1
+
     if(M->status >= 1 && (CTX.mesh.lines || CTX.mesh.lines_num || 
 			  CTX.mesh.points_per_element || CTX.mesh.tangents)) {
       Tree_Action(M->Curves, Draw_Mesh_Curve);
     }
+
+    // Dimension 0
     
     if(M->status >= 0 && !CTX.mesh.points_per_element &&
        (CTX.mesh.points || CTX.mesh.points_num)) {
       Tree_Action(M->Vertices, Draw_Mesh_Point);
     }
+
+    // Done!
+
     CTX.mesh.changed = 0;
 
     for(int i = 0; i < 6; i++)
       glDisable((GLenum)(GL_CLIP_PLANE0 + i));
   }
 
+  if(CTX.render_mode == GMSH_SELECT) return;
+  
   // draw any plugin-specific stuff
   
   if(CTX.post.plugin_draw_function)
@@ -285,9 +325,17 @@ void Draw_Mesh_Volume(void *a, void *b)
   if(!(v->Visible & VIS_MESH))
     return;
 
+  if (CTX.mesh.volumes_num) {
+    int numLabels = Tree_Nbr(v->Simplexes) + Tree_Nbr(v->SimplexesBase) 
+      + Tree_Nbr(v->Hexahedra) + Tree_Nbr(v->Prisms) + Tree_Nbr(v->Pyramids);
+    numLabelsDisplayed = 0;
+    if (CTX.mesh.label_frequency == 0.0) stepLabelsDisplayed = numLabels;
+    else stepLabelsDisplayed = (int)(100.0 / CTX.mesh.label_frequency);
+    if (stepLabelsDisplayed > numLabels) stepLabelsDisplayed = numLabels;
+  }
+
   theVolume = v;
   theColor = v->Color;
-  thePhysical = getFirstPhysical(MSH_PHYSICAL_VOLUME, v->Num);
 
   // we don't use vertex arrays for every volume primitive: only for
   // volume cuts drawn "as surfaces" (using vertex arrays for
@@ -296,6 +344,7 @@ void Draw_Mesh_Volume(void *a, void *b)
      CTX.mesh.vertex_arrays){
     if(CTX.mesh.changed){
       Msg(DEBUG, "regenerate volume mesh vertex arrays");
+      thePhysical = getFirstPhysical(MSH_PHYSICAL_VOLUME, v->Num);
       // triangles
       if(v->TriVertexArray) delete v->TriVertexArray;
       v->TriVertexArray = new VertexArray(3, 1000);
@@ -332,6 +381,7 @@ void Draw_Mesh_Volume(void *a, void *b)
      CTX.mesh.dual || CTX.mesh.volumes_num || CTX.mesh.points_per_element ||
      CTX.mesh.normals){
     Msg(DEBUG, "classic volume data path");
+    thePhysical = getFirstPhysical(MSH_PHYSICAL_VOLUME, v->Num);
     Tree_Action(v->Simplexes, Draw_Mesh_Tetrahedron);
     Tree_Action(v->SimplexesBase, Draw_Mesh_Tetrahedron);
     Tree_Action(v->Hexahedra, Draw_Mesh_Hexahedron);
@@ -348,9 +398,22 @@ void Draw_Mesh_Surface(void *a, void *b)
   if(!(s->Visible & VIS_MESH))
     return;
 
+  if(CTX.render_mode == GMSH_SELECT) {
+    glLoadName(2);
+    glPushName(s->Num);
+  }
+
+  if (CTX.mesh.surfaces_num) {
+    int numLabels = Tree_Nbr(s->Simplexes) + Tree_Nbr(s->SimplexesBase) 
+      + Tree_Nbr(s->Quadrangles);
+    numLabelsDisplayed = 0;
+    if (CTX.mesh.label_frequency == 0.0) stepLabelsDisplayed = numLabels;
+    else stepLabelsDisplayed = (int)(100.0 / CTX.mesh.label_frequency);
+    if (stepLabelsDisplayed > numLabels) stepLabelsDisplayed = numLabels;
+  }
+
   theSurface = s;
   theColor = s->Color;
-  thePhysical = getFirstPhysical(MSH_PHYSICAL_SURFACE, s->Num);
 
   if(CTX.mesh.changed && CTX.mesh.smooth_normals){
     Msg(DEBUG, "pre-processing smooth normals");
@@ -364,8 +427,12 @@ void Draw_Mesh_Surface(void *a, void *b)
   }
 
   if(CTX.mesh.vertex_arrays){
-    if(CTX.mesh.changed){
+    if(CTX.mesh.changed || s->ipar[4] == 2 || s->ipar[4] == -2){
+      if(s->ipar[4] == 2) s->ipar[4] = 1;
+      if(s->ipar[4] == -2) s->ipar[4] = -1;
+
       Msg(DEBUG, "regenerate surface mesh vertex arrays");
+      thePhysical = getFirstPhysical(MSH_PHYSICAL_SURFACE, s->Num);
       // triangles
       if(s->TriVertexArray) delete s->TriVertexArray;
       s->TriVertexArray = new VertexArray(3, Tree_Nbr(s->Simplexes) + 
@@ -398,6 +465,7 @@ void Draw_Mesh_Surface(void *a, void *b)
   if(!s->TriVertexArray || CTX.mesh.dual || CTX.mesh.surfaces_num ||
      CTX.mesh.points_per_element || CTX.mesh.normals){
     Msg(DEBUG, "classic triangle data path");
+    thePhysical = getFirstPhysical(MSH_PHYSICAL_SURFACE, s->Num);
     Tree_Action(s->Simplexes, Draw_Mesh_Triangle);
     Tree_Action(s->SimplexesBase, Draw_Mesh_Triangle);
   }
@@ -405,9 +473,14 @@ void Draw_Mesh_Surface(void *a, void *b)
   if(!s->QuadVertexArray || CTX.mesh.dual || CTX.mesh.surfaces_num ||
      CTX.mesh.points_per_element || CTX.mesh.normals){
     Msg(DEBUG, "classic quadrangle data path");
+    thePhysical = getFirstPhysical(MSH_PHYSICAL_SURFACE, s->Num);
     Tree_Action(s->Quadrangles, Draw_Mesh_Quadrangle);
   }
 
+  if(CTX.render_mode == GMSH_SELECT) {
+    glPopName();
+  }
+
   theSurface = NULL;
 }
 
@@ -416,6 +489,7 @@ void Draw_Mesh_Extruded_Surfaces(void *a, void *b)
   Volume *v = *(Volume **) a;
   if(!(v->Visible & VIS_MESH))
     return;
+  Tree_Action(v->Lin_Surf, Draw_Mesh_Line);
   Tree_Action(v->Simp_Surf, Draw_Mesh_Triangle);
   Tree_Action(v->Quad_Surf, Draw_Mesh_Quadrangle);
 }
@@ -427,11 +501,58 @@ void Draw_Mesh_Curve(void *a, void *b)
     return;
   if(!(c->Visible & VIS_MESH))
     return;
-  theColor = c->Color;
-  thePhysical = getFirstPhysical(MSH_PHYSICAL_LINE, c->Num);
 
-  Tree_Action(c->Simplexes, Draw_Mesh_Line);
-  Tree_Action(c->SimplexesBase, Draw_Mesh_Line);
+  if(CTX.render_mode == GMSH_SELECT) {
+    glLoadName(1);
+    glPushName(c->Num);
+  }
+
+  if (CTX.mesh.lines_num) {
+    int numLabels = Tree_Nbr(c->Simplexes) + Tree_Nbr(c->SimplexesBase);
+    numLabelsDisplayed = 0;
+    if (CTX.mesh.label_frequency == 0.0) stepLabelsDisplayed = numLabels;
+    else stepLabelsDisplayed = (int)(100.0 / CTX.mesh.label_frequency);
+    if (stepLabelsDisplayed > numLabels) stepLabelsDisplayed = numLabels;
+  }
+
+  theCurve = c;
+  theColor = c->Color;
+
+  if(CTX.mesh.vertex_arrays){
+    if(CTX.mesh.changed || c->ipar[3] == 2 || c->ipar[3] == -2){
+      if(c->ipar[3] == 2) c->ipar[3] = 1;
+      if(c->ipar[3] == -2) c->ipar[3] = -1;
+
+      Msg(DEBUG, "regenerate curve mesh vertex array");
+      thePhysical = getFirstPhysical(MSH_PHYSICAL_LINE, c->Num);
+      if(c->LinVertexArray) delete c->LinVertexArray;
+      c->LinVertexArray = new VertexArray(2, Tree_Nbr(c->Simplexes) + 
+					  Tree_Nbr(c->SimplexesBase));
+      c->LinVertexArray->fill = 1;
+      Tree_Action(c->Simplexes, Draw_Mesh_Line);
+      Tree_Action(c->SimplexesBase, Draw_Mesh_Line);
+      if(c->LinVertexArray){
+	Msg(DEBUG, "%d segments in curve vertex array", c->LinVertexArray->num);
+	c->LinVertexArray->fill = 0;
+      }
+    }
+    if(c->LinVertexArray)
+      Draw_Mesh_Array(c->LinVertexArray);
+  }
+    
+  if(!c->LinVertexArray || CTX.mesh.lines_num ||
+     CTX.mesh.points_per_element || CTX.mesh.tangents){
+    Msg(DEBUG, "classic line data path");
+    thePhysical = getFirstPhysical(MSH_PHYSICAL_LINE, c->Num);
+    Tree_Action(c->Simplexes, Draw_Mesh_Line);
+    Tree_Action(c->SimplexesBase, Draw_Mesh_Line);
+  }
+
+  if(CTX.render_mode == GMSH_SELECT) {
+    glPopName();
+  }
+
+  theCurve = NULL;
 }
 
 void Draw_Mesh_Point(int num, double x, double y, double z, int degree, int visible)
@@ -439,10 +560,13 @@ void Draw_Mesh_Point(int num, double x, double y, double z, int degree, int visi
   if(!(visible & VIS_MESH))
     return;
 
+  unsigned int col;
   if(degree == 2)
-    glColor4ubv((GLubyte *) & CTX.color.mesh.vertex_deg2);
+    col = CTX.color.mesh.vertex_deg2;
   else
-    glColor4ubv((GLubyte *) & CTX.color.mesh.vertex);
+    col = CTX.color.mesh.vertex;
+
+  glColor4ubv((GLubyte *) & col);
 
   if(CTX.mesh.points) {
     if(CTX.mesh.point_type) {
@@ -486,8 +610,8 @@ void Draw_Mesh_Line(void *a, void *b)
   if(!(s->Visible & VIS_MESH))
     return;
 
-  MeshPartition **part = (MeshPartition**)List_Pointer_Test(THEM->Partitions, s->iPart);
-  if(part && !(*part)->Visible)
+  int iPart = getPartition(s->iPart);
+  if(iPart < 0)
     return;
 
   if(CTX.mesh.radius_sup) {
@@ -527,27 +651,53 @@ void Draw_Mesh_Line(void *a, void *b)
 		      s->VSUP[0]->Degree, s->VSUP[0]->Visible);
   }
 
-  if(theColor.type)
-    glColor4ubv((GLubyte *) & theColor.mesh);
+  unsigned int col;
+  if(theCurve && theCurve->ipar[3] > 0)
+    col = CTX.color.geom.line_sel;
+  else if(theColor.type)
+    col = theColor.mesh;
   else if(CTX.mesh.color_carousel == 1)
-    glColor4ubv((GLubyte *) & CTX.color.mesh.carousel[abs(s->iEnt % 10)]);
+    col = CTX.color.mesh.carousel[abs(s->iEnt % 20)];
   else if(CTX.mesh.color_carousel == 2)
-    glColor4ubv((GLubyte *) & CTX.color.mesh.carousel[abs(thePhysical % 10)]);
+    col = CTX.color.mesh.carousel[abs(thePhysical % 20)];
   else if(CTX.mesh.color_carousel == 3)
-    glColor4ubv((GLubyte *) & CTX.color.mesh.carousel[abs(s->iPart % 10)]);
+    col = CTX.color.mesh.carousel[abs(iPart % 20)];
   else
-    glColor4ubv((GLubyte *) & CTX.color.mesh.line);
+    col = CTX.color.mesh.line;
 
-  if(CTX.mesh.lines) {
-    glBegin(GL_LINE_STRIP);
-    for(int i = 0; i < N; i++){
-      glVertex3d(X[i], Y[i], Z[i]);
+  if(theCurve && theCurve->LinVertexArray && theCurve->LinVertexArray->fill){
+    theCurve->LinVertexArray->add(X[0], Y[0], Z[0], col);
+    theCurve->LinVertexArray->add(X[1], Y[1], Z[1], col);
+    theCurve->LinVertexArray->num++;
+    if(N == 3){
+      theCurve->LinVertexArray->add(X[1], Y[1], Z[1], col);
+      theCurve->LinVertexArray->add(X[2], Y[2], Z[2], col);
+      theCurve->LinVertexArray->num++;
+    }
+  }
+  else{
+    if(CTX.mesh.lines) {
+      glColor4ubv((GLubyte *) & col);
+      glBegin(GL_LINE_STRIP);
+      for(int i = 0; i < N; i++){
+	glVertex3d(X[i], Y[i], Z[i]);
+      }
+      glEnd();
     }
-    glEnd();
   }
 
-  if(CTX.mesh.lines_num) {
-    sprintf(Num, "%d", s->Num);
+  ++numLabelsDisplayed;
+
+  if(CTX.mesh.lines_num && (numLabelsDisplayed % stepLabelsDisplayed == 0)) {
+    glColor4ubv((GLubyte *) & col);
+    if(CTX.mesh.label_type == 3)
+      sprintf(Num, "%d", iPart);
+    else if(CTX.mesh.label_type == 2)
+      sprintf(Num, "%d", thePhysical);
+    else if(CTX.mesh.label_type == 1)
+      sprintf(Num, "%d", s->iEnt);
+    else
+      sprintf(Num, "%d", s->Num);
     double offset = 0.3 * (CTX.mesh.line_width + CTX.gl_fontsize) * CTX.pixel_equiv_x;
     glRasterPos3d(Xc + offset / CTX.s[0],
                   Yc + offset / CTX.s[1],
@@ -690,40 +840,59 @@ void _quadFace2(double *x, double *y, double *z,
   _triFace(x2[j3], y2[j3], z2[j3], x[i0], y[i0], z[i0], x2[j4], y2[j4], z2[j4]);
 }
 
+// #include <GL/glext.h>
+
 void Draw_Mesh_Array(VertexArray *va, int faces, int edges)
 {
   if(!va->num)
     return;
 
   glVertexPointer(3, GL_FLOAT, 0, va->vertices->array);
-  glNormalPointer(GL_FLOAT, 0, va->normals->array);
   glColorPointer(4, GL_UNSIGNED_BYTE, 0, va->colors->array);
+  glNormalPointer(GL_FLOAT, 0, va->normals->array);
 
   glEnableClientState(GL_VERTEX_ARRAY);
   glEnableClientState(GL_COLOR_ARRAY);
   glEnableClientState(GL_NORMAL_ARRAY);
 
-  if(faces){
-    if(CTX.mesh.light)
-      glEnable(GL_LIGHTING);
-    else
-      glDisableClientState(GL_NORMAL_ARRAY);
-    if(CTX.polygon_offset) glEnable(GL_POLYGON_OFFSET_FILL);
-    glDrawArrays((va->type == 3) ? GL_TRIANGLES : GL_QUADS, 0, va->type * va->num);
-    glDisable(GL_POLYGON_OFFSET_FILL);
-    glDisable(GL_LIGHTING);
-  }
-      
-  if(edges){
-    if(faces){
-      glDisableClientState(GL_COLOR_ARRAY);
-      glColor4ubv((GLubyte *) & CTX.color.mesh.line);
-    }
+  // glLockArraysEXT(0, va->type * va->num); // extension GL_EXT_compiled_vertex_array
+
+  if(va->type == 1){
     glDisableClientState(GL_NORMAL_ARRAY);
-    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
-    glDrawArrays((va->type == 3) ? GL_TRIANGLES : GL_QUADS, 0, va->type * va->num);
-    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
+    glDrawArrays(GL_POINTS, 0, va->type * va->num);
   }
+  else if(va->type == 2){
+    glDisableClientState(GL_NORMAL_ARRAY);
+    glDrawArrays(GL_LINES, 0, va->type * va->num);
+  }
+  else{
+    if(faces){
+      if(CTX.mesh.light)
+	glEnable(GL_LIGHTING);
+      else
+	glDisableClientState(GL_NORMAL_ARRAY);
+      if(CTX.polygon_offset) glEnable(GL_POLYGON_OFFSET_FILL);
+      glDrawArrays((va->type == 3) ? GL_TRIANGLES : GL_QUADS, 0, va->type * va->num);
+      glDisable(GL_POLYGON_OFFSET_FILL);
+      glDisable(GL_LIGHTING);
+    }
+    if(edges){
+      if(faces){
+	glDisableClientState(GL_COLOR_ARRAY);
+	glColor4ubv((GLubyte *) & CTX.color.mesh.line);
+      }
+      if(CTX.mesh.light && CTX.mesh.light_lines)
+	glEnable(GL_LIGHTING);
+      else
+	glDisableClientState(GL_NORMAL_ARRAY);
+      glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
+      glDrawArrays((va->type == 3) ? GL_TRIANGLES : GL_QUADS, 0, va->type * va->num);
+      glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
+      glDisable(GL_LIGHTING);
+    }
+  }
+
+  // glUnlockArraysEXT(); // extension GL_EXT_compiled_vertex_array
 
   glDisableClientState(GL_VERTEX_ARRAY);
   glDisableClientState(GL_COLOR_ARRAY);
@@ -738,16 +907,23 @@ void Draw_Mesh_Triangle(void *a, void *b)
 
   SimplexBase *s = *(SimplexBase **) a;
 
-  if(!s->V[2]) // the old extrusion algo puts lines in the simp_surf tree...
-    return;
-
   if(!(s->Visible & VIS_MESH))
     return;
 
-  MeshPartition **part = (MeshPartition**)List_Pointer_Test(THEM->Partitions, s->iPart);
-  if(part && !(*part)->Visible)
+  int iPart = getPartition(s->iPart);
+  if(iPart < 0)
     return;
 
+  if(CTX.mesh.quality_sup) {
+    double tmp;
+    if(CTX.mesh.quality_type == 2)
+      tmp = s->RhoShapeMeasure();
+    else
+      tmp = 0.0;
+    if(tmp < CTX.mesh.quality_inf || tmp > CTX.mesh.quality_sup)
+      return;
+  }
+
   if(CTX.mesh.radius_sup) {
     double r = s->maxEdge();
     if(r < CTX.mesh.radius_inf || r > CTX.mesh.radius_sup)
@@ -758,14 +934,16 @@ void Draw_Mesh_Triangle(void *a, void *b)
     return;
 
   unsigned int col;
-  if(theColor.type)
+  if(theSurface && theSurface->ipar[4] > 0)
+    col = CTX.color.geom.surface_sel;
+  else if(theColor.type)
     col = theColor.mesh;
   else if(CTX.mesh.color_carousel == 1)
-    col = CTX.color.mesh.carousel[abs(s->iEnt % 10)];
+    col = CTX.color.mesh.carousel[abs(s->iEnt % 20)];
   else if(CTX.mesh.color_carousel == 2)
-    col = CTX.color.mesh.carousel[abs(thePhysical % 10)];
+    col = CTX.color.mesh.carousel[abs(thePhysical % 20)];
   else if(CTX.mesh.color_carousel == 3)
-    col = CTX.color.mesh.carousel[abs(s->iPart % 10)];
+    col = CTX.color.mesh.carousel[abs(iPart % 20)];
   else
     col = CTX.color.mesh.triangle;
 
@@ -825,12 +1003,19 @@ void Draw_Mesh_Triangle(void *a, void *b)
 	glColor4ubv((GLubyte *) & CTX.color.mesh.line);
       else
 	glColor4ubv((GLubyte *) & col);
+      if(CTX.mesh.light && CTX.mesh.light_lines){
+	// FIXME: very crude (not smooth, only 1st order)
+	glEnable(GL_LIGHTING);
+	normal3points(X[0], Y[0], Z[0], X[1], Y[1], Z[1], X[2], Y[2], Z[2], n);
+	glNormal3dv(n);
+      }
       glBegin(GL_LINE_LOOP);
       for(int i = 0; i < 3; i++){
 	glVertex3d(X[i], Y[i], Z[i]);
 	if(s->VSUP) glVertex3d(X2[i], Y2[i], Z2[i]);
       }
       glEnd();
+      glDisable(GL_LIGHTING);
     }
 
     if(CTX.mesh.surfaces_faces) {
@@ -868,12 +1053,21 @@ void Draw_Mesh_Triangle(void *a, void *b)
     gl2psDisable(GL2PS_LINE_STIPPLE);
   }
 
-  if(CTX.mesh.surfaces_num) {
+  ++numLabelsDisplayed;
+
+  if(CTX.mesh.surfaces_num && (numLabelsDisplayed % stepLabelsDisplayed == 0)) {
     if(CTX.mesh.surfaces_faces)
       glColor4ubv((GLubyte *) & CTX.color.mesh.line);
     else
       glColor4ubv((GLubyte *) & col);
-    sprintf(Num, "%d", s->Num);
+    if(CTX.mesh.label_type == 3)
+      sprintf(Num, "%d", iPart);
+    else if(CTX.mesh.label_type == 2)
+      sprintf(Num, "%d", thePhysical);
+    else if(CTX.mesh.label_type == 1)
+      sprintf(Num, "%d", s->iEnt);
+    else
+      sprintf(Num, "%d", s->Num);
     glRasterPos3d(Xc, Yc, Zc);
     Draw_String(Num);
   }
@@ -903,10 +1097,20 @@ void Draw_Mesh_Quadrangle(void *a, void *b)
   if(!(q->Visible & VIS_MESH))
     return;
 
-  MeshPartition **part = (MeshPartition**)List_Pointer_Test(THEM->Partitions, q->iPart);
-  if(part && !(*part)->Visible)
+  int iPart = getPartition(q->iPart);
+  if(iPart < 0)
     return;
 
+  if(CTX.mesh.quality_sup) {
+    double tmp;
+    if(CTX.mesh.quality_type == 2)
+      tmp = q->RhoShapeMeasure();
+    else
+      tmp = 0.0;
+    if(tmp < CTX.mesh.quality_inf || tmp > CTX.mesh.quality_sup)
+      return;
+  }
+
   if(CTX.mesh.radius_sup) {
     double r = q->maxEdge();
     if(r < CTX.mesh.radius_inf || r > CTX.mesh.radius_sup)
@@ -917,14 +1121,16 @@ void Draw_Mesh_Quadrangle(void *a, void *b)
     return;
 
   unsigned int col;
-  if(theColor.type)
+  if(theSurface && theSurface->ipar[4] > 0)
+    col = CTX.color.geom.surface_sel;
+  else if(theColor.type)
     col = theColor.mesh;
   else if(CTX.mesh.color_carousel == 1)
-    col = CTX.color.mesh.carousel[abs(q->iEnt % 10)];
+    col = CTX.color.mesh.carousel[abs(q->iEnt % 20)];
   else if(CTX.mesh.color_carousel == 2)
-    col = CTX.color.mesh.carousel[abs(thePhysical % 10)];
+    col = CTX.color.mesh.carousel[abs(thePhysical % 20)];
   else if(CTX.mesh.color_carousel == 3)
-    col = CTX.color.mesh.carousel[abs(q->iPart % 10)];
+    col = CTX.color.mesh.carousel[abs(iPart % 20)];
   else
     col = CTX.color.mesh.quadrangle;
 
@@ -987,12 +1193,19 @@ void Draw_Mesh_Quadrangle(void *a, void *b)
 	glColor4ubv((GLubyte *) & CTX.color.mesh.line);
       else
 	glColor4ubv((GLubyte *) & col);
+      if(CTX.mesh.light && CTX.mesh.light_lines){
+	// FIXME: very crude (not smooth, only 1st order)
+	glEnable(GL_LIGHTING);
+	normal3points(X[0], Y[0], Z[0], X[1], Y[1], Z[1], X[2], Y[2], Z[2], n);
+	glNormal3dv(n);
+      }
       glBegin(GL_LINE_LOOP);
       for(int i = 0; i < 4; i++){
 	glVertex3d(X[i], Y[i], Z[i]);
 	if(q->VSUP) glVertex3d(X2[i], Y2[i], Z2[i]);
       }
       glEnd();
+      glDisable(GL_LIGHTING);
     }
 
     if(CTX.mesh.surfaces_faces) {
@@ -1030,12 +1243,21 @@ void Draw_Mesh_Quadrangle(void *a, void *b)
     gl2psDisable(GL2PS_LINE_STIPPLE);
   }
 
-  if(CTX.mesh.surfaces_num) {
+  ++numLabelsDisplayed;
+
+  if(CTX.mesh.surfaces_num && (numLabelsDisplayed % stepLabelsDisplayed == 0)) {
     if(CTX.mesh.surfaces_faces)
       glColor4ubv((GLubyte *) & CTX.color.mesh.line);
     else
       glColor4ubv((GLubyte *) & col);
-    sprintf(Num, "%d", q->Num);
+    if(CTX.mesh.label_type == 3)
+      sprintf(Num, "%d", iPart);
+    else if(CTX.mesh.label_type == 2)
+      sprintf(Num, "%d", thePhysical);
+    else if(CTX.mesh.label_type == 1)
+      sprintf(Num, "%d", q->iEnt);
+    else
+      sprintf(Num, "%d", q->Num);
     glRasterPos3d(Xc, Yc, Zc);
     Draw_String(Num);
   }
@@ -1057,20 +1279,26 @@ void Draw_Mesh_Quadrangle(void *a, void *b)
 void Draw_Mesh_Tetrahedron(void *a, void *b)
 {
   char Num[100];
-  double tmp, X[4], Y[4], Z[4], X2[6], Y2[6], Z2[6];
+  double X[4], Y[4], Z[4], X2[6], Y2[6], Z2[6];
 
   SimplexBase *s = *(SimplexBase **) a;
 
   if(!s->V[3] || !(s->Visible & VIS_MESH))
     return;
 
-  MeshPartition **part = (MeshPartition**)List_Pointer_Test(THEM->Partitions, s->iPart);
-  if(part && !(*part)->Visible)
+  int iPart = getPartition(s->iPart);
+  if(iPart < 0)
     return;
 
-  if(CTX.mesh.gamma_sup) {
-    tmp = s->GammaShapeMeasure();
-    if(tmp < CTX.mesh.gamma_inf || tmp > CTX.mesh.gamma_sup)
+  if(CTX.mesh.quality_sup) {
+    double tmp;
+    if(CTX.mesh.quality_type == 2)
+      tmp = s->RhoShapeMeasure();
+    else if(CTX.mesh.quality_type == 1)
+      tmp = s->EtaShapeMeasure();
+    else
+      tmp = s->GammaShapeMeasure();
+    if(tmp < CTX.mesh.quality_inf || tmp > CTX.mesh.quality_sup)
       return;
   }
 
@@ -1090,11 +1318,11 @@ void Draw_Mesh_Tetrahedron(void *a, void *b)
   if(theColor.type)
     col = theColor.mesh;
   else if(CTX.mesh.color_carousel == 1)
-    col = CTX.color.mesh.carousel[abs(s->iEnt % 10)];
+    col = CTX.color.mesh.carousel[abs(s->iEnt % 20)];
   else if(CTX.mesh.color_carousel == 2)
-    col = CTX.color.mesh.carousel[abs(thePhysical % 10)];
+    col = CTX.color.mesh.carousel[abs(thePhysical % 20)];
   else if(CTX.mesh.color_carousel == 3)
-    col = CTX.color.mesh.carousel[abs(s->iPart % 10)];
+    col = CTX.color.mesh.carousel[abs(iPart % 20)];
   else
     col = CTX.color.mesh.tetrahedron;
 
@@ -1215,12 +1443,21 @@ void Draw_Mesh_Tetrahedron(void *a, void *b)
     gl2psDisable(GL2PS_LINE_STIPPLE);
   }
 
-  if(CTX.mesh.volumes_num) {
+  ++numLabelsDisplayed;
+
+  if(CTX.mesh.volumes_num && (numLabelsDisplayed % stepLabelsDisplayed == 0)) {
     if(CTX.mesh.surfaces_faces || faces)
       glColor4ubv((GLubyte *) & CTX.color.mesh.line);
     else
       glColor4ubv((GLubyte *) & col);
-    sprintf(Num, "%d", s->Num);
+    if(CTX.mesh.label_type == 3)
+      sprintf(Num, "%d", iPart);
+    else if(CTX.mesh.label_type == 2)
+      sprintf(Num, "%d", thePhysical);
+    else if(CTX.mesh.label_type == 1)
+      sprintf(Num, "%d", s->iEnt);
+    else
+      sprintf(Num, "%d", s->Num);
     glRasterPos3d(Xc, Yc, Zc);
     Draw_String(Num);
   }
@@ -1237,10 +1474,20 @@ void Draw_Mesh_Hexahedron(void *a, void *b)
   if(!(h->Visible & VIS_MESH))
     return;
 
-  MeshPartition **part = (MeshPartition**)List_Pointer_Test(THEM->Partitions, h->iPart);
-  if(part && !(*part)->Visible)
+  int iPart = getPartition(h->iPart);
+  if(iPart < 0)
     return;
 
+  if(CTX.mesh.quality_sup) {
+    double tmp;
+    if(CTX.mesh.quality_type == 2)
+      tmp = h->RhoShapeMeasure();
+    else
+      tmp = 0.0;
+    if(tmp < CTX.mesh.quality_inf || tmp > CTX.mesh.quality_sup)
+      return;
+  }
+
   if(CTX.mesh.radius_sup) {
     double r = h->maxEdge();
     if(r < CTX.mesh.radius_inf || r > CTX.mesh.radius_sup)
@@ -1257,11 +1504,11 @@ void Draw_Mesh_Hexahedron(void *a, void *b)
   if(theColor.type)
     col = theColor.mesh;
   else if(CTX.mesh.color_carousel == 1)
-    col = CTX.color.mesh.carousel[abs(h->iEnt % 10)];
+    col = CTX.color.mesh.carousel[abs(h->iEnt % 20)];
   else if(CTX.mesh.color_carousel == 2)
-    col = CTX.color.mesh.carousel[abs(thePhysical % 10)];
+    col = CTX.color.mesh.carousel[abs(thePhysical % 20)];
   else if(CTX.mesh.color_carousel == 3)
-    col = CTX.color.mesh.carousel[abs(h->iPart % 10)];
+    col = CTX.color.mesh.carousel[abs(iPart % 20)];
   else
     col = CTX.color.mesh.hexahedron;
 
@@ -1396,12 +1643,21 @@ void Draw_Mesh_Hexahedron(void *a, void *b)
     gl2psDisable(GL2PS_LINE_STIPPLE);
   }
 
-  if(CTX.mesh.volumes_num) {
+  ++numLabelsDisplayed;
+
+  if(CTX.mesh.volumes_num && (numLabelsDisplayed % stepLabelsDisplayed == 0)) {
     if(CTX.mesh.surfaces_faces || faces)
       glColor4ubv((GLubyte *) & CTX.color.mesh.line);
     else
       glColor4ubv((GLubyte *) & col);
-    sprintf(Num, "%d", h->Num);
+    if(CTX.mesh.label_type == 3)
+      sprintf(Num, "%d", iPart);
+    else if(CTX.mesh.label_type == 2)
+      sprintf(Num, "%d", thePhysical);
+    else if(CTX.mesh.label_type == 1)
+      sprintf(Num, "%d", h->iEnt);
+    else
+      sprintf(Num, "%d", h->Num);
     glRasterPos3d(Xc, Yc, Zc);
     Draw_String(Num);
   }
@@ -1418,10 +1674,20 @@ void Draw_Mesh_Prism(void *a, void *b)
   if(!(p->Visible & VIS_MESH))
     return;
 
-  MeshPartition **part = (MeshPartition**)List_Pointer_Test(THEM->Partitions, p->iPart);
-  if(part && !(*part)->Visible)
+  int iPart = getPartition(p->iPart);
+  if(iPart < 0)
     return;
 
+  if(CTX.mesh.quality_sup) {
+    double tmp;
+    if(CTX.mesh.quality_type == 2)
+      tmp = p->RhoShapeMeasure();
+    else
+      tmp = 0.0;
+    if(tmp < CTX.mesh.quality_inf || tmp > CTX.mesh.quality_sup)
+      return;
+  }
+
   if(CTX.mesh.radius_sup) {
     double r = p->maxEdge();
     if(r < CTX.mesh.radius_inf || r > CTX.mesh.radius_sup)
@@ -1438,11 +1704,11 @@ void Draw_Mesh_Prism(void *a, void *b)
   if(theColor.type)
     col = theColor.mesh;
   else if(CTX.mesh.color_carousel == 1)
-    col = CTX.color.mesh.carousel[abs(p->iEnt % 10)];
+    col = CTX.color.mesh.carousel[abs(p->iEnt % 20)];
   else if(CTX.mesh.color_carousel == 2)
-    col = CTX.color.mesh.carousel[abs(thePhysical % 10)];
+    col = CTX.color.mesh.carousel[abs(thePhysical % 20)];
   else if(CTX.mesh.color_carousel == 3)
-    col = CTX.color.mesh.carousel[abs(p->iPart % 10)];
+    col = CTX.color.mesh.carousel[abs(iPart % 20)];
   else
     col = CTX.color.mesh.prism;
 
@@ -1592,12 +1858,21 @@ void Draw_Mesh_Prism(void *a, void *b)
     gl2psDisable(GL2PS_LINE_STIPPLE);
   }
 
-  if(CTX.mesh.volumes_num) {
+  ++numLabelsDisplayed;
+
+  if(CTX.mesh.volumes_num && (numLabelsDisplayed % stepLabelsDisplayed == 0)) {
     if(CTX.mesh.surfaces_faces || faces)
       glColor4ubv((GLubyte *) & CTX.color.mesh.line);
     else
       glColor4ubv((GLubyte *) & col);
-    sprintf(Num, "%d", p->Num);
+    if(CTX.mesh.label_type == 3)
+      sprintf(Num, "%d", iPart);
+    else if(CTX.mesh.label_type == 2)
+      sprintf(Num, "%d", thePhysical);
+    else if(CTX.mesh.label_type == 1)
+      sprintf(Num, "%d", p->iEnt);
+    else
+      sprintf(Num, "%d", p->Num);
     glRasterPos3d(Xc, Yc, Zc);
     Draw_String(Num);
   }
@@ -1614,10 +1889,20 @@ void Draw_Mesh_Pyramid(void *a, void *b)
   if(!(p->Visible & VIS_MESH))
     return;
 
-  MeshPartition **part = (MeshPartition**)List_Pointer_Test(THEM->Partitions, p->iPart);
-  if(part && !(*part)->Visible)
+  int iPart = getPartition(p->iPart);
+  if(iPart < 0)
     return;
 
+  if(CTX.mesh.quality_sup) {
+    double tmp;
+    if(CTX.mesh.quality_type == 2)
+      tmp = p->RhoShapeMeasure();
+    else
+      tmp = 0.0;
+    if(tmp < CTX.mesh.quality_inf || tmp > CTX.mesh.quality_sup)
+      return;
+  }
+
   if(CTX.mesh.radius_sup) {
     double r = p->maxEdge();
     if(r < CTX.mesh.radius_inf || r > CTX.mesh.radius_sup)
@@ -1634,11 +1919,11 @@ void Draw_Mesh_Pyramid(void *a, void *b)
   if(theColor.type)
     col = theColor.mesh;
   else if(CTX.mesh.color_carousel == 1)
-    col = CTX.color.mesh.carousel[abs(p->iEnt % 10)];
+    col = CTX.color.mesh.carousel[abs(p->iEnt % 20)];
   else if(CTX.mesh.color_carousel == 2)
-    col = CTX.color.mesh.carousel[abs(thePhysical % 10)];
+    col = CTX.color.mesh.carousel[abs(thePhysical % 20)];
   else if(CTX.mesh.color_carousel == 3)
-    col = CTX.color.mesh.carousel[abs(p->iPart % 10)];
+    col = CTX.color.mesh.carousel[abs(iPart % 20)];
   else
     col = CTX.color.mesh.pyramid;
 
@@ -1760,12 +2045,21 @@ void Draw_Mesh_Pyramid(void *a, void *b)
     }
   }
 
-  if(CTX.mesh.volumes_num) {
+  ++numLabelsDisplayed;
+
+  if(CTX.mesh.volumes_num && (numLabelsDisplayed % stepLabelsDisplayed == 0)) {
     if(CTX.mesh.surfaces_faces || faces)
       glColor4ubv((GLubyte *) & CTX.color.mesh.line);
     else
       glColor4ubv((GLubyte *) & col);
-    sprintf(Num, "%d", p->Num);
+    if(CTX.mesh.label_type == 3)
+      sprintf(Num, "%d", iPart);
+    else if(CTX.mesh.label_type == 2)
+      sprintf(Num, "%d", thePhysical);
+    else if(CTX.mesh.label_type == 1)
+      sprintf(Num, "%d", p->iEnt);
+    else
+      sprintf(Num, "%d", p->Num);
     glRasterPos3d(Xc, Yc, Zc);
     Draw_String(Num);
   }
