@@ -1,4 +1,4 @@
-// $Id: Post.cpp,v 1.99 2005/03/14 18:55:22 geuzaine Exp $
+// $Id: Post.cpp,v 1.102 2005/09/02 18:04:02 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -430,6 +430,28 @@ void Draw_List(Post_View * v, double ValMin, double ValMax, int type,
   }
 }
 
+// We try to estimate how many primitives will end up in the vertex
+// arrays, since reallocating the arrays takes a HUGE amount of time
+// on Windows/Cygwin. A better way would be to slightly redesign the
+// drawing routines so we can have different pre-processing steps
+// (like the one we have for smooth normals right now), in order to
+// count how many primitives we will have.
+
+int EstimateNumTri(Post_View *v)
+{
+  int num2d = v->NbST + v->NbSQ;
+  int num3d = v->NbSS + v->NbSH + v->NbSI + v->NbSY;
+
+  int heuristic = num2d + num3d/10;
+
+  return (heuristic < 10000) ? 10000 : heuristic;
+}
+
+int EstimateNumLin(Post_View *v)
+{
+  return 10000;
+}
+
 void Draw_Post(void)
 {
   int nb;
@@ -646,8 +668,11 @@ void Draw_Post(void)
 	    if(v->Changed){
 	      Msg(DEBUG, "regenerate View[%d] vertex array", v->Index);
 	      if(v->TriVertexArray) delete v->TriVertexArray;
-	      v->TriVertexArray = new VertexArray(3, 10000);
+	      v->TriVertexArray = new VertexArray(3, EstimateNumTri(v));
 	      v->TriVertexArray->fill = 1;
+	      if(v->LinVertexArray) delete v->LinVertexArray;
+	      v->LinVertexArray = new VertexArray(2, EstimateNumLin(v));
+	      v->LinVertexArray->fill = 1;
 	      goto pass_0;
 	    }
 	  }
@@ -655,14 +680,16 @@ void Draw_Post(void)
 	}
 	else{
 	  // don't even enter the classic data path if we don't have to
-	  if(v->TriVertexArray){
-	    if(v->Boundary < 1 && !v->ShowElement && !v->Normals &&
-	       v->IntervalsType != DRAW_POST_NUMERIC && v->IntervalsType != DRAW_POST_ISO){
+	  if(v->TriVertexArray || v->LinVertexArray){
+	    if(v->Boundary < 1 && !v->ShowElement && !v->Normals && 
+	       (!v->LineType || (v->LineType && v->IntervalsType != DRAW_POST_ISO)) &&
+	       v->IntervalsType != DRAW_POST_NUMERIC){
 	      Msg(DEBUG, "View[%d]: skiping 2D scalar pass alltogether", v->Index);
 	      skip_2d = 1;
 	    }
 	    if(v->Boundary < 2 && !v->ShowElement &&
-	       v->IntervalsType != DRAW_POST_NUMERIC && v->IntervalsType != DRAW_POST_ISO){
+	       (!v->LineType || (v->LineType && v->IntervalsType != DRAW_POST_ISO)) &&
+	       v->IntervalsType != DRAW_POST_NUMERIC){
 	      Msg(DEBUG, "View[%d]: skiping 3D scalar pass alltogether", v->Index);
 	      skip_3d = 1;
 	    }
@@ -742,10 +769,13 @@ void Draw_Post(void)
 	  Msg(DEBUG, "View[%d]; %d tris in vertex array", v->Index, v->TriVertexArray->num);
 	  v->TriVertexArray->fill = 0;
 	}
+	if(v->LinVertexArray && v->LinVertexArray->fill){
+	  Msg(DEBUG, "View[%d]; %d segs in vertex array", v->Index, v->LinVertexArray->num);
+	  v->LinVertexArray->fill = 0;
+	}
       }
 
       if(v->TriVertexArray && v->TriVertexArray->num){
-
 	if(CTX.alpha && ColorTable_IsAlpha(&v->CT) && !v->FakeTransparency &&
 	   (changedEye() || v->Changed)){
 	  Msg(DEBUG, "Sorting View[%d] for transparency (WITH vertex array)", v->Index);
@@ -774,6 +804,16 @@ void Draw_Post(void)
 	glDisableClientState(GL_NORMAL_ARRAY);
       }
 
+      if(v->LinVertexArray && v->LinVertexArray->num){
+	glVertexPointer(3, GL_FLOAT, 0, v->LinVertexArray->vertices->array);
+	glColorPointer(4, GL_UNSIGNED_BYTE, 0, v->LinVertexArray->colors->array);
+	glEnableClientState(GL_VERTEX_ARRAY);
+	glEnableClientState(GL_COLOR_ARRAY);
+	glDrawArrays(GL_LINES, 0, 2 * v->LinVertexArray->num);
+	glDisableClientState(GL_VERTEX_ARRAY);
+	glDisableClientState(GL_COLOR_ARRAY);
+      }
+
       if(v->DrawStrings) {
 	glColor4ubv((GLubyte *) & v->color.text3d);
 	Draw_Text2D3D(3, v->TimeStep, v->NbT3, v->T3D, v->T3C);
