@@ -1,4 +1,4 @@
-// $Id: PostElement.cpp,v 1.63 2005/03/14 21:19:32 geuzaine Exp $
+// $Id: PostElement.cpp,v 1.69 2005/11/20 03:58:29 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -35,8 +35,13 @@
 extern Context_T CTX;
 
 void Draw_ElementBoundary(int type, Post_View * View, 
-			  double *X, double *Y, double *Z)
+			  double *X, double *Y, double *Z, double *n = 0)
 {
+  if(n && View->Light && View->LightLines){
+    glEnable(GL_LIGHTING);
+    glNormal3dv(n);
+  }
+
   switch (type) {
   case POST_POINT:
     glColor4ubv((GLubyte *) & View->color.point);
@@ -151,6 +156,8 @@ void Draw_ElementBoundary(int type, Post_View * View,
     glEnd();
     break;
   }
+
+  glDisable(GL_LIGHTING);
 }
 
 void SaturateValues(int saturate, double min, double max,
@@ -276,12 +283,24 @@ void Draw_ScalarLine(Post_View * View, int preproNormals,
   }
   
   if(View->IntervalsType == DRAW_POST_NUMERIC) {
-    d = (Val[0] + Val[1]) / 2.;
-    if(d >= ValMin && d <= ValMax) {
-      PaletteContinuous(View, ValMin, ValMax, d);
-      sprintf(Num, View->Format, d);
-      glRasterPos3d((X[0] + X[1]) / 2., (Y[0] + Y[1]) / 2., (Z[0] + Z[1]) / 2.);
-      Draw_String(Num);
+    if(View->GlyphLocation == DRAW_POST_LOCATE_COG){
+      d = (Val[0] + Val[1]) / 2.;
+      if(d >= ValMin && d <= ValMax) {
+	PaletteContinuous(View, ValMin, ValMax, d);
+	sprintf(Num, View->Format, d);
+	glRasterPos3d((X[0] + X[1]) / 2., (Y[0] + Y[1]) / 2., (Z[0] + Z[1]) / 2.);
+	Draw_String(Num);
+      }
+    }
+    else{
+      for(int i = 0; i < 2; i++){
+	if(Val[i] >= ValMin && Val[i] <= ValMax) {
+	  PaletteContinuous(View, ValMin, ValMax, Val[i]);
+	  sprintf(Num, View->Format, Val[i]);
+	  glRasterPos3d(X[i], Y[i], Z[i]);
+	  Draw_String(Num);
+	}
+      }
     }
   }
 
@@ -370,8 +389,16 @@ void Draw_ScalarTriangle(Post_View * View, int preproNormals,
     normal3points(X[0], Y[0], Z[0], X[1], Y[1], Z[1], X[2], Y[2], Z[2], nn);
   }
 
-  if(!preproNormals && View->ShowElement)
-    Draw_ElementBoundary(POST_TRIANGLE, View, X, Y, Z);
+  if(!preproNormals && View->ShowElement){
+    if(View->Light && View->LightLines){
+      // FIXME: very crude (not smooth) and inefficient
+      double n[3];
+      normal3points(X[0], Y[0], Z[0], X[1], Y[1], Z[1], X[2], Y[2], Z[2], n);
+      Draw_ElementBoundary(POST_TRIANGLE, View, X, Y, Z, n);
+    }
+    else
+      Draw_ElementBoundary(POST_TRIANGLE, View, X, Y, Z);
+  }
 
   if(!preproNormals && View->Normals){
     double t[3] = { nn[0], nn[1], nn[2] };
@@ -388,14 +415,26 @@ void Draw_ScalarTriangle(Post_View * View, int preproNormals,
   }
   
   if(!preproNormals && View->IntervalsType == DRAW_POST_NUMERIC) {
-    d = (Val[0] + Val[1] + Val[2]) / 3.;
-    if(d >= ValMin && d <= ValMax) {
-      PaletteContinuous(View, ValMin, ValMax, d);
-      sprintf(Num, View->Format, d);
-      glRasterPos3d((X[0] + X[1] + X[2]) / 3.,
-                    (Y[0] + Y[1] + Y[2]) / 3.,
-                    (Z[0] + Z[1] + Z[2]) / 3.);
-      Draw_String(Num);
+    if(View->GlyphLocation == DRAW_POST_LOCATE_COG){
+      d = (Val[0] + Val[1] + Val[2]) / 3.;
+      if(d >= ValMin && d <= ValMax) {
+	PaletteContinuous(View, ValMin, ValMax, d);
+	sprintf(Num, View->Format, d);
+	glRasterPos3d((X[0] + X[1] + X[2]) / 3.,
+		      (Y[0] + Y[1] + Y[2]) / 3.,
+		      (Z[0] + Z[1] + Z[2]) / 3.);
+	Draw_String(Num);
+      }
+    }
+    else{
+      for(int i = 0; i < 3; i++){
+	if(Val[i] >= ValMin && Val[i] <= ValMax) {
+	  PaletteContinuous(View, ValMin, ValMax, Val[i]);
+	  sprintf(Num, View->Format, Val[i]);
+	  glRasterPos3d(X[i], Y[i], Z[i]);
+	  Draw_String(Num);
+	}
+      }
     }
   }
 
@@ -551,13 +590,15 @@ void Draw_ScalarTriangle(Post_View * View, int preproNormals,
     }
   }
   
-  if(!preproNormals && View->IntervalsType == DRAW_POST_ISO) {
+  if(!preproNormals && View->IntervalsType == DRAW_POST_ISO &&
+     (!View->LinVertexArray || (View->LinVertexArray && View->LinVertexArray->fill) ||
+      View->LineType)){
     for(int k = 0; k < View->NbIso; k++) {
       if(ValMin == ValMax)
 	k = View->NbIso / 2;
-      PaletteDiscrete(View, View->NbIso, k);
+      unsigned int col = PaletteDiscrete(View, View->NbIso, k);
       IsoTriangle(View, X, Y, Z, Val,
-		  View->GVFI(ValMin, ValMax, View->NbIso, k));
+		  View->GVFI(ValMin, ValMax, View->NbIso, k), col);
       if(ValMin == ValMax) 
 	break;
     }
@@ -575,18 +616,23 @@ void Draw_ScalarTetrahedron(Post_View * View, int preproNormals,
 
   double *vv = &V[4 * View->TimeStep];
 
-  if(!preproNormals && View->Boundary > 0) {
+  // To draw filled iso-values or continuous maps, we should ideally
+  // do volume rendering. Until we can do that, we just draw the
+  // solution on the boundary of the elements
+  if((View->Boundary > 0) ||
+     (View->IntervalsType == DRAW_POST_DISCRETE ||
+      View->IntervalsType == DRAW_POST_CONTINUOUS)) {
     View->Boundary--;
     int ts = View->TimeStep;
     View->TimeStep = 0;
     REORDER3(0, 2, 1);
-    Draw_ScalarTriangle(View, 0, ValMin, ValMax, Xp, Yp, Zp, Val); // 021
+    Draw_ScalarTriangle(View, preproNormals, ValMin, ValMax, Xp, Yp, Zp, Val); // 021
     REORDER3(0, 1, 3);
-    Draw_ScalarTriangle(View, 0, ValMin, ValMax, Xp, Yp, Zp, Val); // 013
+    Draw_ScalarTriangle(View, preproNormals, ValMin, ValMax, Xp, Yp, Zp, Val); // 013
     REORDER3(0, 3, 2);
-    Draw_ScalarTriangle(View, 0, ValMin, ValMax, Xp, Yp, Zp, Val); // 032
+    Draw_ScalarTriangle(View, preproNormals, ValMin, ValMax, Xp, Yp, Zp, Val); // 032
     REORDER3(3, 1, 2);
-    Draw_ScalarTriangle(View, 0, ValMin, ValMax, Xp, Yp, Zp, Val); // 312
+    Draw_ScalarTriangle(View, preproNormals, ValMin, ValMax, Xp, Yp, Zp, Val); // 312
     View->TimeStep = ts;
     View->Boundary++;
     return;
@@ -598,14 +644,26 @@ void Draw_ScalarTetrahedron(Post_View * View, int preproNormals,
     Draw_ElementBoundary(POST_TETRAHEDRON, View, X, Y, Z);
 
   if(!preproNormals && View->IntervalsType == DRAW_POST_NUMERIC) {
-    d = 0.25 * (Val[0] + Val[1] + Val[2] + Val[3]);
-    if(d >= ValMin && d <= ValMax) {
-      PaletteContinuous(View, ValMin, ValMax, d);
-      sprintf(Num, View->Format, d);
-      glRasterPos3d(0.25 * (X[0] + X[1] + X[2] + X[3]),
-                    0.25 * (Y[0] + Y[1] + Y[2] + Y[3]),
-                    0.25 * (Z[0] + Z[1] + Z[2] + Z[3]));
-      Draw_String(Num);
+    if(View->GlyphLocation == DRAW_POST_LOCATE_COG){
+      d = 0.25 * (Val[0] + Val[1] + Val[2] + Val[3]);
+      if(d >= ValMin && d <= ValMax) {
+	PaletteContinuous(View, ValMin, ValMax, d);
+	sprintf(Num, View->Format, d);
+	glRasterPos3d(0.25 * (X[0] + X[1] + X[2] + X[3]),
+		      0.25 * (Y[0] + Y[1] + Y[2] + Y[3]),
+		      0.25 * (Z[0] + Z[1] + Z[2] + Z[3]));
+	Draw_String(Num);
+      }
+    }
+    else{
+      for(int i = 0; i < 4; i++){
+	if(Val[i] >= ValMin && Val[i] <= ValMax) {
+	  PaletteContinuous(View, ValMin, ValMax, Val[i]);
+	  sprintf(Num, View->Format, Val[i]);
+	  glRasterPos3d(X[i], Y[i], Z[i]);
+	  Draw_String(Num);
+	}
+      }
     }
   }
   else if(!View->TriVertexArray || (View->TriVertexArray && View->TriVertexArray->fill)){
@@ -649,7 +707,14 @@ void Draw_ScalarQuadrangle(Post_View * View, int preproNormals,
 
   if(!preproNormals && show) {
     SaturateValues(View->SaturateValues, ValMin, ValMax, vv, Val, 4);
-    Draw_ElementBoundary(POST_QUADRANGLE, View, X, Y, Z);
+    if(View->Light && View->LightLines){
+      // FIXME: very crude (not smooth) and inefficient
+      double n[3];
+      normal3points(X[0], Y[0], Z[0], X[1], Y[1], Z[1], X[2], Y[2], Z[2], n);
+      Draw_ElementBoundary(POST_QUADRANGLE, View, X, Y, Z, n);
+    }
+    else
+      Draw_ElementBoundary(POST_QUADRANGLE, View, X, Y, Z);
   }
 
   Draw_ScalarTriangle(View, preproNormals, ValMin, ValMax, X, Y, Z, vv); // 012
@@ -671,7 +736,12 @@ void Draw_ScalarHexahedron(Post_View * View, int preproNormals,
   ts = View->TimeStep;
   View->TimeStep = 0;
 
-  if(!preproNormals && View->Boundary > 0) {
+  // To draw filled iso-values or continuous maps, we should ideally
+  // do volume rendering. Until we can do that, we just draw the
+  // solution on the boundary of the elements
+  if((View->Boundary > 0) ||
+     (View->IntervalsType == DRAW_POST_DISCRETE ||
+      View->IntervalsType == DRAW_POST_CONTINUOUS)) {
     View->Boundary--;
     REORDER4(0, 1, 5, 4);
     Draw_ScalarQuadrangle(View, 0, ValMin, ValMax, Xp, Yp, Zp, Val); // 0154
@@ -727,7 +797,12 @@ void Draw_ScalarPrism(Post_View * View, int preproNormals,
   ts = View->TimeStep;
   View->TimeStep = 0;
 
-  if(!preproNormals && View->Boundary > 0) {
+  // To draw filled iso-values or continuous maps, we should ideally
+  // do volume rendering. Until we can do that, we just draw the
+  // solution on the boundary of the elements
+  if((View->Boundary > 0) ||
+     (View->IntervalsType == DRAW_POST_DISCRETE ||
+      View->IntervalsType == DRAW_POST_CONTINUOUS)) {
     View->Boundary--;
     REORDER4(0, 1, 4, 3);
     Draw_ScalarQuadrangle(View, 0, ValMin, ValMax, Xp, Yp, Zp, Val);
@@ -737,7 +812,7 @@ void Draw_ScalarPrism(Post_View * View, int preproNormals,
     Draw_ScalarQuadrangle(View, 0, ValMin, ValMax, Xp, Yp, Zp, Val);
     REORDER4(1, 2, 5, 4);
     Draw_ScalarQuadrangle(View, 0, ValMin, ValMax, Xp, Yp, Zp, Val);
-    Draw_ScalarTriangle(View, 0, ValMin, ValMax, &X[3], &Y[3], &Z[3], &V[3]);
+    Draw_ScalarTriangle(View, 0, ValMin, ValMax, &X[3], &Y[3], &Z[3], &vv[3]);
     View->TimeStep = ts;
     View->Boundary++;
     return;
@@ -772,7 +847,12 @@ void Draw_ScalarPyramid(Post_View * View, int preproNormals,
   ts = View->TimeStep;
   View->TimeStep = 0;
 
-  if(!preproNormals && View->Boundary > 0) {
+  // To draw filled iso-values or continuous maps, we should ideally
+  // do volume rendering. Until we can do that, we just draw the
+  // solution on the boundary of the elements
+  if((View->Boundary > 0) ||
+     (View->IntervalsType == DRAW_POST_DISCRETE ||
+      View->IntervalsType == DRAW_POST_CONTINUOUS)) {
     View->Boundary--;
     REORDER4(0, 3, 2, 1);
     Draw_ScalarQuadrangle(View, 0, ValMin, ValMax, Xp, Yp, Zp, Val);
@@ -937,7 +1017,7 @@ void Draw_VectorElement(int type, Post_View * View, int preproNormals,
     
     if(ext_nbcomp == 1){
       Draw_ScalarElement(type, View, preproNormals, ext_min, ext_max, xx, yy, zz, ext_norm);
-      if(type == POST_POINT && ts > 0) {  // draw point "trajectory"
+      if(type == POST_POINT && ts > 0 && View->LineWidth) {  // draw point "trajectory"
 	if(View->LineType) {
 	  double dx, dy, dz, dx2, dy2, dz2, XX[2], YY[2], ZZ[2];
 	  for(int j = 0; j < ts; j++) {
@@ -994,9 +1074,7 @@ void Draw_VectorElement(int type, Post_View * View, int preproNormals,
     Draw_ElementBoundary(type, View, X, Y, Z);
 
   double xc = 0., yc = 0., zc = 0.;
-  if(View->Normals || View->Tangents || 
-     View->IntervalsType == DRAW_POST_NUMERIC ||
-     View->ArrowLocation == DRAW_POST_LOCATE_COG){
+  if(View->Normals || View->Tangents || View->GlyphLocation == DRAW_POST_LOCATE_COG){
     for(int k = 0; k < nbnod; k++) {
       xc += X[k];
       yc += Y[k];
@@ -1031,8 +1109,7 @@ void Draw_VectorElement(int type, Post_View * View, int preproNormals,
 		xc, yc, zc, t[0], t[1], t[2], View->Light);
   }
 
-  if(View->ArrowLocation == DRAW_POST_LOCATE_COG ||
-     View->IntervalsType == DRAW_POST_NUMERIC) {
+  if(View->GlyphLocation == DRAW_POST_LOCATE_COG) {
     double dd = 0., ext_dd = 0., dx = 0., dy = 0., dz = 0.;
     for(int k = 0; k < nbnod; k++) {
       dx += Val[k][0];
@@ -1058,9 +1135,12 @@ void Draw_VectorElement(int type, Post_View * View, int preproNormals,
 	PaletteDiscrete(View, View->NbIso,
 			View->GIFV(ext_min, ext_max, View->NbIso, ext_dd));
       if(View->IntervalsType == DRAW_POST_NUMERIC) {
-	char Num[100];
+	char Num[100], Numx[100], Numy[100], Numz[100];
         glRasterPos3d(xc, yc, zc);
-        sprintf(Num, View->Format, ext_dd);
+        sprintf(Numx, View->Format, dx);
+        sprintf(Numy, View->Format, dy);
+        sprintf(Numz, View->Format, dz);
+        sprintf(Num, "(%s,%s,%s)", Numx, Numy, Numz);
         Draw_String(Num);
       }
       else {
@@ -1090,22 +1170,33 @@ void Draw_VectorElement(int type, Post_View * View, int preproNormals,
 	else
 	  PaletteDiscrete(View, View->NbIso,
 			  View->GIFV(ext_min, ext_max, View->NbIso, ext_norm[k]));
-        double fact = CTX.pixel_equiv_x / CTX.s[0] * View->ArrowSize /
-	  (View->ArrowSizeProportional ? ValMax : norm[k]);
-        if(View->ScaleType == DRAW_POST_LOGARITHMIC && ValMin > 0) {
-          Val[k][0] /= norm[k];
-          Val[k][1] /= norm[k];
-          Val[k][2] /= norm[k];
-          norm[k] = log10(norm[k] / ValMin);
-          Val[k][0] *= norm[k];
-          Val[k][1] *= norm[k];
-          Val[k][2] *= norm[k];
-        }
-        Draw_Vector(View->VectorType, View->IntervalsType != DRAW_POST_ISO,
-		    View->ArrowRelHeadRadius, View->ArrowRelStemLength,
-		    View->ArrowRelStemRadius, X[k], Y[k], Z[k], 
-		    fact * Val[k][0], fact * Val[k][1], fact * Val[k][2], 
-		    View->Light);
+	if(View->IntervalsType == DRAW_POST_NUMERIC) {
+	  char Num[100], Numx[100], Numy[100], Numz[100];
+	  glRasterPos3d(X[k], Y[k], Z[k]);
+	  sprintf(Numx, View->Format, Val[k][0]);
+	  sprintf(Numy, View->Format, Val[k][1]);
+	  sprintf(Numz, View->Format, Val[k][2]);
+	  sprintf(Num, "(%s,%s,%s)", Numx, Numy, Numz);
+	  Draw_String(Num);
+	}
+	else{
+	  double fact = CTX.pixel_equiv_x / CTX.s[0] * View->ArrowSize /
+	    (View->ArrowSizeProportional ? ValMax : norm[k]);
+	  if(View->ScaleType == DRAW_POST_LOGARITHMIC && ValMin > 0) {
+	    Val[k][0] /= norm[k];
+	    Val[k][1] /= norm[k];
+	    Val[k][2] /= norm[k];
+	    norm[k] = log10(norm[k] / ValMin);
+	    Val[k][0] *= norm[k];
+	    Val[k][1] *= norm[k];
+	    Val[k][2] *= norm[k];
+	  }
+	  Draw_Vector(View->VectorType, View->IntervalsType != DRAW_POST_ISO,
+		      View->ArrowRelHeadRadius, View->ArrowRelStemLength,
+		      View->ArrowRelStemRadius, X[k], Y[k], Z[k], 
+		      fact * Val[k][0], fact * Val[k][1], fact * Val[k][2], 
+		      View->Light);
+	}
       }
     }
   }
