@@ -1,4 +1,4 @@
-// $Id: Create.cpp,v 1.69 2005/02/26 04:53:13 geuzaine Exp $
+// $Id: Create.cpp,v 1.79 2005/11/28 19:13:49 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -510,10 +510,6 @@ void End_Surface(Surface * s, int reset_orientations)
 Curve *Create_Curve(int Num, int Typ, int Order, List_T * Liste,
                     List_T * Knots, int p1, int p2, double u1, double u2)
 {
-  Curve *pC;
-  Vertex *v;
-  int i, j, iPnt;
-  double d;
   double matcr[4][4] = { {-0.5, 1.5, -1.5, 0.5},
 			 {1.0, -2.5, 2.0, -0.5},
 			 {-0.5, 0.0, 0.5, 0.0},
@@ -527,14 +523,14 @@ Curve *Create_Curve(int Num, int Typ, int Order, List_T * Liste,
 			  {-3, 3.0, 0, 0.0},
 			  {1, 0, 0, 0.0} };
 
-  pC = (Curve *) Malloc(sizeof(Curve));
-  pC->Dirty = 0;
+  Curve *pC = (Curve *) Malloc(sizeof(Curve));
+  //  pC->bds = 0;
+  pC->LinVertexArray = NULL;
   pC->Color.type = 0;
   pC->Visible = VIS_GEOM | VIS_MESH;
   pC->cp = NULL;
   pC->Vertices = List_Create(2, 20, sizeof(Vertex *));
   pC->Extrude = NULL;
-  pC->theSegmRep = 0;
   pC->Typ = Typ;
   pC->Num = Num;
   THEM->MaxLineNum = IMAX(THEM->MaxLineNum, Num);
@@ -545,24 +541,24 @@ Curve *Create_Curve(int Num, int Typ, int Order, List_T * Liste,
   pC->Circle.n[0] = 0.0;
   pC->Circle.n[1] = 0.0;
   pC->Circle.n[2] = 1.0;
-  for(i = 0; i < 4; i++) {
+  for(int i = 0; i < 4; i++) {
     pC->ipar[i] = 0;
     pC->dpar[i] = 0.0;
   }
 
   if(Typ == MSH_SEGM_SPLN) {
-    for(i = 0; i < 4; i++)
-      for(j = 0; j < 4; j++)
+    for(int i = 0; i < 4; i++)
+      for(int j = 0; j < 4; j++)
         pC->mat[i][j] = matcr[i][j];
   }
   else if(Typ == MSH_SEGM_BSPLN) {
-    for(i = 0; i < 4; i++)
-      for(j = 0; j < 4; j++)
+    for(int i = 0; i < 4; i++)
+      for(int j = 0; j < 4; j++)
         pC->mat[i][j] = matbs[i][j] / 6.0;
   }
   else if(Typ == MSH_SEGM_BEZIER) {
-    for(i = 0; i < 4; i++)
-      for(j = 0; j < 4; j++)
+    for(int i = 0; i < 4; i++)
+      for(int j = 0; j < 4; j++)
         pC->mat[i][j] = matbez[i][j];
   }
 
@@ -576,7 +572,8 @@ Curve *Create_Curve(int Num, int Typ, int Order, List_T * Liste,
     List_Read(Knots, List_Nbr(Knots) - 1, &kmax);
     pC->ubeg = kmin;
     pC->uend = kmax;
-    for(i = 0; i < List_Nbr(Knots); i++) {
+    for(int i = 0; i < List_Nbr(Knots); i++) {
+      double d;
       List_Read(Knots, i, &d);
       pC->k[i] = (float)d;
     }
@@ -586,13 +583,14 @@ Curve *Create_Curve(int Num, int Typ, int Order, List_T * Liste,
 
   if(Liste) {
     pC->Control_Points = List_Create(List_Nbr(Liste), 1, sizeof(Vertex *));
-    for(j = 0; j < List_Nbr(Liste); j++) {
+    for(int j = 0; j < List_Nbr(Liste); j++) {
+      int iPnt;
       List_Read(Liste, j, &iPnt);
+      Vertex *v;
       if((v = FindPoint(iPnt, THEM)))
         List_Add(pC->Control_Points, &v);
       else{
         Msg(GERROR, "Unknown control point %d in Curve %d", iPnt, pC->Num);
-	pC->Dirty = 1;
       }
     }
   }
@@ -608,13 +606,13 @@ Curve *Create_Curve(int Num, int Typ, int Order, List_T * Liste,
     List_Read(pC->Control_Points, List_Nbr(pC->Control_Points) - 1, &pC->end);
   }
   else {
+    Vertex *v;
     if((v = FindPoint(p1, THEM))) {
       Msg(INFO, "Curve %d first control point %d ", pC->Num, v->Num);
       pC->beg = v;
     }
     else {
       Msg(GERROR, "Unknown control point %d in Curve %d", p1, pC->Num);
-      pC->Dirty = 1;
     }
     if((v = FindPoint(p2, THEM))) {
       Msg(INFO, "Curve %d first control point %d ", pC->Num, v->Num);
@@ -622,7 +620,6 @@ Curve *Create_Curve(int Num, int Typ, int Order, List_T * Liste,
     }
     else {
       Msg(GERROR, "Unknown control point %d in Curve %d", p2, pC->Num);
-      pC->Dirty = 1;
     }
   }
 
@@ -635,6 +632,8 @@ void Free_Curve(void *a, void *b)
 {
   Curve *pC = *(Curve **) a;
   if(pC) {
+    if(pC->LinVertexArray)
+      delete pC->LinVertexArray;
     List_Delete(pC->Vertices);
     Tree_Action(pC->Simplexes, Free_Simplex);
     Tree_Delete(pC->Simplexes);
@@ -643,8 +642,6 @@ void Free_Curve(void *a, void *b)
     Free(pC->k);
     List_Delete(pC->Control_Points);
     Free(pC->cp);
-    if(pC->theSegmRep)
-      delete pC->theSegmRep;
     Free(pC);
     pC = NULL;
   }
@@ -652,21 +649,18 @@ void Free_Curve(void *a, void *b)
 
 Surface *Create_Surface(int Num, int Typ)
 {
-  Surface *pS;
-  int i;  
-
-  pS = (Surface *) Malloc(sizeof(Surface));
-  pS->Dirty = 0;
-  pS->thePolyRep = 0;
+  Surface *pS = (Surface *) Malloc(sizeof(Surface));
+  //  pS->bds = 0;
   pS->Color.type = 0;
   pS->Visible = VIS_GEOM | VIS_MESH;
   pS->Num = Num;
   THEM->MaxSurfaceNum = IMAX(THEM->MaxSurfaceNum, Num);
   pS->Typ = Typ;
   pS->Method = LIBRE;
-  for(i = 0; i < 5; i++)
+  for(int i = 0; i < 5; i++)
     pS->ipar[i] = 0;
   pS->Recombine = 0;
+  pS->Recombine_Dir = 1;
   pS->RecombineAngle = 75;
   pS->TrsfPoints = List_Create(4, 4, sizeof(Vertex *));
   pS->Simplexes = Tree_Create(sizeof(Simplex *), compareQuality);
@@ -714,8 +708,6 @@ void Free_Surface(void *a, void *b)
       delete pS->QuadVertexArray;
     if(pS->normals)
       delete pS->normals;
-    if(pS->thePolyRep)
-      delete pS->thePolyRep;
     Free(pS);
     pS = NULL;
   }
@@ -723,18 +715,14 @@ void Free_Surface(void *a, void *b)
 
 Volume *Create_Volume(int Num, int Typ)
 {
-  Volume *pV;
-  int i;
-
-  pV = (Volume *) Malloc(sizeof(Volume));
-  pV->Dirty = 0;
+  Volume *pV = (Volume *) Malloc(sizeof(Volume));
   pV->Color.type = 0;
   pV->Visible = VIS_GEOM | VIS_MESH;
   pV->Num = Num;
   THEM->MaxVolumeNum = IMAX(THEM->MaxVolumeNum, Num);
   pV->Typ = Typ;
   pV->Method = LIBRE;
-  for(i = 0; i < 8; i++)
+  for(int i = 0; i < 8; i++)
     pV->ipar[i] = 0;
   pV->TrsfPoints = List_Create(6, 6, sizeof(Vertex *));
   pV->Surfaces = List_Create(1, 2, sizeof(Surface *));
@@ -749,6 +737,7 @@ Volume *Create_Volume(int Num, int Typ)
   pV->Edges = NULL;
   pV->Faces = NULL;
   // for old extrusion mesh generator
+  pV->Lin_Surf = Tree_Create(sizeof(Simplex *), compareSimplex);
   pV->Simp_Surf = Tree_Create(sizeof(Simplex *), compareSimplex);
   pV->Quad_Surf = Tree_Create(sizeof(Simplex *), compareQuadrangle);
   pV->TriVertexArray = NULL;
@@ -779,6 +768,7 @@ void Free_Volume_But_Not_Elements(void *a, void *b)
     List_Delete(pV->SurfacesOrientations);
     Tree_Delete(pV->Simplexes);
     Tree_Delete(pV->SimplexesBase);
+    Tree_Delete(pV->Lin_Surf); // for old extrusion mesh generator
     Tree_Delete(pV->Simp_Surf); // for old extrusion mesh generator
     Tree_Delete(pV->Quad_Surf); // for old extrusion mesh generator
     Tree_Delete(pV->Vertices);  // vertices freed elsewhere
