@@ -34,7 +34,9 @@ class Element {
   virtual ~Element();
   double lij(Vertex *Vi, Vertex *Vj);
   virtual double maxEdge() = 0;
-  virtual void ExportLcField(FILE *f) = 0;
+  virtual double minEdge() = 0;
+  double RhoShapeMeasure();
+  virtual void ExportStatistics(FILE *f) = 0;
 };
 
 class Quadrangle : public Element{
@@ -44,7 +46,8 @@ class Quadrangle : public Element{
   Quadrangle(Vertex *v1, Vertex *v2, Vertex *v3, Vertex *v4);
   ~Quadrangle() {}
   double maxEdge();
-  void ExportLcField(FILE *f);
+  double minEdge();
+  void ExportStatistics(FILE *f);
 };
 
 class Hexahedron : public Element{
@@ -56,7 +59,8 @@ class Hexahedron : public Element{
   ~Hexahedron() {}
   double Orientation();
   double maxEdge();
-  void ExportLcField(FILE *f);
+  double minEdge();
+  void ExportStatistics(FILE *f);
 };
 
 class Prism : public Element{
@@ -68,7 +72,8 @@ class Prism : public Element{
   ~Prism() {}
   double Orientation();
   double maxEdge();
-  void ExportLcField(FILE *f);
+  double minEdge();
+  void ExportStatistics(FILE *f);
 };
 
 class Pyramid : public Element{
@@ -79,7 +84,8 @@ class Pyramid : public Element{
   ~Pyramid() {}
   double Orientation();
   double maxEdge();
-  void ExportLcField(FILE *f);
+  double minEdge();
+  void ExportStatistics(FILE *f);
 };
 
 // C interface
