@@ -1,4 +1,4 @@
-// $Id: Generator.cpp,v 1.63 2005/01/01 19:35:30 geuzaine Exp $
+// $Id: Generator.cpp,v 1.73 2005/10/26 15:19:24 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -19,6 +19,7 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
+#include "BDS.h"
 #include "Gmsh.h"
 #include "Numeric.h"
 #include "Mesh.h"
@@ -26,6 +27,7 @@
 #include "Context.h"
 #include "OpenFile.h"
 #include "Views.h"
+#include "PartitionMesh.h"
 
 extern Mesh *THEM;
 extern Context_T CTX;
@@ -50,6 +52,7 @@ void GetStatistics(double stat[50])
   stat[1] = Tree_Nbr(THEM->Curves);
   stat[2] = Tree_Nbr(THEM->Surfaces);
   stat[3] = Tree_Nbr(THEM->Volumes);
+  stat[45] = List_Nbr(THEM->PhysicalGroups);
   
   stat[4] = 0.;
   if(Tree_Nbr(THEM->Curves)) {
@@ -196,12 +199,47 @@ void ApplyLcFactor(Mesh * M)
   List_Action(M->Metric->Attractors, ApplyLcFactor_Attractor);
 }
 
+void Move_SimplexBaseToSimplex(Mesh * M, int dimension)
+{
+  if(dimension >= 1){
+    List_T *Curves = Tree2List(M->Curves);
+    for(int i = 0; i < List_Nbr(Curves); i++) {
+      Curve *c;
+      List_Read(Curves, i, &c);
+      Move_SimplexBaseToSimplex(&c->SimplexesBase, c->Simplexes);
+    }
+    List_Delete(Curves);
+  }
+
+  if(dimension >= 2){
+    List_T *Surfaces = Tree2List(M->Surfaces);
+    for(int i = 0; i < List_Nbr(Surfaces); i++){
+      Surface *s;
+      List_Read(Surfaces, i, &s);
+      Move_SimplexBaseToSimplex(&s->SimplexesBase, s->Simplexes);
+    }
+    List_Delete(Surfaces);
+  }
+  
+  if(dimension >= 3){
+    List_T *Volumes = Tree2List(M->Volumes);
+    for(int i = 0; i < List_Nbr(Volumes); i++){
+      Volume *v;
+      List_Read(Volumes, i, &v);
+      Move_SimplexBaseToSimplex(&v->SimplexesBase, v->Simplexes);
+    }
+    List_Delete(Volumes);
+  }
+}
+
 void Maillage_Dimension_1(Mesh * M)
 {
   double t1, t2;
 
   t1 = Cpu();
+
   Tree_Action(M->Curves, Maillage_Curve);
+
   t2 = Cpu();
   M->timing[0] = t2 - t1;
 }
@@ -238,6 +276,11 @@ void Maillage_Dimension_2(Mesh * M)
 
   Tree_Action(M->Surfaces, Maillage_Surface);
 
+  // global "all-quad" recombine
+
+  if(CTX.mesh.algo_recombine == 2)
+    Recombine_All(M);
+
   t2 = Cpu();
 
   M->timing[1] = t2 - t1;
@@ -301,11 +344,29 @@ void Maillage_Dimension_3(Mesh * M)
   M->timing[2] = t2 - t1;
 }
 
+void Init_Mesh0(Mesh * M)
+{
+  THEM = M;
+  M->bds = 0;
+  M->bds_mesh = 0;
+  M->Vertices = NULL;
+  M->Simplexes = NULL;
+  M->Points = NULL;
+  M->Curves = NULL;
+  M->SurfaceLoops = NULL;
+  M->EdgeLoops = NULL;
+  M->Surfaces = NULL;
+  M->Volumes = NULL;
+  M->PhysicalGroups = NULL;
+  M->Partitions = NULL;
+  M->Metric = NULL;
+  M->BGM.bgm = NULL;
+  M->Grid.init = 0;
+}
 
 void Init_Mesh(Mesh * M)
 {
   THEM = M;
-
   M->MaxPointNum = 0;
   M->MaxLineNum = 0;
   M->MaxLineLoopNum = 0;
@@ -318,6 +379,9 @@ void Init_Mesh(Mesh * M)
 
   ExitExtrude();
 
+  if(M->bds) delete M->bds;
+  M->bds = 0;
+
   Tree_Action(M->Vertices, Free_Vertex);  
   Tree_Delete(M->Vertices);
 
@@ -332,10 +396,10 @@ void Init_Mesh(Mesh * M)
   Tree_Action(M->Curves, Free_Curve);
   Tree_Delete(M->Curves);
 
-  Tree_Action (M->SurfaceLoops, Free_SurfaceLoop);
+  Tree_Action(M->SurfaceLoops, Free_SurfaceLoop);
   Tree_Delete(M->SurfaceLoops);
 
-  Tree_Action (M->EdgeLoops, Free_EdgeLoop);
+  Tree_Action(M->EdgeLoops, Free_EdgeLoop);
   Tree_Delete(M->EdgeLoops);
 
   Tree_Action(M->Surfaces, Free_Surface);
@@ -397,7 +461,7 @@ void mai3d(Mesh * M, int Asked)
 
   oldstatus = M->status;
 
-  // re-read data
+  // Re-read data
 
   if((Asked > oldstatus && Asked >= 0 && oldstatus < 0) ||
      (Asked < oldstatus)) {
@@ -407,6 +471,9 @@ void mai3d(Mesh * M, int Asked)
 
   CTX.threads_lock = 1;
 
+  // Clean up all the 2nd order nodes and transfer all SimplexBase
+  // into "real" Simplexes
+
   Degre1();
 
   // 1D mesh
@@ -461,11 +528,16 @@ void mai3d(Mesh * M, int Asked)
   if(M->status == 3 && CTX.mesh.optimize)
     Optimize_Netgen(M);
 
-  // Second order elements
+  // Create second order elements
 
   if(M->status && CTX.mesh.order == 2)
     Degre2(M->status);
 
+  // Partition
+
+  if(M->status > 1 && CTX.mesh.nbPartitions != 1)
+    PartitionMesh(M, CTX.mesh.nbPartitions);
+
   CTX.threads_lock = 0;
   CTX.mesh.changed = 1;
 }
