@@ -1,4 +1,4 @@
-// $Id: Print_Mesh.cpp,v 1.60 2005/02/28 23:57:59 geuzaine Exp $
+// $Id: Print_Mesh.cpp,v 1.67 2005/11/03 06:46:54 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -28,6 +28,7 @@
 #include "Context.h"
 
 extern Context_T CTX;
+extern Mesh *THEM;
 
 // Write mesh in native MSH format
 
@@ -98,6 +99,15 @@ static void _msh_process_nodes(Mesh *M)
     fprintf(MSHFILE, "$ENDNOD\n");
 }
 
+int _get_partition(int index)
+{
+  MeshPartition **part = (MeshPartition**)List_Pointer_Test(THEM->Partitions, index);
+  if(!part)
+    return -1; // OK, no partitions available
+  else
+    return (*part)->Num; // partition number
+}
+
 static void _msh_print_simplex(void *a, void *b)
 {
   int i, type, nbn, nbs = 0;
@@ -159,10 +169,13 @@ static void _msh_print_simplex(void *a, void *b)
     }
   }
 
-  if(CTX.mesh.msh_file_version == 2.0)
-    fprintf(MSHFILE, "%d %d 2 %d %d",
-	    MSH_ELEMENT_NUM++, type, MSH_PHYSICAL_NUM ? MSH_PHYSICAL_NUM : s->iEnt, 
-	    s->iEnt);
+  if(CTX.mesh.msh_file_version == 2.0){
+    int part = _get_partition(s->iPart);
+    fprintf(MSHFILE, "%d %d %d %d %d",
+	    MSH_ELEMENT_NUM++, type, (part >= 0) ? 3 : 2, 
+	    MSH_PHYSICAL_NUM ? MSH_PHYSICAL_NUM : s->iEnt, s->iEnt);
+    if(part >= 0) fprintf(MSHFILE, " %d", part);
+  }
   else
     fprintf(MSHFILE, "%d %d %d %d %d",
 	    MSH_ELEMENT_NUM++, type,
@@ -207,10 +220,13 @@ static void _msh_print_quadrangle(void *a, void *b)
   else
     type = QUADRANGLE;
 
-  if(CTX.mesh.msh_file_version == 2.0)
-    fprintf(MSHFILE, "%d %d 2 %d %d",
-	    MSH_ELEMENT_NUM++, type, MSH_PHYSICAL_NUM ? MSH_PHYSICAL_NUM : q->iEnt, 
-	    q->iEnt);
+  if(CTX.mesh.msh_file_version == 2.0){
+    int part = _get_partition(q->iPart);
+    fprintf(MSHFILE, "%d %d %d %d %d",
+	    MSH_ELEMENT_NUM++, type, (part >= 0) ? 3 : 2, 
+	    MSH_PHYSICAL_NUM ? MSH_PHYSICAL_NUM : q->iEnt, q->iEnt);
+    if(part >= 0) fprintf(MSHFILE, " %d", part);
+  }
   else
     fprintf(MSHFILE, "%d %d %d %d %d",
 	    MSH_ELEMENT_NUM++, type,
@@ -254,14 +270,33 @@ static void _msh_print_hexahedron(void *a, void *b)
   if(h->VSUP) {
     type = HEXAHEDRON_2;
     nbs = 12 + 6 + 1;
+    if(h->Orientation() < 0) {
+      Vertex *temp;
+      temp = h->V[0]; h->V[0] = h->V[2]; h->V[2] = temp;
+      temp = h->V[4]; h->V[4] = h->V[6]; h->V[6] = temp;
+      temp = h->VSUP[0]; h->VSUP[0] = h->VSUP[4];  h->VSUP[4]  = temp;
+      temp = h->VSUP[1]; h->VSUP[1] = h->VSUP[10]; h->VSUP[10] = temp;
+      temp = h->VSUP[2]; h->VSUP[2] = h->VSUP[8];  h->VSUP[8]  = temp;
+      temp = h->VSUP[5]; h->VSUP[5] = h->VSUP[6];  h->VSUP[6]  = temp;
+      temp = h->VSUP[7]; h->VSUP[7] = h->VSUP[11]; h->VSUP[11] = temp;
+    }
   }
-  else
+  else {
     type = HEXAHEDRON;
+    if(h->Orientation() < 0) {
+      Vertex *temp;
+      temp = h->V[0]; h->V[0] = h->V[2]; h->V[2] = temp;
+      temp = h->V[4]; h->V[4] = h->V[6]; h->V[6] = temp;
+    }
+  }
 
-  if(CTX.mesh.msh_file_version == 2.0)
-    fprintf(MSHFILE, "%d %d 2 %d %d",
-	    MSH_ELEMENT_NUM++, type, MSH_PHYSICAL_NUM ? MSH_PHYSICAL_NUM : h->iEnt,
-	    h->iEnt);
+  if(CTX.mesh.msh_file_version == 2.0){
+    int part = _get_partition(h->iPart);
+    fprintf(MSHFILE, "%d %d %d %d %d",
+	    MSH_ELEMENT_NUM++, type, (part >= 0) ? 3 : 2, 
+	    MSH_PHYSICAL_NUM ? MSH_PHYSICAL_NUM : h->iEnt, h->iEnt);
+    if(part >= 0) fprintf(MSHFILE, " %d", part);
+  }
   else
     fprintf(MSHFILE, "%d %d %d %d %d",
 	    MSH_ELEMENT_NUM++, type,
@@ -294,15 +329,31 @@ static void _msh_print_prism(void *a, void *b)
   if(p->VSUP) {
     type = PRISM_2;
     nbs = 9 + 3;
+    if(p->Orientation() < 0) {
+      Vertex *temp;
+      temp = p->V[0]; p->V[0] = p->V[1]; p->V[1] = temp;
+      temp = p->V[3]; p->V[3] = p->V[4]; p->V[4] = temp;
+      temp = p->VSUP[1]; p->VSUP[1] = p->VSUP[3]; p->VSUP[3] = temp;
+      temp = p->VSUP[2]; p->VSUP[2] = p->VSUP[4]; p->VSUP[4] = temp;
+      temp = p->VSUP[7]; p->VSUP[7] = p->VSUP[8]; p->VSUP[8] = temp;
+    }
   }
   else {
     type = PRISM;
+    if(p->Orientation() < 0) {
+      Vertex *temp;
+      temp = p->V[0]; p->V[0] = p->V[1]; p->V[1] = temp;
+      temp = p->V[3]; p->V[3] = p->V[4]; p->V[4] = temp;
+    }
   }
 
-  if(CTX.mesh.msh_file_version == 2.0)
-    fprintf(MSHFILE, "%d %d 2 %d %d",
-	    MSH_ELEMENT_NUM++, type, MSH_PHYSICAL_NUM ? MSH_PHYSICAL_NUM : p->iEnt,
-	    p->iEnt);
+  if(CTX.mesh.msh_file_version == 2.0){
+    int part = _get_partition(p->iPart);
+    fprintf(MSHFILE, "%d %d %d %d %d",
+	    MSH_ELEMENT_NUM++, type, (part >= 0) ? 3 : 2, 
+	    MSH_PHYSICAL_NUM ? MSH_PHYSICAL_NUM : p->iEnt, p->iEnt);
+    if(part >= 0) fprintf(MSHFILE, " %d", part);
+  }
   else
     fprintf(MSHFILE, "%d %d %d %d %d",
 	    MSH_ELEMENT_NUM++, type,
@@ -335,15 +386,29 @@ static void _msh_print_pyramid(void *a, void *b)
   if(p->VSUP) {
     type = PYRAMID_2;
     nbs = 8 + 1;
+    if(p->Orientation() < 0) {
+      Vertex *temp;
+      temp = p->V[0]; p->V[0] = p->V[2]; p->V[2] = temp;
+      temp = p->VSUP[0]; p->VSUP[0] = p->VSUP[3]; p->VSUP[3] = temp;
+      temp = p->VSUP[1]; p->VSUP[1] = p->VSUP[5]; p->VSUP[5] = temp;
+      temp = p->VSUP[2]; p->VSUP[2] = p->VSUP[6]; p->VSUP[6] = temp;
+    }
   }
   else {
     type = PYRAMID;
+    if(p->Orientation() < 0) {
+      Vertex *temp;
+      temp = p->V[0]; p->V[0] = p->V[2]; p->V[2] = temp;
+    }
   }
 
-  if(CTX.mesh.msh_file_version == 2.0)
-    fprintf(MSHFILE, "%d %d 2 %d %d",
-	    MSH_ELEMENT_NUM++, type, MSH_PHYSICAL_NUM ? MSH_PHYSICAL_NUM : p->iEnt,
-	    p->iEnt);
+  if(CTX.mesh.msh_file_version == 2.0){
+    int part = _get_partition(p->iPart);
+    fprintf(MSHFILE, "%d %d %d %d %d",
+	    MSH_ELEMENT_NUM++, type, (part >= 0) ? 3 : 2, 
+	    MSH_PHYSICAL_NUM ? MSH_PHYSICAL_NUM : p->iEnt, p->iEnt);
+    if(part >= 0) fprintf(MSHFILE, " %d", part);
+  }
   else
     fprintf(MSHFILE, "%d %d %d %d %d",
 	    MSH_ELEMENT_NUM++, type,
@@ -415,8 +480,7 @@ static void _msh_print_elements(Mesh *M)
             List_Read(p->Entities, j, &Num);
             MSH_LIN_NUM = abs(Num);
             MSH_PHYSICAL_ORI = sign(Num);
-            Tree_Action(pV->Simp_Surf, _msh_print_simplex);
-            Tree_Action(pV->Quad_Surf, _msh_print_quadrangle);
+            Tree_Action(pV->Lin_Surf, _msh_print_simplex);
           }
         }
       }
@@ -512,6 +576,12 @@ static void _msh_print_all_simpsurf(void *a, void *b)
   Tree_Action(v->Quad_Surf, _msh_print_quadrangle);
 }
 
+static void _msh_print_all_linsurf(void *a, void *b)
+{
+  Volume *v = *(Volume **) a;
+  Tree_Action(v->Lin_Surf, _msh_print_simplex);
+}
+
 static void _msh_print_all_volumes(void *a, void *b)
 {
   Volume *v = *(Volume **) a;
@@ -530,6 +600,7 @@ static void _msh_print_all_elements(Mesh *M)
 
   if(CTX.mesh.oldxtrude) {
     Tree_Action(M->Volumes, _msh_print_all_simpsurf);
+    Tree_Action(M->Volumes, _msh_print_all_linsurf);
   }
   else {
     Tree_Action(M->Curves, _msh_print_all_curves);
@@ -583,7 +654,7 @@ void Print_Mesh_MSH(Mesh *M, FILE *fp)
   else if(CTX.mesh.msh_file_version == 2.0){
     fprintf(MSHFILE, "$MeshFormat\n");
     fprintf(MSHFILE, "%g %d %d\n", CTX.mesh.msh_file_version,
-	    LIST_FORMAT_ASCII, sizeof(double));
+	    LIST_FORMAT_ASCII, (int)sizeof(double));
     fprintf(MSHFILE, "$EndMeshFormat\n");
   }
   else{
@@ -682,100 +753,64 @@ static void _unv_print_record(int num, int fetyp, int geo, int n, int nsup,
 static void _unv_process_1D_elements(Mesh *m)
 {
   List_T *ListCurves = Tree2List(m->Curves);
-  List_T *AllCurves = List_Create(2, 2, sizeof(Surface *));
-  List_T *ListSurfaces = Tree2List(m->Surfaces);
   List_T *Elements;
   SimplexBase *sx;
   Curve *c;
-  Surface *surf;
-
-  for(int i = 0; i < List_Nbr(ListSurfaces); i++) {
-    List_Read(ListSurfaces, i, &surf);
-    for(int j = 0; j < List_Nbr(surf->Generatrices); j++) {
-      List_Read(surf->Generatrices, j, &c);
-      if(Tree_Nbr(c->Simplexes) || Tree_Nbr(c->SimplexesBase))
-        List_Add(AllCurves, &c);
-      c = FindCurve(-c->Num, m);
-      if(Tree_Nbr(c->Simplexes) || Tree_Nbr(c->SimplexesBase))
-        List_Add(AllCurves, &c);
-    }
-  }
 
   for(int i = 0; i < List_Nbr(ListCurves); i++) {
     List_Read(ListCurves, i, &c);
-    if(!List_Search(AllCurves, &c, compareCurve)) {
-      for(int simtype = 0; simtype < 2; simtype ++){
-	Elements = (!simtype) ? Tree2List(c->Simplexes) : Tree2List(c->SimplexesBase);
-	for(int j = 0; j < List_Nbr(Elements); j++) {
-	  List_Read(Elements, j, &sx);
-	  if(sx->VSUP)
-	    _unv_print_record(sx->Num, BEAM2, c->Num, 2, 2, &sx->V[0], sx->VSUP);
-	  else 
-	    _unv_print_record(sx->Num, BEAM, c->Num, 2, 0, &sx->V[0], NULL);
-	}
-	List_Delete(Elements);
+    for(int simtype = 0; simtype < 2; simtype ++){
+      Elements = (!simtype) ? Tree2List(c->Simplexes) : Tree2List(c->SimplexesBase);
+      for(int j = 0; j < List_Nbr(Elements); j++) {
+	List_Read(Elements, j, &sx);
+	if(sx->VSUP)
+	  _unv_print_record(sx->Num, BEAM2, c->Num, 2, 1, &sx->V[0], sx->VSUP);
+	else 
+	  _unv_print_record(sx->Num, BEAM, c->Num, 2, 0, &sx->V[0], NULL);
       }
+      List_Delete(Elements);
     }
   }
 
-  List_Delete(AllCurves);
-  List_Delete(ListSurfaces);
   List_Delete(ListCurves);
 }
 
 static void _unv_process_2D_elements(Mesh *m)
 {
   List_T *ListSurfaces = Tree2List(m->Surfaces);
-  List_T *AllSurfaces = List_Create(2, 2, sizeof(Surface *));
-  List_T *ListVolumes = Tree2List(m->Volumes);
   List_T *Elements;
-  Volume *vol;
   Surface *s;
   SimplexBase *sx;
   Quadrangle *qx;
 
-  for(int i = 0; i < List_Nbr(ListVolumes); i++) {
-    List_Read(ListVolumes, i, &vol);
-    for(int j = 0; j < List_Nbr(vol->Surfaces); j++) {
-      List_Read(vol->Surfaces, j, &s);
-      if(Tree_Nbr(s->Simplexes) || Tree_Nbr(s->SimplexesBase) ||
-	 Tree_Nbr(s->Quadrangles))
-        List_Add(AllSurfaces, &s);
-    }
-  }
-
   for(int i = 0; i < List_Nbr(ListSurfaces); i++) {
     List_Read(ListSurfaces, i, &s);
-    if(!List_Search(AllSurfaces, &s, compareSurface)) {
       
-      // triangles
-      for(int simtype = 0; simtype < 2; simtype++){
-	Elements = (!simtype) ? Tree2List(s->Simplexes) : Tree2List(s->SimplexesBase);
-	for(int j = 0; j < List_Nbr(Elements); j++) {
-	  List_Read(Elements, j, &sx);
-	  if(sx->VSUP)
-	    _unv_print_record(abs(sx->Num), THINSHLL, s->Num, 3, 3, &sx->V[0], sx->VSUP);
-	  else
-	    _unv_print_record(abs(sx->Num), THINSHLL, s->Num, 3, 0, &sx->V[0], NULL);
-	}
-	List_Delete(Elements);
-      }
-
-      // quadrangles
-      Elements = Tree2List(s->Quadrangles);
+    // triangles
+    for(int simtype = 0; simtype < 2; simtype++){
+      Elements = (!simtype) ? Tree2List(s->Simplexes) : Tree2List(s->SimplexesBase);
       for(int j = 0; j < List_Nbr(Elements); j++) {
-        List_Read(Elements, j, &qx);
-	if(qx->VSUP)
-	  _unv_print_record(abs(qx->Num), QUAD, s->Num, 4, 4+1, &qx->V[0], qx->VSUP);
+	List_Read(Elements, j, &sx);
+	if(sx->VSUP)
+	  _unv_print_record(abs(sx->Num), THINSHLL, s->Num, 3, 3, &sx->V[0], sx->VSUP);
 	else
-	  _unv_print_record(abs(qx->Num), QUAD2, s->Num, 4, 0, &qx->V[0], NULL);
+	  _unv_print_record(abs(sx->Num), THINSHLL, s->Num, 3, 0, &sx->V[0], NULL);
       }
       List_Delete(Elements);
     }
+    
+    // quadrangles
+    Elements = Tree2List(s->Quadrangles);
+    for(int j = 0; j < List_Nbr(Elements); j++) {
+      List_Read(Elements, j, &qx);
+      if(qx->VSUP)
+	_unv_print_record(abs(qx->Num), QUAD, s->Num, 4, 4+1, &qx->V[0], qx->VSUP);
+      else
+	_unv_print_record(abs(qx->Num), QUAD2, s->Num, 4, 0, &qx->V[0], NULL);
+    }
+    List_Delete(Elements);
   }
   List_Delete(ListSurfaces);
-  List_Delete(ListVolumes);
-  List_Delete(AllSurfaces);
 }
 
 static void _unv_process_3D_elements(Mesh *m)
@@ -968,7 +1003,7 @@ void Print_Mesh_UNV(Mesh *M, FILE *fp)
   ELEMENT_ID = 1;
   _unv_process_3D_elements(M);
   _unv_process_2D_elements(M);
-  if(0) _unv_process_1D_elements(M);
+  _unv_process_1D_elements(M);
   fprintf(UNVFILE, "%6d\n", -1);
   _unv_process_groups(M);
 }
@@ -1617,7 +1652,6 @@ int _p3d_cmp_entities(const void *a, const void *b)
   return (*e1)->iEnt - (*e2)->iEnt;
 }
 
-
 int _p3d_cmp_surf_num(const void *a, const void *b)
 {
   Surface **e1 = (Surface **) a;
@@ -1668,7 +1702,6 @@ void _p3d_print_hex(List_T *ListHex, int Nu, int Nv, int Nw)
   double coord;
   Hexahedron *pH;
 
-  
   for (c = 0; c < 3; c++) {
     for (k = 0; k < (Nu - 1); k++) {
       for (j = 0; j < (Nv - 1); j++) {
