@@ -1,4 +1,4 @@
-// $Id: Read_Mesh.cpp,v 1.86 2005/03/15 15:36:06 geuzaine Exp $
+// $Id: Read_Mesh.cpp,v 1.95 2005/10/16 03:15:31 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -26,7 +26,9 @@
 #include "3D_Mesh.h"
 #include "Create.h"
 #include "MinMax.h"
+#include "Numeric.h"
 #include "Context.h"
+#include "PartitionMesh.h"
 
 extern Context_T CTX;
 
@@ -50,31 +52,14 @@ extern Context_T CTX;
 
 #define NB_NOD_MAX_ELM 30
 
-void addPhysicalGroup(Mesh * M, int Type, int Physical, int Elementary)
-{
-  PhysicalGroup *pg;
-  if((pg = FindPhysicalGroup(Physical, Type, M))) {
-    List_Insert(pg->Entities, &Elementary, fcmp_int);
-  }
-  else {
-    List_T *tmp = List_Create(1, 1, sizeof(int));
-    List_Add(tmp, &Elementary);
-    pg = Create_PhysicalGroup(Physical, Type, tmp);
-    List_Add(M->PhysicalGroups, &pg);
-    List_Delete(tmp);
-  }
-}
-
-/* Note: the "Dirty" flag only has an influence if one doesn't load
-   the geometry along with the mesh (since we only add "dirty"
-   geometrical entities if they don't already exist). */
+// If a "normal" elementary entity does not exist, we create a
+// "discrete" entity, i.e., an entity entirely defined by its mesh
 
 Curve *addElementaryCurve(Mesh * M, int Num)
 {
   Curve *c;
   if(!(c = FindCurve(Num, M))) {
-    c = Create_Curve(Num, MSH_SEGM_LINE, 0, NULL, NULL, -1, -1, 0., 1.);
-    c->Dirty = 1;
+    c = Create_Curve(Num, MSH_SEGM_DISCRETE, 0, NULL, NULL, -1, -1, 0., 1.);
     Tree_Add(M->Curves, &c);
   }
   return c;
@@ -84,8 +69,7 @@ Surface *addElementarySurface(Mesh * M, int Num)
 {
   Surface *s;
   if(!(s = FindSurface(Num, M))) {
-    s = Create_Surface(Num, MSH_SURF_PLAN);
-    s->Dirty = 1;
+    s = Create_Surface(Num, MSH_SURF_DISCRETE);
     Tree_Add(M->Surfaces, &s);
   }
   return s;
@@ -95,13 +79,51 @@ Volume *addElementaryVolume(Mesh * M, int Num)
 {
   Volume *v;
   if(!(v = FindVolume(Num, M))) {
-    v = Create_Volume(Num, MSH_VOLUME);
-    v->Dirty = 1;
+    v = Create_Volume(Num, MSH_VOLUME_DISCRETE);
     Tree_Add(M->Volumes, &v);
   }
   return v;
 }
 
+void addPhysicalGroup(Tree_T * groups, int Type, int Physical, int Elementary)
+{
+  // we add in a temporary group tree for performance reasons; the
+  // tree is converted back into a list in the mesh at the end of read_mesh
+  PhysicalGroup g, *pg;
+  pg = &g;
+  pg->Num = Physical;
+  pg->Typ = Type;
+  if(Tree_Query(groups, &pg)) {
+    List_Insert(pg->Entities, &Elementary, fcmp_int);
+  }
+  else {
+    List_T *tmp = List_Create(1, 1, sizeof(int));
+    List_Add(tmp, &Elementary);
+    pg = Create_PhysicalGroup(Physical, Type, tmp);
+    Tree_Add(groups, &pg);
+    List_Delete(tmp);
+  }
+}
+
+int addMeshPartition(int Num, Mesh * M)
+{
+  MeshPartition P, *p, **pp;
+  p = &P;
+  p->Num = Num;
+  if((pp = (MeshPartition**)List_PQuery(M->Partitions, &p, compareMeshPartitionNum))){
+    return (*pp)->Index;
+  }
+  else{
+    p = (MeshPartition*)Malloc(sizeof(MeshPartition));
+    p->Num = Num;
+    p->Visible = VIS_GEOM | VIS_MESH;
+    p->Index = List_Nbr(M->Partitions);
+    List_Add(M->Partitions, &p);
+    return p->Index;
+  }
+  return 0;
+}
+
 int getNbrNodes(int Type)
 {
   switch (Type) {
@@ -124,6 +146,23 @@ int getNbrNodes(int Type)
   }
 }
 
+double SetLC(Vertex *v1, Vertex *v2, Vertex *v3, Vertex *v4 = 0)
+{ 
+  double lc1 = sqrt((v1->Pos.X - v2->Pos.X) * (v1->Pos.X - v2->Pos.X) +
+		    (v1->Pos.Y - v2->Pos.Y) * (v1->Pos.Y - v2->Pos.Y) +
+		    (v1->Pos.Z - v2->Pos.Z) * (v1->Pos.Z - v2->Pos.Z));
+  double lc2 = sqrt((v1->Pos.X - v3->Pos.X) * (v1->Pos.X - v3->Pos.X) +
+		    (v1->Pos.Y - v3->Pos.Y) * (v1->Pos.Y - v3->Pos.Y) +
+		    (v1->Pos.Z - v3->Pos.Z) * (v1->Pos.Z - v3->Pos.Z));
+  double lc3 = sqrt((v2->Pos.X - v3->Pos.X) * (v2->Pos.X - v3->Pos.X) +
+		    (v2->Pos.Y - v3->Pos.Y) * (v2->Pos.Y - v3->Pos.Y) +
+		    (v2->Pos.Z - v3->Pos.Z) * (v2->Pos.Z - v3->Pos.Z));
+  double lc = DMAX(lc1, DMAX(lc2, lc3)) * CTX.mesh.lc_factor;
+  v1->lc = v2->lc = v3->lc = lc;
+  if(v4) v4->lc = lc;
+  return lc;
+}
+
 void Read_Mesh_MSH(Mesh * M, FILE * fp)
 {
   char String[256];
@@ -143,6 +182,7 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
   Surface *s;
   Volume *v;
   Tree_T *Duplicates = NULL;
+  Tree_T *groups = List2Tree(M->PhysicalGroups, comparePhysicalGroup);
 
   while(1) {
     do {
@@ -251,7 +291,7 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
 	if(version <= 1.0){
 	  fscanf(fp, "%d %d %d %d %d",
 		 &Num, &Type, &Physical, &Elementary, &Nbr_Nodes);
-	  Partition = Physical;
+	  Partition = 1;
 	  int Nbr_Nodes_Check = getNbrNodes(Type);
 	  if(!Nbr_Nodes_Check){
 	    Msg(GERROR, "Unknown type for element %d", Num); 
@@ -314,11 +354,11 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
         case LGN1:
         case LGN2:
 	  c = addElementaryCurve(M, abs(Elementary));
-	  addPhysicalGroup(M, MSH_PHYSICAL_LINE, Physical, abs(Elementary));
+	  addPhysicalGroup(groups, MSH_PHYSICAL_LINE, Physical, abs(Elementary));
           simp = Create_SimplexBase(vertsp[0], vertsp[1], NULL, NULL);
           simp->Num = Num;
           simp->iEnt = Elementary;
-          simp->iPart = Add_MeshPartition(Partition, M);
+          simp->iPart = addMeshPartition(Partition, M);
 	  if(Type == LGN2){
 	    simp->VSUP = (Vertex **) Malloc(1 * sizeof(Vertex *));
 	    simp->VSUP[0] = vertsp[2];
@@ -329,6 +369,7 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
 	    Free_SimplexBase(&simp, 0);
 	  }
 	  else{
+	    // this can be quite slow if there are many nodes on the curve...
 	    for(i = 0; i < Nbr_Nodes; i++)
 	      List_Insert(c->Vertices, &vertsp[i], fcmp_int);
 	  }
@@ -336,11 +377,12 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
         case TRI1:
         case TRI2:
 	  s = addElementarySurface(M, Elementary);
-	  addPhysicalGroup(M, MSH_PHYSICAL_SURFACE, Physical, Elementary);
+	  addPhysicalGroup(groups, MSH_PHYSICAL_SURFACE, Physical, Elementary);
           simp = Create_SimplexBase(vertsp[0], vertsp[1], vertsp[2], NULL);
           simp->Num = Num;
           simp->iEnt = Elementary;
-          simp->iPart = Add_MeshPartition(Partition, M);
+          simp->iPart = addMeshPartition(Partition, M);
+	  SetLC(vertsp[0], vertsp[1], vertsp[2]);
 	  if(Type == TRI2){
 	    simp->VSUP = (Vertex **) Malloc(3 * sizeof(Vertex *));
 	    for(i = 0; i < 3; i++){
@@ -360,11 +402,12 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
         case QUA1:
         case QUA2:
 	  s = addElementarySurface(M, Elementary);
-	  addPhysicalGroup(M, MSH_PHYSICAL_SURFACE, Physical, Elementary);
+	  addPhysicalGroup(groups, MSH_PHYSICAL_SURFACE, Physical, Elementary);
           quad = Create_Quadrangle(vertsp[0], vertsp[1], vertsp[2], vertsp[3]);
           quad->Num = Num;
           quad->iEnt = Elementary;
-          quad->iPart = Add_MeshPartition(Partition, M);
+          quad->iPart = addMeshPartition(Partition, M);
+	  SetLC(vertsp[0], vertsp[1], vertsp[2], vertsp[3]);
 	  if(Type == QUA2){
 	    quad->VSUP = (Vertex **) Malloc((4 + 1) * sizeof(Vertex *));
 	    for(i = 0; i < 4 + 1; i++){
@@ -384,11 +427,11 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
         case TET1:
         case TET2:
 	  v = addElementaryVolume(M, Elementary);
-	  addPhysicalGroup(M, MSH_PHYSICAL_VOLUME, Physical, Elementary);
+	  addPhysicalGroup(groups, MSH_PHYSICAL_VOLUME, Physical, Elementary);
           simp = Create_SimplexBase(vertsp[0], vertsp[1], vertsp[2], vertsp[3]);
           simp->Num = Num;
           simp->iEnt = Elementary;
-          simp->iPart = Add_MeshPartition(Partition, M);
+          simp->iPart = addMeshPartition(Partition, M);
 	  if(Type == TET2){
 	    simp->VSUP = (Vertex **) Malloc(6 * sizeof(Vertex *));
 	    for(i = 0; i < 6; i++){
@@ -410,12 +453,12 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
         case HEX1:
         case HEX2:
 	  v = addElementaryVolume(M, Elementary);
-	  addPhysicalGroup(M, MSH_PHYSICAL_VOLUME, Physical, Elementary);
+	  addPhysicalGroup(groups, MSH_PHYSICAL_VOLUME, Physical, Elementary);
           hex = Create_Hexahedron(vertsp[0], vertsp[1], vertsp[2], vertsp[3],
                                   vertsp[4], vertsp[5], vertsp[6], vertsp[7]);
           hex->Num = Num;
           hex->iEnt = Elementary;
-          hex->iPart = Add_MeshPartition(Partition, M);
+          hex->iPart = addMeshPartition(Partition, M);
 	  if(Type == HEX2){
 	    hex->VSUP = (Vertex **) Malloc((12 + 6 + 1) * sizeof(Vertex *));
 	    for(i = 0; i < 12 + 6 + 1; i++){
@@ -437,12 +480,12 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
         case PRI1:
         case PRI2:
 	  v = addElementaryVolume(M, Elementary);
-	  addPhysicalGroup(M, MSH_PHYSICAL_VOLUME, Physical, Elementary);
+	  addPhysicalGroup(groups, MSH_PHYSICAL_VOLUME, Physical, Elementary);
           pri = Create_Prism(vertsp[0], vertsp[1], vertsp[2],
                              vertsp[3], vertsp[4], vertsp[5]);
           pri->Num = Num;
           pri->iEnt = Elementary;
-          pri->iPart = Add_MeshPartition(Partition, M);
+          pri->iPart = addMeshPartition(Partition, M);
 	  if(Type == PRI2){
 	    pri->VSUP = (Vertex **) Malloc((9 + 3) * sizeof(Vertex *));
 	    for(i = 0; i < 9 + 3; i++){
@@ -464,12 +507,12 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
         case PYR1:
         case PYR2:
 	  v = addElementaryVolume(M, Elementary);
-	  addPhysicalGroup(M, MSH_PHYSICAL_VOLUME, Physical, Elementary);
+	  addPhysicalGroup(groups, MSH_PHYSICAL_VOLUME, Physical, Elementary);
           pyr = Create_Pyramid(vertsp[0], vertsp[1], vertsp[2],
                                vertsp[3], vertsp[4]);
           pyr->Num = Num;
           pyr->iEnt = Elementary;
-          pyr->iPart = Add_MeshPartition(Partition, M);
+          pyr->iPart = addMeshPartition(Partition, M);
 	  if(Type == PYR2){
 	    pyr->VSUP = (Vertex **) Malloc((8 + 1) * sizeof(Vertex *));
 	    for(i = 0; i < 8 + 1; i++){
@@ -489,7 +532,7 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
 #endif
           break;
         case PNT:
-	  addPhysicalGroup(M, MSH_PHYSICAL_POINT, Physical, Elementary);
+	  addPhysicalGroup(groups, MSH_PHYSICAL_POINT, Physical, Elementary);
 	  // we need to make a new one: vertices in M->Vertices and
 	  // M->Points should never point to the same memory location
 	  vert = Create_Vertex(vertsp[0]->Num, vertsp[0]->Pos.X, vertsp[0]->Pos.Y, 
@@ -540,6 +583,11 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
   // re-sort the list according to these indices to allow direct
   // access through List_Pointer & co.
   List_Sort(M->Partitions, compareMeshPartitionIndex);
+
+  // Transfer the temp group tree back into the mesh
+  List_Delete(M->PhysicalGroups);
+  M->PhysicalGroups = Tree2List(groups);
+  Tree_Delete(groups);
 }
 
 // Read mesh in VTK format
@@ -560,7 +608,6 @@ void Read_Mesh_VTK(Mesh * m, FILE *fp)
 
   sscanf(line, "%s %d %s", dumline1, &NbVertices, dumline2);
   Surface *surf = Create_Surface(1, MSH_SURF_DISCRETE);
-  surf->Dirty = 1;
   Tree_Add(m->Surfaces, &surf);
   for(i = 0; i < NbVertices; i++) {
     fscanf(fp, "%le %le %le", &x, &y, &z);
@@ -592,7 +639,6 @@ void Read_Mesh_VTK(Mesh * m, FILE *fp)
     }
     if(!(surf = FindSurface(1, m))) {
       surf = Create_Surface(1, MSH_SURF_DISCRETE);
-      surf->Dirty = 1;
       Tree_Add(m->Surfaces, &surf);
     }
     Tree_Add(surf->Simplexes, &s);
@@ -601,8 +647,7 @@ void Read_Mesh_VTK(Mesh * m, FILE *fp)
   if(NbFaces)
     m->status = 2;
 
-  Volume *vol = Create_Volume(1, MSH_VOLUME);
-  vol->Dirty = 1;
+  Volume *vol = Create_Volume(1, MSH_VOLUME_DISCRETE);
   vol->Surfaces = List_Create(1, 1, sizeof(Surface *));
   List_Add(vol->Surfaces, &surf);
   Tree_Add(m->Volumes, &vol);
@@ -699,7 +744,6 @@ void Read_Mesh_SMS(Mesh * m, FILE * in)
         Curve *c;
         if(!(c = FindCurve(GEntityId, m))) {
           c = Create_Curve(GEntityId, MSH_SEGM_DISCRETE, 1, NULL, NULL, -1, -1, 0, 1);
-	  c->Dirty = 1;
           Tree_Add(m->Curves, &c);
         }
         s->iEnt = GEntityId;
@@ -711,8 +755,7 @@ void Read_Mesh_SMS(Mesh * m, FILE * in)
 
   AllFaces = List_Create(NbFaces, 1, sizeof(Simplex *));
 
-  Volume *vol = Create_Volume(1, MSH_VOLUME);
-  vol->Dirty = 1;
+  Volume *vol = Create_Volume(1, MSH_VOLUME_DISCRETE);
   vol->Surfaces = List_Create(1, 1, sizeof(Surface *));
   Tree_Add(m->Volumes, &vol);
 
@@ -722,24 +765,19 @@ void Read_Mesh_SMS(Mesh * m, FILE * in)
     if(GEntityId) {
       fscanf(in, "%d %d", &GEntityType, &NbEdgesOnFace);
 
-      List_T *Lists[4] = { 0, 0, 0, 0 };
-
       if(NbEdgesOnFace == 3) {
         fscanf(in, "%d %d %d %d", &Edge1, &Edge2, &Edge3, &nbPts);
         List_Read(AllEdges, abs(Edge1) - 1, &e);
-        Lists[0] = e.Points;
         if(Edge1 > 0)
           v1 = e.V[0];
         else
           v1 = e.V[1];
         List_Read(AllEdges, abs(Edge2) - 1, &e);
-        Lists[1] = e.Points;
         if(Edge2 > 0)
           v2 = e.V[0];
         else
           v2 = e.V[1];
         List_Read(AllEdges, abs(Edge3) - 1, &e);
-        Lists[2] = e.Points;
         if(Edge3 > 0)
           v3 = e.V[0];
         else
@@ -800,7 +838,6 @@ void Read_Mesh_SMS(Mesh * m, FILE * in)
       case ENTITY_FACE:
         if(!(surf = FindSurface(GEntityId + 10000, m))) {
           surf = Create_Surface(GEntityId + 10000, MSH_SURF_DISCRETE);
-          surf->Dirty = 1;
           if(!NbRegions)
             List_Add(vol->Surfaces, &surf);
           Tree_Add(m->Surfaces, &surf);
@@ -843,8 +880,7 @@ void Read_Mesh_SMS(Mesh * m, FILE * in)
       Simplex *s = Create_Simplex(v1, v2, v3, v4);
 
       if(!(vol = FindVolume(GEntityId, m))) {
-        vol = Create_Volume(GEntityId, MSH_VOLUME);
-	vol->Dirty = 1;
+        vol = Create_Volume(GEntityId, MSH_VOLUME_DISCRETE);
         Tree_Add(m->Volumes, &vol);
       }
       s->iEnt = GEntityId;
@@ -893,4 +929,7 @@ void Read_Mesh(Mesh * M, FILE * fp, char *filename, int type)
     Msg(INFO, "Read mesh file '%s'", filename);
     Msg(STATUS2N, "Read '%s'", filename);
   }
+
+  if(CTX.mesh.nbPartitions != 1)
+    PartitionMesh(M, CTX.mesh.nbPartitions);
 }
