@@ -1,4 +1,4 @@
-// $Id: Simplex.cpp,v 1.38 2005/01/01 19:35:31 geuzaine Exp $
+// $Id: Simplex.cpp,v 1.44 2005/07/03 08:02:24 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -172,45 +172,40 @@ double SimplexBase::maxEdge()
   return maxlij;
 }
 
+double SimplexBase::minEdge()
+{
+  double minlij = 1.e25;
+  int N = V[3] ? 6 : 3;
+
+  if(V[3] || V[2])
+    for(int i = 0; i < N; i++)
+      minlij = DMIN(minlij, lij(V[edges_tetra[i][0]], V[edges_tetra[i][1]]));
+  else if(V[1])
+    minlij = lij(V[0], V[1]);
+
+  return minlij;
+}
+
 double SimplexBase::EtaShapeMeasure()
 {
-  int i, j;
+  if(!V[3]) return 0.;
+
   double lij2 = 0.0;
-  for(i = 0; i <= 3; i++) {
-    for(j = i + 1; j <= 3; j++) {
+  for(int i = 0; i <= 3; i++) {
+    for(int j = i + 1; j <= 3; j++) {
       lij2 += DSQR(lij(V[i], V[j]));
     }
   }
   return 12. * pow(9. / 10. * DSQR(fabs(Volume_Simplexe())), 1./3.) / (lij2);
 }
 
-double SimplexBase::RhoShapeMeasure()
-{
-  int i, j;
-  double minlij = 1.e25, maxlij = 0.0;
-  for(i = 0; i <= 3; i++) {
-    for(j = i + 1; j <= 3; j++) {
-      if(i != j) {
-        minlij = DMIN(minlij, fabs(lij(V[i], V[j])));
-        maxlij = DMAX(maxlij, fabs(lij(V[i], V[j])));
-      }
-    }
-  }
-  return minlij / maxlij;
-}
-
 double SimplexBase::GammaShapeMeasure()
 {
-  int i, j, N;
+  if(!V[3]) return 0.;
+
   double maxlij = 0.0;
-
-  if(V[3])
-    N = 4;
-  else
-    N = 3;
-
-  for(i = 0; i <= N - 1; i++) {
-    for(j = i + 1; j <= N - 1; j++) {
+  for(int i = 0; i <= 3; i++) {
+    for(int j = i + 1; j <= 3; j++) {
       if(i != j)
         maxlij = DMAX(maxlij, lij(V[i], V[j]));
     }
@@ -218,24 +213,78 @@ double SimplexBase::GammaShapeMeasure()
   return 12. * rhoin() / (sqrt(6.) * maxlij);
 }
 
-void SimplexBase::ExportLcField(FILE * f)
+void SimplexBase::ExportStatistics(FILE * f)
 {
-  if(!V[2])
-    fprintf(f, "SL(%.16g,%.16g,%.16g,%.16g,%.16g,%.16g){%.16g,%.16g};\n",
-            V[0]->Pos.X, V[0]->Pos.Y, V[0]->Pos.Z, V[1]->Pos.X, V[1]->Pos.Y,
-            V[1]->Pos.Z, V[0]->lc, V[1]->lc);
-  else if(!V[3])
-    fprintf(f, "ST(%.16g,%.16g,%.16g,%.16g,%.16g,%.16g,%.16g,%.16g,%.16g)"
-	    "{%.16g,%.16g,%.16g};\n",
-            V[0]->Pos.X, V[0]->Pos.Y, V[0]->Pos.Z, V[1]->Pos.X, V[1]->Pos.Y,
-            V[1]->Pos.Z, V[2]->Pos.X, V[2]->Pos.Y, V[2]->Pos.Z, V[0]->lc,
-            V[1]->lc, V[2]->lc);
-  else
-    fprintf(f, "SS(%.16g,%.16g,%.16g,%.16g,%.16g,%.16g,%.16g,%.16g,%.16g,"
-	    "%.16g,%.16g,%.16g){%.16g,%.16g,%.16g,%.16g};\n",
-	    V[0]->Pos.X, V[0]->Pos.Y, V[0]->Pos.Z, V[1]->Pos.X, V[1]->Pos.Y,
-	    V[1]->Pos.Z, V[2]->Pos.X, V[2]->Pos.Y, V[2]->Pos.Z, V[3]->Pos.X,
-	    V[3]->Pos.Y, V[3]->Pos.Z, V[0]->lc, V[1]->lc, V[2]->lc, V[3]->lc);
+  int N;
+
+  if(!V[2]){
+    if(!VSUP){
+      N = 2;
+      fprintf(f, "SL(%g,%g,%g,%g,%g,%g){%g,%g",
+	      V[0]->Pos.X, V[0]->Pos.Y, V[0]->Pos.Z, V[1]->Pos.X, V[1]->Pos.Y,
+	      V[1]->Pos.Z, V[0]->lc, V[1]->lc);
+    }
+    else{
+      N = 3;
+      fprintf(f, "SL2(%g,%g,%g,%g,%g,%g,%g,%g,%g){%g,%g,%g",
+	      V[0]->Pos.X, V[0]->Pos.Y, V[0]->Pos.Z, V[1]->Pos.X, V[1]->Pos.Y,
+	      V[1]->Pos.Z, VSUP[0]->Pos.X, VSUP[0]->Pos.Y, VSUP[0]->Pos.Z,
+	      V[0]->lc, V[1]->lc, VSUP[0]->lc);
+    }
+  }
+  else if(!V[3]){
+    if(!VSUP){
+      N = 3;
+      fprintf(f, "ST(%g,%g,%g,%g,%g,%g,%g,%g,%g){%g,%g,%g",
+	      V[0]->Pos.X, V[0]->Pos.Y, V[0]->Pos.Z, V[1]->Pos.X, V[1]->Pos.Y,
+	      V[1]->Pos.Z, V[2]->Pos.X, V[2]->Pos.Y, V[2]->Pos.Z, V[0]->lc,
+	      V[1]->lc, V[2]->lc);
+    }
+    else{
+      N = 6;
+      fprintf(f, "ST2(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g)"
+	      "{%g,%g,%g,%g,%g,%g",
+	      V[0]->Pos.X, V[0]->Pos.Y, V[0]->Pos.Z, V[1]->Pos.X, V[1]->Pos.Y,
+	      V[1]->Pos.Z, V[2]->Pos.X, V[2]->Pos.Y, V[2]->Pos.Z, VSUP[0]->Pos.X, 
+	      VSUP[0]->Pos.Y, VSUP[0]->Pos.Z, VSUP[1]->Pos.X, VSUP[1]->Pos.Y,
+	      VSUP[1]->Pos.Z, VSUP[2]->Pos.X, VSUP[2]->Pos.Y, VSUP[2]->Pos.Z, 
+	      V[0]->lc, V[1]->lc, V[2]->lc, VSUP[0]->lc, VSUP[1]->lc, VSUP[2]->lc);
+    }
+  }
+  else{
+    if(!VSUP){
+      N = 4;
+      fprintf(f, "SS(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g){%g,%g,%g,%g",
+	      V[0]->Pos.X, V[0]->Pos.Y, V[0]->Pos.Z, V[1]->Pos.X, V[1]->Pos.Y,
+	      V[1]->Pos.Z, V[2]->Pos.X, V[2]->Pos.Y, V[2]->Pos.Z, V[3]->Pos.X,
+	      V[3]->Pos.Y, V[3]->Pos.Z, V[0]->lc, V[1]->lc, V[2]->lc, V[3]->lc);
+    }
+    else{
+      N = 10;
+      fprintf(f, "SS2(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,"
+	      "%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g){%g,%g,%g,%g,%g,%g,%g,%g,%g,%g",
+	      V[0]->Pos.X, V[0]->Pos.Y, V[0]->Pos.Z, V[1]->Pos.X, V[1]->Pos.Y,
+	      V[1]->Pos.Z, V[2]->Pos.X, V[2]->Pos.Y, V[2]->Pos.Z, V[3]->Pos.X,
+	      V[3]->Pos.Y, V[3]->Pos.Z, VSUP[0]->Pos.X, VSUP[0]->Pos.Y, VSUP[0]->Pos.Z, 
+	      VSUP[1]->Pos.X, VSUP[1]->Pos.Y, VSUP[1]->Pos.Z, VSUP[2]->Pos.X, 
+	      VSUP[2]->Pos.Y, VSUP[2]->Pos.Z, VSUP[3]->Pos.X, VSUP[3]->Pos.Y, 
+	      VSUP[3]->Pos.Z, VSUP[4]->Pos.X, VSUP[4]->Pos.Y, VSUP[4]->Pos.Z, 
+	      VSUP[5]->Pos.X, VSUP[5]->Pos.Y, VSUP[5]->Pos.Z, V[0]->lc, V[1]->lc, 
+	      V[2]->lc, V[3]->lc, VSUP[0]->lc, VSUP[1]->lc, VSUP[2]->lc, VSUP[3]->lc,
+	      VSUP[4]->lc, VSUP[5]->lc);
+    }
+  }
+
+  double g = GammaShapeMeasure();
+  double e = EtaShapeMeasure();
+  double r = RhoShapeMeasure();
+  double n = Num;
+  for(int i = 0; i < N; i++) fprintf(f, ",%g", g);
+  for(int i = 0; i < N; i++) fprintf(f, ",%g", e);
+  for(int i = 0; i < N; i++) fprintf(f, ",%g", r);
+  for(int i = 0; i < N; i++) fprintf(f, ",%g", n);
+  
+  fprintf(f, "};\n");
 }
 
 SimplexBase *Create_SimplexBase(Vertex * v1, Vertex * v2, Vertex * v3, Vertex * v4)
@@ -454,6 +503,32 @@ int compareSimplex(const void *a, const void *b)
   return (q->Num - w->Num);
 }
 
+int compareSimplexBarycenter(const void *a, const void *b)
+{
+  Simplex *q = *(Simplex **) a;
+  Simplex *w = *(Simplex **) b;
+  int nq = q->V[3] ? 4 : q->V[2] ? 3 : q->V[1] ? 2 : 1;
+  int nw = w->V[3] ? 4 : w->V[2] ? 3 : w->V[1] ? 2 : 1;
+  double bq, bw, TOL = CTX.lc * 1.e-6;
+
+  bq = bw = 0.;
+  for(int i = 0; i < nq; i++) bq += q->V[i]->Pos.X;
+  for(int i = 0; i < nw; i++) bw += w->V[i]->Pos.X;
+  if(bq-bw > TOL) return 1; else if(bq-bw < -TOL) return -1;
+
+  bq = bw = 0.;
+  for(int i = 0; i < nq; i++) bq += q->V[i]->Pos.Y;
+  for(int i = 0; i < nw; i++) bw += w->V[i]->Pos.Y;
+  if(bq-bw > TOL) return 1; else if(bq-bw < -TOL) return -1;
+
+  bq = bw = 0.;
+  for(int i = 0; i < nq; i++) bq += q->V[i]->Pos.Z;
+  for(int i = 0; i < nw; i++) bw += w->V[i]->Pos.Z;
+  if(bq-bw > TOL) return 1; else if(bq-bw < -TOL) return -1;
+
+  return 0;
+}
+
 int Simplex::Pt_In_Simplexe(Vertex * v, double uvw[3], double tol)
 {
   double mat[3][3];
@@ -774,7 +849,7 @@ bool Simplex::SwapEdge(int iFac)
 bool Simplex::SwapFace(int iFac, List_T * newsimp, List_T * delsimp)
 {
   Simplex *s = S[iFac], *s1, *s2, *s3;
-  Vertex *o[2];
+  Vertex *o[2] = {NULL, NULL};
   int i;
 
   if(!s || s == &MyNewBoundary || !s->iEnt)
@@ -813,3 +888,43 @@ bool Simplex::SwapFace(int iFac, List_T * newsimp, List_T * delsimp)
   return true;
 }
 
+// Transfer SimplexBase to Simplex
+
+static Tree_T *simptree = NULL;
+
+void Copy_SimplexBaseToSimplex(void *a, void *b)
+{
+  SimplexBase *base = *(SimplexBase**)a;
+  Simplex *s = Create_Simplex(base->V[0], base->V[1], base->V[2], base->V[3]);
+  s->iEnt = base->iEnt;
+  s->Num = base->Num;
+  s->iPart = base->iPart;
+  s->Visible = base->Visible;
+  if(base->VSUP){
+    int ns;
+    if(base->V[3])
+      ns = 6;
+    else if(base->V[2])
+      ns = 3;
+    else if(base->V[1])
+      ns = 1;
+    else
+      ns = 0;
+    if(ns){
+      s->VSUP = (Vertex **) Malloc(ns * sizeof(Vertex *));
+      memcpy(s->VSUP, base->VSUP, ns * sizeof(Vertex *));
+    }
+  }
+  Tree_Insert(simptree, &s);
+}
+
+void Move_SimplexBaseToSimplex(Tree_T **base, Tree_T *simp)
+{
+  if(!Tree_Nbr(*base)) return;
+  simptree = simp;
+  Tree_Action(*base, Copy_SimplexBaseToSimplex);
+  Tree_Action(*base, Free_SimplexBase);
+  Tree_Delete(*base);
+  *base = Tree_Create(sizeof(SimplexBase *), compareSimplexBase);
+}
+
