@@ -1,4 +1,4 @@
-// $Id: Annotate.cpp,v 1.11 2005/03/13 09:10:35 geuzaine Exp $
+// $Id: Annotate.cpp,v 1.13 2005/08/24 20:28:43 remacle Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -19,6 +19,8 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
+#include <iostream>
+#include <vector>
 #include "Plugin.h"
 #include "Annotate.h"
 #include "List.h"
