@@ -1,7 +1,7 @@
 #ifndef _CUT_MAP_H_
-#define _CUT_MAP_H
+#define _CUT_MAP_H_
 
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -39,6 +39,9 @@ class GMSH_CutMapPlugin : public GMSH_LevelsetPlugin
   StringXNumber* getOption (int iopt);  
   Post_View *execute (Post_View *);
   static double callbackA(int, int, double);
+  static double callbackVol(int, int, double);
+  static double callbackRecur(int, int, double);
+  static double callbackTarget(int, int, double);
 };
 
 #endif
