@@ -1,6 +1,6 @@
-// $Id: CutSphere.cpp,v 1.35 2004/11/09 16:27:53 remacle Exp $
+// $Id: CutSphere.cpp,v 1.44 2005/01/09 02:18:59 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -36,8 +36,9 @@ StringXNumber CutSphereOptions_Number[] = {
   {GMSH_FULLRC, "Yc", GMSH_CutSpherePlugin::callbackY, 0.},
   {GMSH_FULLRC, "Zc", GMSH_CutSpherePlugin::callbackZ, 0.},
   {GMSH_FULLRC, "R", GMSH_CutSpherePlugin::callbackR, 0.25},
-  {GMSH_FULLRC, "iView", NULL, -1.},
-  {GMSH_FULLRC, "recurLevel", NULL, 4}
+  {GMSH_FULLRC, "ExtractVolume", GMSH_CutSpherePlugin::callbackVol, 0.},
+  {GMSH_FULLRC, "RecurLevel", GMSH_CutSpherePlugin::callbackRecur, 4},
+  {GMSH_FULLRC, "iView", NULL, -1.}
 };
 
 extern "C"
@@ -67,6 +68,7 @@ void GMSH_CutSpherePlugin::draw()
   glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
   glPushMatrix();
   glColor4ubv((GLubyte *) & CTX.color.fg);
+  glLineWidth(CTX.line_width);
   glTranslated(CutSphereOptions_Number[0].def,
 	       CutSphereOptions_Number[1].def,
 	       CutSphereOptions_Number[2].def);
@@ -76,77 +78,61 @@ void GMSH_CutSpherePlugin::draw()
 #endif
 }
 
-void GMSH_CutSpherePlugin::callback()
+double GMSH_CutSpherePlugin::callback(int num, int action, double value, double *opt,
+				      double step, double min, double max)
 {
-#if defined(HAVE_FLTK)
-  CTX.post.plugin_draw_function = draw;
-  if(CTX.fast_redraw){
-    CTX.post.draw = 0;
-    CTX.mesh.draw = 0;
+  switch(action){ // configure the input field
+  case 1: return step;
+  case 2: return min;
+  case 3: return max;
+  default: break;
   }
-  if(!CTX.batch) 
-    Draw();
-  CTX.post.plugin_draw_function = NULL;
-  CTX.post.draw = 1;
-  CTX.mesh.draw = 1;
+  *opt = value;
+#if defined(HAVE_FLTK)
+  DrawPlugin(draw);
 #endif
+  return 0.;
 }
 
 double GMSH_CutSpherePlugin::callbackX(int num, int action, double value)
 {
-  switch(action){ // configure the input field
-  case 1: return CTX.lc/200.;
-  case 2: return -CTX.lc;
-  case 3: return CTX.lc;
-  default: break;
-  }
-  CutSphereOptions_Number[0].def = value;
-  callback();
-  return 0.;
+  return callback(num, action, value, &CutSphereOptions_Number[0].def,
+		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
 }
 
 double GMSH_CutSpherePlugin::callbackY(int num, int action, double value)
 {
-  switch(action){ // configure the input field
-  case 1: return CTX.lc/200.;
-  case 2: return -CTX.lc;
-  case 3: return CTX.lc;
-  default: break;
-  }
-  CutSphereOptions_Number[1].def = value;
-  callback();
-  return 0.;
+  return callback(num, action, value, &CutSphereOptions_Number[1].def,
+		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
 }
 
 double GMSH_CutSpherePlugin::callbackZ(int num, int action, double value)
 {
-  switch(action){ // configure the input field
-  case 1: return CTX.lc/200.;
-  case 2: return -CTX.lc;
-  case 3: return CTX.lc;
-  default: break;
-  }
-  CutSphereOptions_Number[2].def = value;
-  callback();
-  return 0.;
+  return callback(num, action, value, &CutSphereOptions_Number[2].def,
+		  CTX.lc/100., -2*CTX.lc, 2*CTX.lc);
 }
 
 double GMSH_CutSpherePlugin::callbackR(int num, int action, double value)
 {
-  switch(action){
-  case 1: return CTX.lc/200.;
-  case 2: return 0.;
-  case 3: return 2*CTX.lc;
-  default: break;
-  }
-  CutSphereOptions_Number[3].def = value;
-  callback();
-  return 0.;
+  return callback(num, action, value, &CutSphereOptions_Number[3].def,
+		  CTX.lc/100., 0., 2*CTX.lc);
+}
+
+double GMSH_CutSpherePlugin::callbackVol(int num, int action, double value)
+{
+  return callback(num, action, value, &CutSphereOptions_Number[4].def,
+		  1., -1., 1.);
+}
+
+double GMSH_CutSpherePlugin::callbackRecur(int num, int action, double value)
+{
+  return callback(num, action, value, &CutSphereOptions_Number[5].def,
+		  1, 0, 10);
 }
 
 void GMSH_CutSpherePlugin::getName(char *name) const
 {
-  strcpy(name, "Cut sphere");
+  strcpy(name, "Cut Sphere");
 }
 
 void GMSH_CutSpherePlugin::getInfos(char *author, char *copyright,
@@ -157,8 +143,10 @@ void GMSH_CutSpherePlugin::getInfos(char *author, char *copyright,
   strcpy(help_text,
          "Plugin(CutSphere) cuts the view `iView' with the\n"
 	 "sphere (X-`Xc')^2 + (Y-`Yc')^2 + (Z-`Zc')^2 = `R'^2.\n"
-	 "If `iView' < 0, the plugin is run on the current\n"
-	 "view.\n"
+	 "If `ExtractVolume' is nonzero, the plugin extracts\n"
+	 "the elements inside (if `ExtractVolume' < 0) or\n"
+	 "outside (if `ExtractVolume' > 0) the sphere. If\n"
+	 "`iView' < 0, the plugin is run on the current view.\n"
 	 "\n"
 	 "Plugin(CutSphere) creates one new view.\n");
 }
@@ -190,10 +178,11 @@ double GMSH_CutSpherePlugin::levelset(double x, double y, double z,
 
 Post_View *GMSH_CutSpherePlugin::execute(Post_View * v)
 {
-  int iView = (int)CutSphereOptions_Number[4].def;
+  int iView = (int)CutSphereOptions_Number[6].def;
   _ref[0] = CutSphereOptions_Number[0].def;
   _ref[1] = CutSphereOptions_Number[1].def;
   _ref[2] = CutSphereOptions_Number[2].def;
+  _extractVolume = (int)CutSphereOptions_Number[4].def;
   _recurLevel = (int)CutSphereOptions_Number[5].def;
 
   _valueIndependent = 1;
@@ -209,7 +198,7 @@ Post_View *GMSH_CutSpherePlugin::execute(Post_View * v)
     return v;
   }
 
-  Post_View *v1 = (Post_View*)List_Pointer(CTX.post.list, iView);
+  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
 
   return GMSH_LevelsetPlugin::execute(v1);
 }
