@@ -1,6 +1,6 @@
-// $Id: DisplacementRaise.cpp,v 1.16 2004/07/05 15:20:06 geuzaine Exp $
+// $Id: DisplacementRaise.cpp,v 1.20 2005/01/03 04:09:27 geuzaine Exp $
 //
-// Copyright (C) 1997-2004 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -51,7 +51,7 @@ GMSH_DisplacementRaisePlugin::GMSH_DisplacementRaisePlugin()
 
 void GMSH_DisplacementRaisePlugin::getName(char *name) const
 {
-  strcpy(name, "Displacement raise");
+  strcpy(name, "Displacement Raise");
 }
 
 void GMSH_DisplacementRaisePlugin::getInfos(char *author, char *copyright,
@@ -64,11 +64,10 @@ void GMSH_DisplacementRaisePlugin::getInfos(char *author, char *copyright,
          "coordinates of the elements in the view `iView'\n"
          "using the vectorial values (the displacements)\n"
          "stored in the `dTimeStep'-th time step of the\n"
-	 "view `dView'. If `iView' < 0, the plugin is\n"
-	 "run on the current view. If `dView' < 0, the\n"
-	 "plugin looks for the displacements in the\n"
-	 "view located just after `iView' in the view\n"
-	 "list.\n"
+	 "view `dView'. If `iView' < 0, the plugin is run\n"
+	 "on the current view. If `dView' < 0, the plugin\n"
+	 "looks for the displacements in the view located\n"
+	 "just after `iView' in the view list.\n"
 	 "\n"
 	 "Plugin(DisplacementRaise) is executed in-place.\n");
 }
@@ -194,8 +193,8 @@ Post_View *GMSH_DisplacementRaisePlugin::execute(Post_View * v)
     return v;
   }
 
-  Post_View *v1 = (Post_View*)List_Pointer(CTX.post.list, iView);
-  Post_View *v2 = (Post_View*)List_Pointer(CTX.post.list, dView);
+  Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
+  Post_View *v2 = *(Post_View **)List_Pointer(CTX.post.list, dView);
 
   displacementRaise(v1, v2, factor, dTimeStep);
 
