@@ -1,4 +1,4 @@
-// $Id: Evaluate.cpp,v 1.19 2005/03/04 19:08:38 geuzaine Exp $
+// $Id: Evaluate.cpp,v 1.23 2005/10/28 17:25:59 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -93,7 +93,7 @@ void GMSH_EvaluatePlugin::getInfos(char *author, char *copyright,
 	 "\n"
 	 "- the symbol w, to retrieve the `Component'-th\n"
 	 "component of the field in `ExternalView' at the\n"
-	 "`ExternalTimeStep'-th time step. if `ExternalView'\n"
+	 "`ExternalTimeStep'-th time step. If `ExternalView'\n"
 	 "and `iView' are based on different spatial grids,\n"
 	 "or if their data types are different, `ExternalView'\n"
 	 "is interpolated onto `iView';\n"
@@ -102,7 +102,7 @@ void GMSH_EvaluatePlugin::getInfos(char *author, char *copyright,
 	 "component of the field in `ExternalView' at the\n"
 	 "`ExternalTimeStep'-th time step.\n"
 	 "\n"
-	 "If `TimeStep' < 0, the plugin atomatically loops\n"
+	 "If `TimeStep' < 0, the plugin automatically loops\n"
 	 "over all the time steps in `iView' and evaluates\n"
 	 "`Expression' for each one. If `ExternalTimeStep'\n"
 	 "< 0, the plugin uses `TimeStep' instead. If\n"
@@ -184,9 +184,10 @@ void GMSH_EvaluatePlugin::evaluate(Post_View *v1, List_T *list1, int nbElm1,
       for(int k = 0; k < nbComp; k++) v[k] = val1[k];
 
       // store data from the external view into w
-      double *val2, tmp[9];
       double w[9] = {0., 0., 0., 0., 0., 0., 0., 0., 0.};
+      double *val2;
       if(_octree){
+	double tmp[9];
 	val2 = tmp;
 	if(_octree->searchScalar(x[j], y[j], z[j], val2, timeStep2)){
 	  w[0] = val2[0];
@@ -240,7 +241,7 @@ Post_View *GMSH_EvaluatePlugin::execute(Post_View * v)
 
   Post_View *v1 = *(Post_View **)List_Pointer(CTX.post.list, iView);
 
-  if(timeStep > v->NbTimeStep - 1){
+  if(timeStep > v1->NbTimeStep - 1){
     Msg(GERROR, "Invalid time step (%d) in View[%d]: using step 0 instead",
 	timeStep, v1->Index);
     timeStep = 0;
