@@ -1,4 +1,4 @@
-// $Id: Levelset.cpp,v 1.24 2005/01/03 04:09:27 geuzaine Exp $
+// $Id: Levelset.cpp,v 1.27 2005/10/19 15:45:10 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -374,8 +374,10 @@ int GMSH_LevelsetPlugin::zeroLevelset(int timeStep,
     }
   }
 
-  // if we compute isovolumes, add the nodes on the chosen side
+  // if we compute isovolumes, add the nodes on the chosen side 
   if(_extractVolume){
+    // FIXME: when cutting 2D views, the elts we get can have the wrong
+    // orientation
     int nbCut = np;
     for(int k = 0; k < nbNod; k++){
       if((_extractVolume < 0. && levels[k] < 0.0) ||
@@ -762,7 +764,7 @@ Post_View *GMSH_LevelsetPlugin::execute(Post_View * v)
 static bool recur_sign_change (adapt_triangle *t, double val, const GMSH_LevelsetPlugin *plug)
 {
 
-  if (!t->t[0]|| t->visible)
+  if (!t->e[0]|| t->visible)
     {
       double v1 = plug->levelset (t->p[0]->X,t->p[0]->Y,t->p[0]->Z,t->p[0]->val);
       double v2 = plug->levelset (t->p[1]->X,t->p[1]->Y,t->p[1]->Z,t->p[1]->val);
@@ -775,16 +777,16 @@ static bool recur_sign_change (adapt_triangle *t, double val, const GMSH_Levelse
     }
   else
     {
-      bool sc1= recur_sign_change(t->t[0],val,plug);
-      bool sc2= recur_sign_change(t->t[1],val,plug);
-      bool sc3= recur_sign_change(t->t[2],val,plug);
-      bool sc4= recur_sign_change(t->t[3],val,plug);
+      bool sc1= recur_sign_change(t->e[0],val,plug);
+      bool sc2= recur_sign_change(t->e[1],val,plug);
+      bool sc3= recur_sign_change(t->e[2],val,plug);
+      bool sc4= recur_sign_change(t->e[3],val,plug);
       if (sc1 || sc2 || sc3 || sc4)
 	{
-	  if (!sc1) t->t[0]->visible = true;
-	  if (!sc2) t->t[1]->visible = true;
-	  if (!sc3) t->t[2]->visible = true;
-	  if (!sc4) t->t[3]->visible = true;
+	  if (!sc1) t->e[0]->visible = true;
+	  if (!sc2) t->e[1]->visible = true;
+	  if (!sc3) t->e[2]->visible = true;
+	  if (!sc4) t->e[3]->visible = true;
 	  return true;
 	}
       t->visible = false;
@@ -795,7 +797,7 @@ static bool recur_sign_change (adapt_triangle *t, double val, const GMSH_Levelse
 static bool recur_sign_change (adapt_tet *t, double val, const GMSH_LevelsetPlugin *plug)
 {
 
-  if (!t->t[0] || t->visible)
+  if (!t->e[0] || t->visible)
     {
       double v1 = plug->levelset (t->p[0]->X,t->p[0]->Y,t->p[0]->Z,t->p[0]->val);
       double v2 = plug->levelset (t->p[1]->X,t->p[1]->Y,t->p[1]->Z,t->p[1]->val);
@@ -809,24 +811,24 @@ static bool recur_sign_change (adapt_tet *t, double val, const GMSH_LevelsetPlug
     }
   else
     {
-      bool sc1 = recur_sign_change(t->t[0],val,plug);
-      bool sc2 = recur_sign_change(t->t[1],val,plug);
-      bool sc3 = recur_sign_change(t->t[2],val,plug);
-      bool sc4 = recur_sign_change(t->t[3],val,plug);
-      bool sc5 = recur_sign_change(t->t[4],val,plug);
-      bool sc6 = recur_sign_change(t->t[5],val,plug);
-      bool sc7 = recur_sign_change(t->t[6],val,plug);
-      bool sc8 = recur_sign_change(t->t[7],val,plug);
+      bool sc1 = recur_sign_change(t->e[0],val,plug);
+      bool sc2 = recur_sign_change(t->e[1],val,plug);
+      bool sc3 = recur_sign_change(t->e[2],val,plug);
+      bool sc4 = recur_sign_change(t->e[3],val,plug);
+      bool sc5 = recur_sign_change(t->e[4],val,plug);
+      bool sc6 = recur_sign_change(t->e[5],val,plug);
+      bool sc7 = recur_sign_change(t->e[6],val,plug);
+      bool sc8 = recur_sign_change(t->e[7],val,plug);
       if (sc1 || sc2 || sc3 || sc4 || sc5 || sc6 || sc7 || sc8)
 	{
-	  if (!sc1) t->t[0]->visible = true;
-	  if (!sc2) t->t[1]->visible = true;
-	  if (!sc3) t->t[2]->visible = true;
-	  if (!sc4) t->t[3]->visible = true;
-	  if (!sc5) t->t[4]->visible = true;
-	  if (!sc6) t->t[5]->visible = true;
-	  if (!sc7) t->t[6]->visible = true;
-	  if (!sc8) t->t[7]->visible = true;
+	  if (!sc1) t->e[0]->visible = true;
+	  if (!sc2) t->e[1]->visible = true;
+	  if (!sc3) t->e[2]->visible = true;
+	  if (!sc4) t->e[3]->visible = true;
+	  if (!sc5) t->e[4]->visible = true;
+	  if (!sc6) t->e[5]->visible = true;
+	  if (!sc7) t->e[6]->visible = true;
+	  if (!sc8) t->e[7]->visible = true;
 	  return true;
 	}
       t->visible = false;
@@ -837,7 +839,7 @@ static bool recur_sign_change (adapt_tet *t, double val, const GMSH_LevelsetPlug
 static bool recur_sign_change (adapt_hex *t, double val, const GMSH_LevelsetPlugin *plug)
 {
 
-  if (!t->h[0]|| t->visible)
+  if (!t->e[0]|| t->visible)
     {
       double v1 = plug->levelset (t->p[0]->X,t->p[0]->Y,t->p[0]->Z,t->p[0]->val);
       double v2 = plug->levelset (t->p[1]->X,t->p[1]->Y,t->p[1]->Z,t->p[1]->val);
@@ -855,24 +857,24 @@ static bool recur_sign_change (adapt_hex *t, double val, const GMSH_LevelsetPlug
     }
   else
     {
-      bool sc1 = recur_sign_change(t->h[0],val,plug);
-      bool sc2 = recur_sign_change(t->h[1],val,plug);
-      bool sc3 = recur_sign_change(t->h[2],val,plug);
-      bool sc4 = recur_sign_change(t->h[3],val,plug);
-      bool sc5 = recur_sign_change(t->h[4],val,plug);
-      bool sc6 = recur_sign_change(t->h[5],val,plug);
-      bool sc7 = recur_sign_change(t->h[6],val,plug);
-      bool sc8 = recur_sign_change(t->h[7],val,plug);
+      bool sc1 = recur_sign_change(t->e[0],val,plug);
+      bool sc2 = recur_sign_change(t->e[1],val,plug);
+      bool sc3 = recur_sign_change(t->e[2],val,plug);
+      bool sc4 = recur_sign_change(t->e[3],val,plug);
+      bool sc5 = recur_sign_change(t->e[4],val,plug);
+      bool sc6 = recur_sign_change(t->e[5],val,plug);
+      bool sc7 = recur_sign_change(t->e[6],val,plug);
+      bool sc8 = recur_sign_change(t->e[7],val,plug);
       if (sc1 || sc2 || sc3 || sc4 || sc5 || sc6 || sc7 || sc8)
 	{
-	  if (!sc1) t->h[0]->visible = true;
-	  if (!sc2) t->h[1]->visible = true;
-	  if (!sc3) t->h[2]->visible = true;
-	  if (!sc4) t->h[3]->visible = true;
-	  if (!sc5) t->h[4]->visible = true;
-	  if (!sc6) t->h[5]->visible = true;
-	  if (!sc7) t->h[6]->visible = true;
-	  if (!sc8) t->h[7]->visible = true;
+	  if (!sc1) t->e[0]->visible = true;
+	  if (!sc2) t->e[1]->visible = true;
+	  if (!sc3) t->e[2]->visible = true;
+	  if (!sc4) t->e[3]->visible = true;
+	  if (!sc5) t->e[4]->visible = true;
+	  if (!sc6) t->e[5]->visible = true;
+	  if (!sc7) t->e[6]->visible = true;
+	  if (!sc8) t->e[7]->visible = true;
 	  return true;
 	}
       t->visible = false;
@@ -883,7 +885,7 @@ static bool recur_sign_change (adapt_hex *t, double val, const GMSH_LevelsetPlug
 static bool recur_sign_change (adapt_quad *q, double val, const GMSH_LevelsetPlugin *plug)
 {
 
-  if (!q->q[0]|| q->visible)
+  if (!q->e[0]|| q->visible)
     {
       double v1 = plug->levelset (q->p[0]->X,q->p[0]->Y,q->p[0]->Z,q->p[0]->val);
       double v2 = plug->levelset (q->p[1]->X,q->p[1]->Y,q->p[1]->Z,q->p[1]->val);
@@ -897,16 +899,16 @@ static bool recur_sign_change (adapt_quad *q, double val, const GMSH_LevelsetPlu
     }
   else
     {
-      bool sc1 = recur_sign_change(q->q[0],val,plug);
-      bool sc2 = recur_sign_change(q->q[1],val,plug);
-      bool sc3 = recur_sign_change(q->q[2],val,plug);
-      bool sc4 = recur_sign_change(q->q[3],val,plug);
+      bool sc1 = recur_sign_change(q->e[0],val,plug);
+      bool sc2 = recur_sign_change(q->e[1],val,plug);
+      bool sc3 = recur_sign_change(q->e[2],val,plug);
+      bool sc4 = recur_sign_change(q->e[3],val,plug);
       if (sc1 || sc2 || sc3 || sc4 )
 	{
-	  if (!sc1) q->q[0]->visible = true;
-	  if (!sc2) q->q[1]->visible = true;
-	  if (!sc3) q->q[2]->visible = true;
-	  if (!sc4) q->q[3]->visible = true;
+	  if (!sc1) q->e[0]->visible = true;
+	  if (!sc2) q->e[1]->visible = true;
+	  if (!sc3) q->e[2]->visible = true;
+	  if (!sc4) q->e[3]->visible = true;
 	  return true;
 	}
       q->visible = false;
@@ -916,24 +918,24 @@ static bool recur_sign_change (adapt_quad *q, double val, const GMSH_LevelsetPlu
 
 void GMSH_LevelsetPlugin::assign_specific_visibility () const
 {
-  if (adapt_triangle::all_triangles.size())
+  if (adapt_triangle::all_elems.size())
     {
-      adapt_triangle *t  = *adapt_triangle::all_triangles.begin();
-      t->visible = !recur_sign_change (t, _valueView, this);
+      adapt_triangle *t  = *adapt_triangle::all_elems.begin();
+      if (!t->visible)t->visible = !recur_sign_change (t, _valueView, this);
     }
-  if (adapt_tet::all_tets.size())
+  if (adapt_tet::all_elems.size())
     {
-      adapt_tet *te  = *adapt_tet::all_tets.begin();
-      te->visible = !recur_sign_change (te, _valueView, this);
+      adapt_tet *te  = *adapt_tet::all_elems.begin();
+      if (!te->visible)te->visible = !recur_sign_change (te, _valueView, this);
     }
-  if (adapt_quad::all_quads.size())
+  if (adapt_quad::all_elems.size())
     {
-      adapt_quad *qe  = *adapt_quad::all_quads.begin();
-      qe->visible = !recur_sign_change (qe, _valueView, this);
+      adapt_quad *qe  = *adapt_quad::all_elems.begin();
+      if (!qe->visible)if (!qe->visible)qe->visible = !recur_sign_change (qe, _valueView, this);
     }
-  if (adapt_hex::all_hexes.size())
+  if (adapt_hex::all_elems.size())
     {
-      adapt_hex *he  = *adapt_hex::all_hexes.begin();
-      he->visible = !recur_sign_change (he, _valueView, this);
+      adapt_hex *he  = *adapt_hex::all_elems.begin();
+      if (!he->visible)he->visible = !recur_sign_change (he, _valueView, this);
     }
 }
