@@ -1,4 +1,4 @@
-// $Id: Plugin.cpp,v 1.73 2005/01/12 19:10:41 geuzaine Exp $
+// $Id: Plugin.cpp,v 1.78 2005/07/11 16:22:35 geuzaine Exp $
 //
 // Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
 //
@@ -41,6 +41,7 @@
 #include "Skin.h"
 #include "Extract.h"
 #include "HarmonicToTime.h"
+#include "ModulusPhase.h"
 #include "Integrate.h"
 #include "Gradient.h"
 #include "Curl.h"
@@ -165,8 +166,6 @@ GMSH_PluginManager *GMSH_PluginManager::instance()
 
 void GMSH_PluginManager::registerDefaultPlugins()
 {
-  char *homeplugins = getenv("GMSHPLUGINSHOME");
-
   // SOLVE PLUGINS
   if(CTX.solver.plugins){
     allPlugins.insert(std::pair < char *, GMSH_Plugin * >
@@ -201,6 +200,8 @@ void GMSH_PluginManager::registerDefaultPlugins()
 		      ("DisplacementRaise", GMSH_RegisterDisplacementRaisePlugin()));
     allPlugins.insert(std::pair < char *, GMSH_Plugin * >
 		      ("HarmonicToTime", GMSH_RegisterHarmonicToTimePlugin()));
+    allPlugins.insert(std::pair < char *, GMSH_Plugin * >
+		      ("ModulusPhase", GMSH_RegisterModulusPhasePlugin()));
     allPlugins.insert(std::pair < char *, GMSH_Plugin * >
 		      ("Integrate", GMSH_RegisterIntegratePlugin()));
     allPlugins.insert(std::pair < char *, GMSH_Plugin * >
@@ -234,16 +235,17 @@ void GMSH_PluginManager::registerDefaultPlugins()
   }
 
 #if defined(HAVE_FLTK)
-  struct dirent **list;
-  char ext[6];
+  char *homeplugins = getenv("GMSHPLUGINSHOME");
   if(!homeplugins)
     return;
+  struct dirent **list;
   int nbFiles = fl_filename_list(homeplugins, &list);
   if(nbFiles <= 0)
     return;
   for(int i = 0; i < nbFiles; i++) {
     char *name = list[i]->d_name;
     if(strlen(name) > 3) {
+      char ext[6];
       strcpy(ext, name + (strlen(name) - 3));
       if(!strcmp(ext, ".so") || !strcmp(ext, "dll")) {
         addPlugin(homeplugins, name);
@@ -292,9 +294,6 @@ void GMSH_PluginManager::addPlugin(char *dirName, char *pluginName)
     return;
   }
   allPlugins.insert(std::pair < char *, GMSH_Plugin * >(plugin_name, p));
-  Msg(INFO, "Plugin name      : %s", plugin_name);
-  Msg(INFO, "Plugin author    : %s", plugin_author);
-  Msg(INFO, "Plugin copyright : %s", plugin_copyright);
-  Msg(INFO, "Plugin help      : %s", plugin_help);
+  Msg(INFO, "Loaded Plugin '%s' (%s)", plugin_name, plugin_author);
 #endif
 }
